package com.pokkt.thirdparty.AdColony;

import android.text.TextUtils;

import com.adcolony.sdk.AdColony;
import com.adcolony.sdk.AdColonyInterstitial;
import com.adcolony.sdk.AdColonyInterstitialListener;
import com.adcolony.sdk.AdColonyZone;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.debugging.Logger;

import org.json.JSONObject;

import java.util.HashMap;
import java.util.Map;

public class AdColonyInterstitialManager {

    private static final String TAG = AdColonyInterstitialManager.class.getName();
    private AdNetworkInfo adNetworkInfo = null;

    private Map<String, AdColonyInterstitial> adMap = new HashMap<String, AdColonyInterstitial>();//map of ad unit to ad
    private AdConfig adColonyAdConfig = null;
    private Callbacks.WithSuccessAndFailure<Double, String> callback;
    private Map<AdConfig, AdListener> listenerMap = new HashMap<AdConfig, AdListener>();

    public AdColonyInterstitialManager(AdNetworkInfo network) {
        this.adNetworkInfo = network;
    }

    private String getZoneId(AdConfig info) {
        String zoneId = "";
        if (adNetworkInfo.getCustomData().containsKey("screens_mapping_data")) {
            try {
                JSONObject mappingObject = new JSONObject(adNetworkInfo.getCustomData().get("screens_mapping_data"));
                String mapData = mappingObject.optString(info.screenName);
                if (!TextUtils.isEmpty(mapData)) {
                    JSONObject mapObj = new JSONObject(mapData);
                    int isNonRewardedInterstitial = mapObj.optInt("non_incent");
                    if (isNonRewardedInterstitial == 1) {
                        zoneId = mapObj.optString("network_screen", "").trim();
                    } else {
                        Logger.e(TAG+ " Failed!.Requesting rewarded interstitial from adcolony.");
                        return zoneId;
                    }
                }
            } catch (Exception e) {
                Logger.printStackTrace(TAG+ " Invalid Mapping Json", e);
            }
        }
        if (zoneId.length() == 0) {
            zoneId = info.screenName;
        }
        return zoneId;
    }

    public void fetchAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {

        Logger.i(TAG+ " cache ad called !");

        try {
            if (config.isRewarded) {
                Logger.i(TAG+ " Rewarded interstitial is requested !");
                callbacks.onFailure("Invalid Ad Config.");
                return;
            }
            String zoneId = getZoneId(config);
            if (TextUtils.isEmpty(zoneId)) {
                callbacks.onFailure("Fetch Ad Failed. Invalid AdConfig");
                return;
            }
            if (this.adColonyAdConfig != null && !(config.equals(this.adColonyAdConfig))) {
                callbacks.onFailure("Fetch Ad Failed. Wrong AdConfig.");
                return;
            }
            if (this.adColonyAdConfig == null) {
                this.adColonyAdConfig = config.clone();
            }
            AdColonyInterstitial ad = adMap.get(zoneId);
            if (ad != null && !ad.isExpired()) {
                callbacks.onSuccess(0.0);
                return;
            }
            this.callback = callbacks;
            AdListener listener = new AdListener(config);
            AdColony.requestInterstitial(zoneId, listener);
            listenerMap.put(config, listener);
        } catch (Throwable e) {
            this.adColonyAdConfig = null;
            listenerMap.remove(config);
            Logger.printStackTrace(TAG+ " Fetch Ad Failed", e);
            callbacks.onFailure("Fetch Ad Failed");
        }
    }

    public void showAd(final AdConfig config, final Callbacks.WithOnlyFailure<String> failureCallback) {

        Logger.i(TAG+ " play Ad called !");
        try {
            String zoneId = getZoneId(config);
            if (TextUtils.isEmpty(zoneId)) {
                Logger.i(TAG+ " Play Ad Failed. Invalid Zone.");
                requestFailed("Play Ad Failed. Invalid Zone.", failureCallback);
                return;
            }
            if (this.adColonyAdConfig != null && !(config.equals(this.adColonyAdConfig))) {
                Logger.i(TAG+ " Play Ad Failed. Wrong AdConfig");
                requestFailed("Play Ad Failed. Wrong AdConfig", failureCallback);
                return;
            }
            if (this.adColonyAdConfig == null) {
                this.adColonyAdConfig = config.clone();
            }
            AdColonyInterstitial ad = adMap.get(zoneId);
            if (ad != null && !ad.isExpired()) {
                Logger.i(TAG+ " play ad called as v4vc ad is available and ready !");
                ad.show();
            } else {
                Logger.i(TAG+ " No Ad is currently available!");
                requestFailed("No Ad is currently available!!", failureCallback);
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG+ " Show Ad Failed", e);
            requestFailed("Show Ad Failed", failureCallback);
        }
    }

    public boolean checkAdAvailable(AdConfig config) {

        try {
            String zoneId = getZoneId(config);
            if (!TextUtils.isEmpty(zoneId)) {
                if (this.adColonyAdConfig != null && !(config.equals(this.adColonyAdConfig))) {
                    Logger.i(TAG+ " Fetch Ad Failed. Wrong AdConfig.");
                    return false;
                }
                AdColonyInterstitial ad = adMap.get(zoneId);
                if (ad != null && !ad.isExpired()) {
                    Logger.i(TAG+ " ad is available and ready !!");
                    return true;
                }
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG+ " Failed to find availability", e);
        }
        return false;
    }

    public void requestFailed(String message, final Callbacks.WithOnlyFailure<String> failureCallback) {
        failureCallback.onFailure(message);
        this.adColonyAdConfig = null;
    }

    public void notifyTimeOut(AdConfig adConfig) {
        if (listenerMap.get(adConfig) != null) {
            listenerMap.get(adConfig).notifyTimeout();
        }
    }

    private class AdListener extends AdColonyInterstitialListener {
        private AdConfig adConfig;
        private boolean isTimedOut = false;

        AdListener(AdConfig adConfig) {
            this.adConfig = adConfig;
        }

        void notifyTimeout() {
            isTimedOut = true;
            String reason = " Time Out In Fetching Ad";
            Logger.e(TAG+ reason);
            listenerMap.remove(adConfig);
            adColonyAdConfig = null;
        }

        @Override
        public void onClosed(AdColonyInterstitial ad) {
            Logger.i(TAG+ " onClosed");
            adColonyAdConfig = null;
            AdManager.getInstance().adCompleted(adConfig, adNetworkInfo);
            AdManager.getInstance().adClosed(adConfig, adNetworkInfo);
        }

        public void onRequestNotFilled(AdColonyZone zone) {
            Logger.i(TAG+ " onRequestNotFilled called!!");
            if (!isTimedOut) {
                isTimedOut = true;
                listenerMap.remove(adConfig);
                callback.onFailure("Ad Not Available !");
                adColonyAdConfig = null;
            }
        }

        @Override
        public void onOpened(AdColonyInterstitial ad) {
            Logger.i(TAG+ " onAdColonyAdStarted called!!");
            AdManager.getInstance().adDisplayed(adConfig, adNetworkInfo);
        }

        @Override
        public void onRequestFilled(AdColonyInterstitial adColonyInterstitial) {
            if (!isTimedOut) {
                isTimedOut = true;
                callback.onSuccess(0.0);
                listenerMap.remove(adConfig);
            }
            String zoneId = getZoneId(adConfig);
            adMap.put(zoneId, adColonyInterstitial);

        }

        @Override
        public void onExpiring(AdColonyInterstitial ad) {
            Logger.i(TAG+ " onExpiring");
//            AdColony.requestInterstitial(ZONE_ID, this, ad_options );
        }
    }
}
