package com.pokkt.thirdparty.AdColony;

import android.text.TextUtils;

import com.adcolony.sdk.AdColony;
import com.adcolony.sdk.AdColonyAdOptions;
import com.adcolony.sdk.AdColonyInterstitial;
import com.adcolony.sdk.AdColonyInterstitialListener;
import com.adcolony.sdk.AdColonyReward;
import com.adcolony.sdk.AdColonyRewardListener;
import com.adcolony.sdk.AdColonyZone;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.debugging.Logger;

import org.json.JSONObject;

import java.util.HashMap;
import java.util.Map;

public class AdColonyVideoAdManager {

    private static final String TAG = AdColonyVideoAdManager.class.getName();
    private AdNetworkInfo adNetworkInfo = null;

    private AdConfig lastCompletedAdConfig = null;
    private AdConfig adColonyAdConfig = null;
    private Map<String, AdColonyInterstitial> adMap = new HashMap<>();//map of ad unit to ad
    private Callbacks.WithSuccessAndFailure<Double, String> callback;
    private Map<AdConfig, AdListener> listenerMap = new HashMap<>();

    public AdColonyVideoAdManager(AdNetworkInfo network) {
        this.adNetworkInfo = network;
    }

    private String getZoneId(AdConfig info) {
        String zoneId = "";
        if (adNetworkInfo.getCustomData().containsKey("screens_mapping_data")) {
            try {
                JSONObject mappingObject = new JSONObject(adNetworkInfo.getCustomData().get("screens_mapping_data"));
                String mapData = mappingObject.optString(info.screenName);
                if (!TextUtils.isEmpty(mapData)) {
                    JSONObject mapObj = new JSONObject(mapData);
                    int isRewardedVideo = mapObj.optInt("incent");
                    if (isRewardedVideo == 1) {
                        zoneId = mapObj.optString("network_screen", "").trim();
                    } else {
                        Logger.i(TAG + " Failed.Requesting Non rewarded video from adcolony.");
                        return zoneId;
                    }
                }
            } catch (Exception e) {
                Logger.printStackTrace(TAG + " Invalid Mapping Json", e);
            }
        }
        if (zoneId.length() == 0) {
            zoneId = info.screenName;
        }
        return zoneId;
    }

    public void fetchAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {

        Logger.i(TAG + " Cache ad called !");

        try {
            String zoneId = getZoneId(config);
            if (TextUtils.isEmpty(zoneId)) {
                callbacks.onFailure("Fetch Ad Failed. Invalid AdConfig");
                return;
            }
            if (this.adColonyAdConfig != null && !(config.equals(this.adColonyAdConfig))) {
                callbacks.onFailure("Fetch Ad Failed. Wrong AdConfig.");
                return;
            }
            if (this.adColonyAdConfig == null) {
                this.adColonyAdConfig = config.clone();
            }
            AdColonyInterstitial ad = adMap.get(zoneId);
            if (ad != null && !ad.isExpired()) {
                callbacks.onSuccess(0.0);
                return;
            }
            this.callback = callbacks;
            AdListener listener = new AdListener(config);
            AdColonyAdOptions ad_options = new AdColonyAdOptions()
                    .enableConfirmationDialog(false)
                    .enableResultsDialog(false);
            AdColony.requestInterstitial(zoneId, listener, ad_options);
            listenerMap.put(config, listener);
        } catch (Throwable e) {
            this.adColonyAdConfig = null;
            Logger.printStackTrace(TAG + " Fetch Ad Failed", e);
            callbacks.onFailure("Fetch Ad Failed");
            listenerMap.remove(config);
        }
    }

    public void showAd(final AdConfig config, final Callbacks.WithOnlyFailure<String> failureCallback) {

        Logger.i(TAG + " play Ad called !");
        try {
            String zoneId = getZoneId(config);
            if (TextUtils.isEmpty(zoneId)) {
                Logger.i(TAG + " Play Ad Failed. Invalid Zone.");
                requestFailed("Play Ad Failed. Invalid Zone.", failureCallback);
                return;
            }
            if (this.adColonyAdConfig != null && !(config.equals(this.adColonyAdConfig))) {
                Logger.i(TAG + " Play Ad Failed. Wrong AdConfig");
                requestFailed("Play Ad Failed. Wrong AdConfig", failureCallback);
                return;
            }
            if (this.adColonyAdConfig == null) {
                this.adColonyAdConfig = config.clone();
            }
            AdColonyInterstitial ad = adMap.get(zoneId);
            if (ad != null && !ad.isExpired()) {
                Logger.i(TAG + " play ad called as v4vc ad is available and ready !");
                lastCompletedAdConfig = config;
                AdColony.setRewardListener(new AdListener(config));
                ad.show();
            } else {
                Logger.i(TAG + " No Ad is currently available!");
                requestFailed("No Ad is currently available!", failureCallback);
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Show Ad Failed", e);
            requestFailed("Show Ad Failed", failureCallback);
        }
    }

    private void requestFailed(String message, final Callbacks.WithOnlyFailure<String> failureCallback) {
        failureCallback.onFailure(message);
        this.adColonyAdConfig = null;
    }

    public boolean checkAdAvailable(AdConfig config) {

        try {
            String zoneId = getZoneId(config);
            if (!TextUtils.isEmpty(zoneId)) {
                if (this.adColonyAdConfig != null && !(config.equals(this.adColonyAdConfig))) {
                    Logger.i(TAG + " Fetch Ad Failed. Wrong AdConfig.");
                    return false;
                }
                AdColonyInterstitial ad = adMap.get(zoneId);
                if (ad != null && !ad.isExpired()) {
                    Logger.i(TAG + " ad is available and ready !");
                    return true;
                }
            } else {
                Logger.i(TAG + " Invalid zoneid");
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        return false;
    }

    public void notifyTimeOut(AdConfig adConfig) {
        if (listenerMap.get(adConfig) != null) {
            listenerMap.get(adConfig).notifyTimeout();
        }
    }

    private class AdListener extends AdColonyInterstitialListener implements AdColonyRewardListener {
        private AdConfig adConfig;
        private boolean isTimedOut = false;

        AdListener(AdConfig adConfig) {
            this.adConfig = adConfig;
        }

        void notifyTimeout() {
            isTimedOut = true;
            String reason = " Time Out In Fetching Ad";
            Logger.e(TAG + reason);
            listenerMap.remove(adConfig);
            adColonyAdConfig = null;
        }

        @Override
        public void onClosed(AdColonyInterstitial ad) {
            Logger.i(TAG + " onClosed");
            adColonyAdConfig = null;
            AdManager.getInstance().adCompleted(adConfig, adNetworkInfo);
            AdManager.getInstance().adClosed(adConfig, adNetworkInfo);
        }

        public void onRequestNotFilled(AdColonyZone zone) {
            Logger.i(TAG + " onRequestNotFilled called!!");
            if (!isTimedOut) {
                isTimedOut = true;
                listenerMap.remove(adConfig);
                adColonyAdConfig = null;
                callback.onFailure("Ad Not Available !");
            }
        }

        @Override
        public void onOpened(AdColonyInterstitial ad) {
            Logger.i(TAG + " onAdColonyAdStarted called!!");
            AdManager.getInstance().adDisplayed(adConfig, adNetworkInfo);
        }

        @Override
        public void onRequestFilled(AdColonyInterstitial adColonyInterstitial) {
            if (!isTimedOut) {
                isTimedOut = true;
                callback.onSuccess(0.0);
                listenerMap.remove(adConfig);
            }
            String zoneId = getZoneId(adConfig);
            adMap.put(zoneId, adColonyInterstitial);
        }

        @Override
        public void onExpiring(AdColonyInterstitial ad) {
            Logger.i(TAG + " onExpiring");
//            AdColony.requestInterstitial(ZONE_ID, this, ad_options );
        }

        @Override
        public void onReward(AdColonyReward reward) {
            try {
                if (reward.success()) {
                    Logger.i(TAG + " onAdColonyV4VCReward is success. Reward amount: " + reward.getRewardAmount());
                    if (lastCompletedAdConfig != null && adConfig.isRewarded) {
                        double value = reward.getRewardAmount();
                        AdManager.getInstance().adGratified(adConfig, value, adNetworkInfo);
                    }
                }
                AdColony.removeRewardListener();
            } catch (Throwable e) {
                Logger.printStackTrace(TAG + " Gratify failed", e);
            }
        }
    }
}
