package com.pokkt.thirdparty.AerServ;

import android.app.Activity;
import android.content.Context;
import android.text.TextUtils;

import com.aerserv.sdk.AerServBanner;
import com.aerserv.sdk.AerServConfig;
import com.aerserv.sdk.AerServEvent;
import com.aerserv.sdk.AerServEventListener;
import com.pokkt.PokktAds;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.banners.PokktBannerView;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;

import org.json.JSONObject;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class AerServBannerManager {

    private static final String TAG = AerServBannerManager.class.getName();

    private AdNetworkInfo adNetworkInfo;
    private Callbacks.WithSuccessAndFailure<AdCampaign, String> callback;
    private Map<AdConfig, AerServBanner> adMap = new HashMap<>();
    private Map<AdConfig, BannerListener> listenerMap = new HashMap<>();

    public AerServBannerManager(AdNetworkInfo info) {
        this.adNetworkInfo = info;
    }

    public void loadBanner(final AdConfig config, final BannerUnit bannerUnit, final Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks, final Context context) {
        Logger.d(TAG + " load banner called !");

        final String placementId = getPlacementId(config);
        if (TextUtils.isEmpty(placementId)) {
            callbacks.onFailure("Invalid Ad Unit");
            return;
        }

        this.callback = callbacks;

        AerServBanner banner = adMap.get(config);
        if (banner != null) {
            callbacks.onSuccess(null);
            return;
        }
        ((Activity) context).runOnUiThread(new Runnable() {
            @Override
            public void run() {
                try {
                    AerServBanner aerServBanner = new AerServBanner(context);
//                    AerServConfig aerServConfig = new AerServConfig(context, "1000834");
//                    AerServConfig config = new AerServConfig(context,"1000834");
                    AerServConfig aerServConfig = new AerServConfig(context, placementId);
                    aerServConfig.setRefreshInterval(0);//Never refresh
                    aerServConfig.setDebug(PokktAds.Debugging.isEnabled());
                    BannerListener listener = new BannerListener(config, bannerUnit, aerServBanner, context);
                    aerServConfig.setEventListener(listener);
                    aerServBanner.configure(aerServConfig).show();
                    listenerMap.put(config, listener);
                } catch (Throwable e) {
                    Logger.printStackTrace(TAG + " Load banner failed", e);
                    listenerMap.remove(config);
                    callbacks.onFailure(" Ad Caching Failed");
                }
            }
        });
    }

    private String getPlacementId(AdConfig config) {
        String adUnitId = "";
        if (adNetworkInfo.getCustomData().containsKey("screens_mapping_data")) {
            try {
                JSONObject mappingObject = new JSONObject(adNetworkInfo.getCustomData().get("screens_mapping_data"));
                String mapData = mappingObject.optString(config.screenName);
                if (!TextUtils.isEmpty(mapData)) {
                    JSONObject mapObj = new JSONObject(mapData);
                    int isBanner = mapObj.optInt("non_incent");
                    if (isBanner == 1) {
                        adUnitId = mapObj.optString("network_screen", "").trim();
                    } else {
                        Logger.d(TAG + " Failed.Requesting rewarded banner.");
                        return adUnitId;
                    }
                }
            } catch (Exception e) {
                Logger.printStackTrace(TAG + " Invalid Mapping Json", e);
            }
        }
        if (adUnitId.length() == 0) {
            adUnitId = config.screenName;
        }
        return adUnitId;
    }

    public void destroyBanner(AdConfig adConfig) {
        AerServBanner unit = adMap.get(adConfig);
        if (unit != null) {
            unit.pause();
            unit.kill();
            adMap.remove(adConfig);
            listenerMap.remove(adConfig);
        }
    }

    public void notifyCachingTimeout(AdConfig config) {
        String reason = " Time Out In Fetching Ad";
        Logger.e(TAG + reason);
        listenerMap.get(config).notifyTimeout();
    }

    private class BannerListener implements AerServEventListener {

        private final AdConfig adConfig;
        private final BannerUnit bannerUnit;
        private final AerServBanner aerServBanner;
        private final Context context;
        private boolean isTimedOut;

        BannerListener(AdConfig config, BannerUnit unit, AerServBanner banner, Context ctx) {
            this.adConfig = config;
            this.bannerUnit = unit;
            this.aerServBanner = banner;
            this.context = ctx;
        }

        @Override
        public void onAerServEvent(AerServEvent event, List<Object> list) {
            switch (event) {
                case AD_LOADED:
                    Logger.d(TAG + " Ad Loaded !");
                    if (!isTimedOut) {
                        if (!(adMap.containsKey(adConfig))) {
                            adMap.put(adConfig, aerServBanner);
                        }
                        isTimedOut = true;
                        ((Activity) context).runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                List<PokktBannerView> containerList = bannerUnit.getContainerList();
                                if (containerList.size() > 0) {
                                    PokktBannerView container = containerList.get(0);
                                    if (container.getChildAt(0) == null) {
                                        container.addView(aerServBanner);
                                    } else {
                                        container.removeView(container.getChildAt(0));
                                        container.addView(aerServBanner);
                                    }
                                    callback.onSuccess(null);
                                } else {
                                    destroyBanner(adConfig);
                                }
                            }
                        });
                    }
                    break;
                case AD_FAILED:
                    Logger.d(TAG + " onError !! " + list.get(0));
                    if (!isTimedOut) {
                        isTimedOut = true;
                        callback.onFailure("Ad Load Failed ! " + list.get(0));
                        ((Activity) context).runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                destroyBanner(adConfig);
                            }
                        });
                    }
                    break;
            }
        }

        void notifyTimeout() {
            isTimedOut = false;
        }
    }
}
