package com.pokkt.thirdparty.AerServ;

import android.content.Context;
import android.text.TextUtils;

import com.aerserv.sdk.AerServConfig;
import com.aerserv.sdk.AerServEvent;
import com.aerserv.sdk.AerServEventListener;
import com.aerserv.sdk.AerServInterstitial;
import com.aerserv.sdk.AerServVirtualCurrency;
import com.pokkt.PokktAds;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.debugging.Logger;

import org.json.JSONObject;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class AerServInterstitialManager {

    private static final String TAG = AerServInterstitialManager.class.getName();
    private double amount = 0;

    private AdNetworkInfo adNetworkInfo;
    private Map<String, AerServInterstitial> adMap = new HashMap<>();//map of ad unit to ad
    private AdConfig aerServAdConfig = null;
    private Callbacks.WithSuccessAndFailure<Double, String> callback;
    private boolean isTimedOut = false;

    public AerServInterstitialManager(AdNetworkInfo info, double amount) {
        this.adNetworkInfo = info;
        this.amount = amount;
    }

    public void fetchAd(final AdConfig config, final Callbacks.WithSuccessAndFailure<Double, String> callbacks, Context context) {
        Logger.d(TAG + " cache Ad called !");

        AdListener listener = new AdListener(config);

        try {
            String placementId = getPlacementId(config);
            if (TextUtils.isEmpty(placementId)) {
                callbacks.onFailure("Ad Caching Failed");
                return;
            }
            if (this.aerServAdConfig != null && !(config.equals(this.aerServAdConfig))) {
                callbacks.onFailure("Fetch Ad Failed. Wrong AdConfig.");
                return;
            }
            if (this.aerServAdConfig == null) {
                this.aerServAdConfig = config.clone();
            }
            AerServInterstitial ad = adMap.get(getPlacementId(config));
            if (ad != null) {
                callbacks.onSuccess(0.0);
                return;
            }

            this.callback = callbacks;
//            AerServConfig aerServConfig = new AerServConfig(context, "1000741");//TODO remove this
//            AerServConfig aerServConfig = new AerServConfig(context, "1021823");
            AerServConfig aerServConfig = new AerServConfig(context, placementId);
            aerServConfig.setPreload(true);
            aerServConfig.setEventListener(listener);
            aerServConfig.setDebug(PokktAds.Debugging.isEnabled());
            AerServInterstitial interstitial = new AerServInterstitial(aerServConfig);
            listener.setPlacementId(placementId);
            listener.setInterstitialAd(interstitial);
            isTimedOut = false;
        } catch (Throwable e) {
            this.aerServAdConfig = null;
            Logger.printStackTrace(TAG + " Cache Ad failed", e);
            isTimedOut = true;
            callbacks.onFailure("Ad Caching Failed");
        }
    }

    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback, Context context) {

        Logger.d(TAG + " show Interstitial called !");
        String adUnitId = "";
        try {
            adUnitId = getPlacementId(config);
            if (TextUtils.isEmpty(adUnitId)) {
                adMap.remove(adUnitId);
                requestFailed("Ad Not Available !", failureCallback);
                return;
            }
            if (this.aerServAdConfig != null && !(config.equals(this.aerServAdConfig))) {
                failureCallback.onFailure("Fetch Ad Failed. Wrong AdConfig.");
                return;
            }
            if (this.aerServAdConfig == null) {
                this.aerServAdConfig = config.clone();
            }
            AerServInterstitial ad = adMap.get(getPlacementId(config));
            if (ad != null) {
                ad.show();
            } else {
                Logger.d(TAG + " No Ad is currently available!!");
                adMap.remove(adUnitId);
                requestFailed("Ad Not Available !", failureCallback);
            }
        } catch (Throwable e) {
            adMap.remove(adUnitId);
            requestFailed("Ad Not Available !", failureCallback);
        }
    }

    private void requestFailed(String msg, Callbacks.WithOnlyFailure<String> failureCallback) {
        this.aerServAdConfig = null;
        failureCallback.onFailure(msg);
    }

    public boolean isInterstitialAvailable(AdConfig config) {
        try {
            String zoneId = getPlacementId(config);
            if (!TextUtils.isEmpty(zoneId)) {
                if (this.aerServAdConfig != null && !(config.equals(this.aerServAdConfig))) {
                    Logger.d(TAG + " Fetch Ad Failed. Wrong AdConfig.");
                    return false;
                }
            }
            AerServInterstitial ad = adMap.get(getPlacementId(config));
            if (ad != null) {
                Logger.d(TAG + " ad is available and ready !");
                return true;
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        return false;
    }

    private String getPlacementId(AdConfig config) {
        String placementId = "";
        if (adNetworkInfo.getCustomData().containsKey("screens_mapping_data")) {
            try {
                JSONObject mappingObject = new JSONObject(adNetworkInfo.getCustomData().get("screens_mapping_data"));
                String mapData = mappingObject.optString(config.screenName);
                if (!TextUtils.isEmpty(mapData)) {
                    JSONObject mapObj = new JSONObject(mapData);
                    int isRewarded = mapObj.optInt("incent");
                    if (config.isRewarded) {
                        if (isRewarded == 1) {
                            placementId = mapObj.optString("network_screen", "").trim();
                        } else {
                            Logger.d(TAG + " Failed. Dashboard setting and adconfig request not matching.");
                            return placementId;
                        }
                    } else {
                        if (isRewarded == 0) {
                            placementId = mapObj.optString("network_screen", "").trim();
                        } else {
                            Logger.d(TAG + " Failed. Dashboard setting and adconfig request not matching.");
                            return placementId;
                        }
                    }
                }
            } catch (Exception e) {
                Logger.printStackTrace(TAG + " Invalid Mapping Json", e);
            }
        }
        if (placementId.length() == 0) {
            placementId = config.screenName;
        }
        return placementId;
    }

    public void notifyCachingTimeout() {
        isTimedOut = true;
        String reason = " Time Out In Fetching Ad";
        aerServAdConfig = null;
        Logger.e(TAG + reason);
    }

    private class AdListener implements AerServEventListener {
        private AdConfig adConfig;
        private String placementId = "";
        private AerServInterstitial interstitialAd = null;

        void setPlacementId(String id) {
            this.placementId = id;
        }

        void setInterstitialAd(AerServInterstitial interstitialAd) {
            this.interstitialAd = interstitialAd;
        }

        AdListener(AdConfig config) {
            this.adConfig = config;
        }

        @Override
        public void onAerServEvent(AerServEvent event, List params) {
            switch (event) {
                case VC_REWARDED:
                    AerServVirtualCurrency vc = (AerServVirtualCurrency) params.get(0);
                    amount = vc.getAmount().longValue();
                    break;
                case PRELOAD_READY:
                    Logger.d(TAG + " onAdLoadSucceeded !");
                    adMap.put(placementId, interstitialAd);
                    if (!isTimedOut) {
                        isTimedOut = true;
                        callback.onSuccess(0.0);
                    }
                    break;
                case AD_LOADED:
                    Logger.d(TAG + " onAdOpened !");
                    AdManager.getInstance().adDisplayed(adConfig, adNetworkInfo);
                    break;
                case AD_COMPLETED:
                    Logger.d(TAG + " onAdCompleted ! ");
                    AdManager.getInstance().adCompleted(adConfig, adNetworkInfo);
                    break;
                case AD_DISMISSED:
                    Logger.d(TAG + " onAdClosed !");
                    adMap.remove(placementId);
                    AdManager.getInstance().adClosed(adConfig, adNetworkInfo);
                    if (adConfig.isRewarded) {
                        AdManager.getInstance().adGratified(adConfig, amount, adNetworkInfo);
                    }
                    aerServAdConfig = null;
                    break;
                case AD_FAILED:
                    Logger.d(TAG + " onError ! " + params.get(0));
                    if (!isTimedOut) {
                        isTimedOut = true;
                        callback.onFailure("Ad Load Failed ! " + params.get(0));
                        aerServAdConfig = null;
                    }
                    break;
            }
        }
    }
}