package com.pokkt.thirdparty;

import android.app.Activity;
import android.content.Context;
import android.support.annotation.Keep;

import com.aerserv.sdk.AerServConfig;
import com.pokkt.PokktAds;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdFormat;
import com.pokkt.sdk.adnetworks.AdNetwork;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;
import com.pokkt.thirdparty.AerServ.AerServBannerManager;
import com.pokkt.thirdparty.AerServ.AerServInterstitialManager;

@Keep
public class AerServNetwork implements AdNetwork {

    private static final String TAG = AerServNetwork.class.getName();

    private Context context = null;
    private AdNetworkInfo adNetworkInfo;
    private boolean isInitialized = false;
    private AerServInterstitialManager aerServInterstitialManager;
    private AerServBannerManager aerServBannerManager;
    private static final String APP_ID = "appId";
    private double amount = 0;

    @Override
    public void initNetwork(final Context context, final AdNetworkInfo info) {
        Logger.d(TAG + " init network called !");

        synchronized (AerServNetwork.class) {
            if (isInitialized) {
                Logger.d(TAG + " Network already initialized !");
                return;
            }
            if (!(context instanceof Activity)) {
                Logger.d(TAG + " Activity context needed !");
                return;
            }
            this.context = context;

            if (!sdkEnabled()) {
                Logger.d(TAG + " Init Configurations Error !");
                return;
            }

            if (extrasAreValid(info)) {
                this.adNetworkInfo = info;
                Logger.d(TAG + " init network initialized !");

                if (info.getCustomData().containsKey("amount")) {
                    try {
                        amount = Float.parseFloat(info.getCustomData().get("amount"));
                    } catch (Exception e) {
                        Logger.printStackTrace(TAG + " Could not parse amount", e);
                        //do nothing
                    }
                }
                AerServConfig.setTestMode(PokktAds.Debugging.isEnabled());
                aerServInterstitialManager = new AerServInterstitialManager(info, amount);
                aerServBannerManager = new AerServBannerManager(info);
                isInitialized = true;
            } else {
                Logger.d(TAG + " Init Configurations Error or SDK Error !");
            }
        }
    }

    @Override
    public AdNetworkInfo getAdNetworkInfo() {
        return adNetworkInfo;
    }

    private boolean extrasAreValid(AdNetworkInfo info) {
        return info != null && info.getCustomData() != null && info.getCustomData().containsKey(APP_ID);
    }

    @Override
    public void cacheAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
        if (AdFormat.INTERSTITIAL == config.adFormat) {
            aerServInterstitialManager.fetchAd(config, callbacks, context);
        } else {
            callbacks.onFailure("Invalid Ad Format");
        }
    }

    @Override
    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {
        if (AdFormat.INTERSTITIAL == config.adFormat) {
            aerServInterstitialManager.showAd(config, failureCallback, context);
        } else {
            failureCallback.onFailure("Invalid Ad Format");
        }
    }

    @Override
    public void adClosed(AdConfig adConfig) {

    }

    @Override
    public void checkAdAvailability(AdConfig config, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        try {
            if (AdFormat.INTERSTITIAL == config.adFormat) {
                if (aerServInterstitialManager.isInterstitialAvailable(config)) {
                    callbacks.onSuccess(null);
                    return;
                }
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        callbacks.onFailure("Ad Not Available");
    }

    @Override
    public void loadBannerAd(AdConfig adConfig, BannerUnit bannerUnit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        aerServBannerManager.loadBanner(adConfig, bannerUnit, callbacks, context);
    }

    @Override
    public void destroyBannerAd(AdConfig adConfig) {
        aerServBannerManager.destroyBanner(adConfig);
    }

    @Override
    public void notifyCachingTimeout(AdConfig adConfig) {
        if (AdFormat.INTERSTITIAL == adConfig.adFormat) {
            aerServInterstitialManager.notifyCachingTimeout();
        } else if (AdFormat.BANNER == adConfig.adFormat) {
            aerServBannerManager.notifyCachingTimeout(adConfig);
        }
    }

    @Override
    public double getAdVC(AdConfig adConfig) {
        if (adConfig.isRewarded) {
            return amount;
        } else {
            return 0.0;
        }
    }

    @Override
    public boolean supportsAdConfig(AdConfig config) {
        if (AdFormat.BRANDING == config.adFormat) {
            return false;
        }

        if (AdFormat.VIDEO == config.adFormat) {
            return false;
        }
        return true;
    }

    @Override
    public boolean isInitialised() {
        return isInitialized;
    }

    @Override
    public boolean isPokktNetwork() {
        return false;
    }

    @Override
    public boolean verifyCampaignForAdConfig(AdConfig adConfig, boolean verifyCached) {
        return true;
    }

    private boolean sdkEnabled() {
        try {
            Class.forName("com.aerserv.sdk.AerServSdk");
            return true;
        } catch (Throwable ignore) {
            Logger.e(TAG + " SDK not found");
            return false;
        }
    }
}