package com.pokkt.thirdparty.AppLovin;

import android.content.Context;

import com.applovin.adview.AppLovinInterstitialAd;
import com.applovin.adview.AppLovinInterstitialAdDialog;
import com.applovin.sdk.AppLovinAd;
import com.applovin.sdk.AppLovinAdDisplayListener;
import com.applovin.sdk.AppLovinSdk;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.debugging.Logger;

public class AppLovinInterstitialManager {

    private static final String TAG = AppLovinInterstitialManager.class.getName();
    private AdNetworkInfo adNetworkInfo = null;

    private AppLovinInterstitialAdDialog adDialog;
    private AdConfig appLovinAdConfig;

    public AppLovinInterstitialManager(AdNetworkInfo network) {
        this.adNetworkInfo = network;
    }

    public void fetchAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks, Context context) {
        Logger.d(TAG + " Cache Ad called !");

        try {
            if (config.isRewarded) {
                Logger.d(TAG + " Rewarded interstitial is not supported !");
                callbacks.onFailure("Invalid Ad Config.");
                return;
            }
            if (adDialog != null && appLovinAdConfig != null && !appLovinAdConfig.equals(config)) {
                callbacks.onFailure("Ad Caching Failed !");
                return;
            }
            if (adDialog != null && appLovinAdConfig != null && appLovinAdConfig.equals(config) && AppLovinInterstitialAd.isAdReadyToDisplay(context)) {
                Logger.d(TAG + " Interstitial is cached since last call !");
                callbacks.onSuccess(0.0);
                return;
            }
            if (appLovinAdConfig == null) {
                this.appLovinAdConfig = config.clone();
            }
            if (AppLovinInterstitialAd.isAdReadyToDisplay(context)) {
                callbacks.onSuccess(0.0);
            } else {
                callbacks.onFailure("Ad Caching Failed");
            }
        } catch (Throwable e) {
            appLovinAdConfig = null;
            Logger.printStackTrace(TAG + " Ad Caching Failed", e);
            callbacks.onFailure("Ad Caching Failed");
        }
    }

    public void showAd(final AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback, Context context) {
        Logger.d(TAG + " Play Ad called !");

        try {
            if (appLovinAdConfig != null && appLovinAdConfig.equals(config) || appLovinAdConfig == null) {
                if (appLovinAdConfig == null) {
                    this.appLovinAdConfig = config.clone();
                }
                if (AppLovinInterstitialAd.isAdReadyToDisplay(context)) {
                    // An ad is available to display.  It's safe to call show.
                    adDialog = AppLovinInterstitialAd.create(AppLovinSdk.getInstance(context), context);
                    adDialog.setAdDisplayListener(new AppLovinAdDisplayListener() {
                        @Override
                        public void adDisplayed(AppLovinAd appLovinAd) {
                            Logger.d(TAG + " interstitial Displayed !");
                            AdManager.getInstance().adDisplayed(config, adNetworkInfo);
                        }

                        @Override
                        public void adHidden(AppLovinAd appLovinAd) {
                            Logger.d(TAG + " interstitial Closed !");
                            AdManager.getInstance().adCompleted(config, adNetworkInfo);
                            AdManager.getInstance().adClosed(config, adNetworkInfo);
                            adDialog.dismiss();
                            adDialog = null;
                            appLovinAdConfig = null;
                        }
                    });
//                        adDialog.setAdLoadListener(this);
                    adDialog.show();
                } else {
                    requestFailed("Ad Not Available !", failureCallback);
                }
            } else {
                requestFailed("Ad Not Available !", failureCallback);
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Ad Not Available", e);
            requestFailed("Ad Not Available !", failureCallback);
        }
    }

    private void requestFailed(String message, Callbacks.WithOnlyFailure<String> failureCallback) {
        adDialog = null;
        appLovinAdConfig = null;
        failureCallback.onFailure(message);
    }

    public boolean checkAdAvailable(AdConfig config, Context context) {
        try {
            if (appLovinAdConfig != null && appLovinAdConfig.equals(config) || appLovinAdConfig == null) {
                if (AppLovinInterstitialAd.isAdReadyToDisplay(context)) {
                    return true;
                }
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        return false;
    }
}