package com.pokkt.thirdparty.AppLovin;

import android.app.Activity;
import android.content.Context;

import com.applovin.adview.AppLovinIncentivizedInterstitial;
import com.applovin.sdk.AppLovinAd;
import com.applovin.sdk.AppLovinAdClickListener;
import com.applovin.sdk.AppLovinAdDisplayListener;
import com.applovin.sdk.AppLovinAdLoadListener;
import com.applovin.sdk.AppLovinAdRewardListener;
import com.applovin.sdk.AppLovinAdVideoPlaybackListener;
import com.applovin.sdk.AppLovinErrorCodes;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.debugging.Logger;

import java.util.Map;

public class AppLovinVideoAdManager {

    private static final String TAG = AppLovinVideoAdManager.class.getName();
    private AdNetworkInfo adNetworkInfo = null;
    private AppLovinIncentivizedInterstitial incentivizedInterstitial;
    private double amount = 0;
    private boolean isTimedOut = false;
    private AdConfig appLovinAdConfig;//config for video
    private Callbacks.WithSuccessAndFailure<Double, String> callback;


    public AppLovinVideoAdManager(AdNetworkInfo network, double amount) {
        this.adNetworkInfo = network;
        this.amount = amount;
    }

    public void fetchAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks, Context context) {
        Logger.d(TAG + " Cache Ad called.");
        try {
            AppLovinListener listener = new AppLovinListener(config);
            if (incentivizedInterstitial != null && appLovinAdConfig != null && !appLovinAdConfig.equals(config)) {
                callbacks.onFailure("Cache Ad Failed !");
                return;
            }
            if (incentivizedInterstitial != null && appLovinAdConfig != null && appLovinAdConfig.equals(config) && incentivizedInterstitial.isAdReadyToDisplay()) {
                Logger.d(TAG + " Video is cached since last call !");
                callbacks.onSuccess(0.0);
                return;
            }

            if (appLovinAdConfig == null) {
                appLovinAdConfig = config.clone();
            }
            this.callback = callbacks;
            if (incentivizedInterstitial == null) {
                Logger.d(TAG + " Creating incentivized interstitial !");
                incentivizedInterstitial = AppLovinIncentivizedInterstitial.create(context);
            }
            incentivizedInterstitial.preload(listener);
            isTimedOut = false;
        } catch (Throwable e) {
            incentivizedInterstitial = null;
            appLovinAdConfig = null;
            isTimedOut = true;
            Logger.printStackTrace(TAG + " Ad Caching Failed", e);
            callbacks.onFailure("Ad Caching Failed");
        }
    }

    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback, Context context) {
        Logger.d(TAG + " Play Ad called.");

        try {
            if (appLovinAdConfig != null && appLovinAdConfig.equals(config) || appLovinAdConfig == null) {
                if (appLovinAdConfig == null) {
                    this.appLovinAdConfig = config.clone();
                }
                if (incentivizedInterstitial != null && incentivizedInterstitial.isAdReadyToDisplay()) {
                    AppLovinListener listener = new AppLovinListener(config);
                    incentivizedInterstitial.show((Activity) context, listener, listener, listener, listener);
                } else {
                    Logger.d(TAG + " No rewarded ad is currently available !");
                    requestFailed("Ad Not Available !", failureCallback);
                }
            } else {
                Logger.d(TAG + " Wrong Info Passed After caching !");
                requestFailed("Ad Not Available !", failureCallback);
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Ad Not Available", e);
            requestFailed("Ad Not Available !", failureCallback);
        }
    }

    private void requestFailed(String message, Callbacks.WithOnlyFailure<String> failureCallback) {
        failureCallback.onFailure(message);
        incentivizedInterstitial = null;
        this.appLovinAdConfig = null;
    }

    public boolean checkAdAvailable(AdConfig config) {
        try {
            if (appLovinAdConfig != null && appLovinAdConfig.equals(config) || appLovinAdConfig == null) {
                if (incentivizedInterstitial != null && incentivizedInterstitial.isAdReadyToDisplay()) {
                    return true;
                }
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        return false;
    }

    public void notifyCachingTimeout() {
        incentivizedInterstitial = null;
        appLovinAdConfig = null;
        isTimedOut = true;
        String reason = " Time Out In Fetching Video";
        Logger.e(TAG + reason);
    }

    private class AppLovinListener implements AppLovinAdRewardListener, AppLovinAdVideoPlaybackListener,
            AppLovinAdDisplayListener, AppLovinAdLoadListener, AppLovinAdClickListener {

        private final AdConfig adConfig;

        AppLovinListener(AdConfig config) {
            this.adConfig = config;
        }

        @Override
        public void adClicked(AppLovinAd appLovinAd) {
            Logger.d(TAG + " adClicked !");
        }

        @Override
        public void adDisplayed(AppLovinAd appLovinAd) {
            Logger.d(TAG + " adDisplayed !");
        }

        @Override
        public void adHidden(AppLovinAd appLovinAd) {
            Logger.d(TAG + " adHidden !");
            AdManager.getInstance().adClosed(adConfig, adNetworkInfo);
            isTimedOut = false;
        }

        @Override
        public void adReceived(AppLovinAd appLovinAd) {
            Logger.d(TAG + " adReceived !");
            if (!isTimedOut) {
                isTimedOut = true;
                callback.onSuccess(0.0);
            }
        }

        @Override
        public void failedToReceiveAd(int i) {
            Logger.d(TAG + " failedToReceiveAd !" + i);
            String reason;
            switch (i) {
                case AppLovinErrorCodes.NO_FILL:
                    reason = "No Campaigns from Applovin.";
                    break;
                case AppLovinErrorCodes.FETCH_AD_TIMEOUT:
                    reason = "Network conditions prevented the SDK from receiving an ad.";
                    break;
                case AppLovinErrorCodes.INCENTIVIZED_NO_AD_PRELOADED:
                    reason = "Requested for a rewarded video before one was available.";
                    break;
                case AppLovinErrorCodes.INCENTIVIZED_SERVER_TIMEOUT:
                    reason = "Reward validation requested timed out.";
                    break;
                case AppLovinErrorCodes.INCENTIVIZED_UNKNOWN_SERVER_ERROR:
                    reason = "Unknown server-side error.";
                    break;
                case AppLovinErrorCodes.INCENTIVIZED_USER_CLOSED_VIDEO:
                    reason = "User exited out of the video.";
                    //may be dont gratify code
                    break;
                case AppLovinErrorCodes.UNABLE_TO_PRECACHE_IMAGE_RESOURCES:
                case AppLovinErrorCodes.UNABLE_TO_PRECACHE_RESOURCES:
                case AppLovinErrorCodes.UNABLE_TO_PRECACHE_VIDEO_RESOURCES:
                    reason = "Attempt to cache a resource to the filesystem failed.";
                    break;
                case AppLovinErrorCodes.UNABLE_TO_RENDER_AD:
                    reason = "Failure to render an ad on screen.";
                    break;
                default:
                    reason = "Unspecified Error !";
                    break;
            }
            Logger.d(TAG + " Could not show ad from AppLovin for reason: " + reason);
            if (!isTimedOut) {
                isTimedOut = true;
                callback.onFailure(reason);
            }
            incentivizedInterstitial = null;
        }

        @Override
        public void userRewardVerified(AppLovinAd appLovinAd, Map map) {
            if (adConfig.isRewarded) {
                Logger.d(TAG + " userRewardVerified !");
                try {
                    if (map.containsKey("amount")) {
                        amount = Double.parseDouble((String) map.get("amount"));
                    }
                } catch (Exception e) {
                    Logger.printStackTrace(TAG + " Failed to parse amount", e);
                }
                AdManager.getInstance().adGratified(adConfig, amount, adNetworkInfo);
            }
        }

        @Override
        public void userOverQuota(AppLovinAd appLovinAd, Map map) {
            Logger.d(TAG + " userOverQuota !");
        }

        @Override
        public void userRewardRejected(AppLovinAd appLovinAd, Map map) {
            Logger.d(TAG + " userRewardRejected !");
        }

        @Override
        public void validationRequestFailed(AppLovinAd appLovinAd, int i) {
            Logger.d(TAG + " validationRequestFailed !");
        }

        @Override
        public void userDeclinedToViewAd(AppLovinAd appLovinAd) {
            Logger.d(TAG + " userDeclinedToViewAd !");
            AdManager.getInstance().adClosed(adConfig, adNetworkInfo);
            isTimedOut = false;
        }

        @Override
        public void videoPlaybackBegan(AppLovinAd appLovinAd) {
            Logger.d(TAG + " videoPlaybackBegan !");
            AdManager.getInstance().adDisplayed(adConfig, adNetworkInfo);
        }

        @Override
        public void videoPlaybackEnded(AppLovinAd appLovinAd, double percentViewed, boolean fullyWatched) {
            Logger.d(TAG + " videoPlaybackEnded!! " + "PercentageViewed :" + percentViewed + " fullywatched: " + fullyWatched);
            if (!fullyWatched) {
                AdManager.getInstance().adSkipped(adConfig, adNetworkInfo);
            } else {
                AdManager.getInstance().adCompleted(adConfig, adNetworkInfo);
            }
            isTimedOut = false;
        }
    }
}