package com.pokkt.thirdparty;

import android.app.Activity;
import android.content.Context;
import android.support.annotation.Keep;

import com.applovin.sdk.AppLovinSdk;
import com.applovin.sdk.AppLovinSdkSettings;
import com.pokkt.PokktAds;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdFormat;
import com.pokkt.sdk.adnetworks.AdNetwork;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;
import com.pokkt.thirdparty.AppLovin.AppLovinInterstitialManager;
import com.pokkt.thirdparty.AppLovin.AppLovinVideoAdManager;

@Keep
public class AppLovinNetwork implements AdNetwork {

    private static final String TAG = AppLovinNetwork.class.getName();
    private Context context = null;
    private boolean isInitialized = false;
    private double amount = 0;
    private AdNetworkInfo adNetworkInfo = null;

    private AppLovinVideoAdManager videoAdManager;
    private AppLovinInterstitialManager interstitialManager;

    @Override
    public void initNetwork(final Context context, final AdNetworkInfo networkInfo) {
        Logger.d(TAG + " Init network called!!");

        synchronized (AppLovinNetwork.class) {

            if (isInitialized) {
                Logger.d(TAG + " Network already initialized!!");
                return;
            }

            if (!(context instanceof Activity)) {
                Logger.d(TAG + " Init Configurations Error ! Needs Activity context");
                return;
            }

            if (!sdkEnabled()) {
                return;
            }
            AppLovinSdkSettings appLovinSdkSettings = new AppLovinSdkSettings();
            appLovinSdkSettings.setVerboseLogging(PokktAds.Debugging.isEnabled());

            this.context = context;
            this.adNetworkInfo = networkInfo;
            if (adNetworkInfo != null && adNetworkInfo.getCustomData() != null) {
                if (adNetworkInfo.getCustomData().containsKey("amount")) {
                    try {
                        amount = Float.parseFloat(adNetworkInfo.getCustomData().get("amount"));
                    } catch (Exception e) {
                        Logger.printStackTrace(TAG + " Could not parse amount", e);
                        //do nothing
                    }
                }
            } else {
                Logger.d(TAG + " Configurations Error !");
            }
            try {
                AppLovinSdk.initializeSdk(context);
                videoAdManager = new AppLovinVideoAdManager(adNetworkInfo, amount);
                interstitialManager = new AppLovinInterstitialManager(adNetworkInfo);
            } catch (Throwable e) {
                Logger.printStackTrace(TAG + " Init Configurations Error !", e);
                return;
            }
            Logger.d(TAG + " Network Initialized." + AppLovinSdk.VERSION);
            isInitialized = true;
        }
    }

    @Override
    public AdNetworkInfo getAdNetworkInfo() {
        return adNetworkInfo;
    }

    @Override
    public void cacheAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
        Logger.d(TAG + " Cache Ad called !");

        if (AdFormat.VIDEO == config.adFormat) {
            videoAdManager.fetchAd(config, callbacks, context);
        } else {
            interstitialManager.fetchAd(config, callbacks, context);
        }
    }

    private boolean sdkEnabled() {
        try {
            Class.forName("com.applovin.sdk.AppLovinSdk");
            Class.forName("com.applovin.adview.AppLovinIncentivizedInterstitial");
            Class.forName("com.applovin.sdk.AppLovinAd");
            return true;
        } catch (Throwable ignore) {
            Logger.e(TAG + " SDK not found");
            return false;
        }
    }

    @Override
    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {
        Logger.d(TAG + " Play Ad called !");

        if (AdFormat.VIDEO == config.adFormat) {
            videoAdManager.showAd(config, failureCallback, context);
        } else {
            interstitialManager.showAd(config, failureCallback, context);
        }
    }

    @Override
    public void adClosed(AdConfig adConfig) {

    }

    @Override
    public void checkAdAvailability(AdConfig config, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        try {
            if (AdFormat.VIDEO == config.adFormat) {
                if (videoAdManager.checkAdAvailable(config)) {
                    callbacks.onSuccess(null);
                }
            } else {
                if (interstitialManager.checkAdAvailable(config, context)) {
                    callbacks.onSuccess(null);
                }
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        callbacks.onFailure("No Ad Available.");
    }

    @Override
    public void loadBannerAd(AdConfig adConfig, BannerUnit bannerUnit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        callbacks.onFailure("Invalid Network !");
    }

    @Override
    public void destroyBannerAd(AdConfig adConfig) {

    }

    @Override
    public void notifyCachingTimeout(AdConfig config) {
        if (AdFormat.VIDEO == config.adFormat) {
            videoAdManager.notifyCachingTimeout();
        }
    }

    @Override
    public double getAdVC(AdConfig adConfig) {
        if (adConfig.isRewarded) {
            return amount;
        } else {
            return 0.0;
        }
    }

    @Override
    public boolean supportsAdConfig(AdConfig config) {
        if (AdFormat.BRANDING == config.adFormat) {
            return false;
        }

        if (AdFormat.INTERSTITIAL == config.adFormat) {
            if (config.isRewarded) {
                return false;
            }
        }
        if (AdFormat.VIDEO == config.adFormat) {
            if (!(config.isRewarded)) {
                return false;
            }
        }
        if (AdFormat.BANNER == config.adFormat) {
            return false;
        }
        return true;
    }

    @Override
    public boolean isInitialised() {
        return isInitialized;
    }

    @Override
    public boolean isPokktNetwork() {
        return false;
    }

    @Override
    public boolean verifyCampaignForAdConfig(AdConfig adConfig, boolean verifyCached) {
        return true;
    }
}