package com.pokkt.thirdparty;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.os.Build;
import android.os.Bundle;
import android.support.annotation.Keep;
import android.support.annotation.RequiresApi;

import com.chartboost.sdk.CBImpressionActivity;
import com.chartboost.sdk.Chartboost;
import com.chartboost.sdk.ChartboostDelegate;
import com.chartboost.sdk.Libraries.CBLogging;
import com.chartboost.sdk.Model.CBError;
import com.pokkt.PokktAds;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdFormat;
import com.pokkt.sdk.adnetworks.AdNetwork;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;

import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

@RequiresApi(api = Build.VERSION_CODES.ICE_CREAM_SANDWICH)
@Keep
public class ChartboostNetwork implements AdNetwork, Application.ActivityLifecycleCallbacks {

    private static final String TAG = ChartboostNetwork.class.getName();
    private String activityClassName = null;

    private static final String APP_ID_KEY = "appId";
    private static final String VC_KEY = "vc";
    private static final String APP_SIGNATURE = "appSignature";

    private String appId = "";
    private String appSignature = "";

    private boolean isInitialized = false;
    private static boolean isChartboostPlayed = false;
    private Map<AdConfig, Callbacks.WithSuccessAndFailure<Double, String>> callackMap = new HashMap<>();
    private Map<String, AdConfig> rewardedAdConfigMap = new HashMap<>();//location to adconfig mapping
    private Map<String, AdConfig> nonRewardedAdConfigMap = new HashMap<>();//location to adconfig mapping
    private AdNetworkInfo adNetworkInfo;
    private double rewardValue = 0;

    @Override
    public void initNetwork(Context context, AdNetworkInfo info) {

        this.adNetworkInfo = info;

        synchronized (ChartboostNetwork.class) {

            if (isInitialized) {
                Logger.d(TAG + " Network already initialized !");
                return;
            }

            if (!(context instanceof Activity)) {
                Logger.d(TAG + " Init Configurations Error!. Needs Activity context");
                return;
            }

            if (!sdkEnabled()) {
                Logger.d(TAG + " Init Configurations Error!");
                return;
            }


            if (PokktAds.Debugging.isEnabled()) {
                Chartboost.setLoggingLevel(CBLogging.Level.ALL);
            } else {
                Chartboost.setLoggingLevel(CBLogging.Level.NONE);
            }

            this.activityClassName = ((Activity) context).getClass().getName();
            ((Activity) context).getApplication().registerActivityLifecycleCallbacks(this);

            if (extrasAreValid(adNetworkInfo)) {
                appId = adNetworkInfo.getCustomData().get(APP_ID_KEY);
                appSignature = adNetworkInfo.getCustomData().get(APP_SIGNATURE);
                if (adNetworkInfo.getCustomData().containsKey(VC_KEY)) {
                    try {
                        rewardValue = Double.parseDouble(adNetworkInfo.getCustomData().get(VC_KEY));
                    } catch (Exception e) {
                        Logger.printStackTrace(TAG + " Parsing Failed", e);
                        //do nothing
                    }
                }
            } else {
                Logger.d(TAG + " Init Configurations Error!");
                return;
            }

            //Default key. Comment it before live
            //appId = "4f7b433509b6025804000002";
            //appSignature = "dd2d41b69ac01b80f443f5b6cf06096d457f82bd";

            try {
                chartboostInit(AdManager.getInstance().getAssignedActivity());
            } catch (Throwable e) {
                Logger.printStackTrace(TAG + " SDK Failed or Init configurations error", e);
                return;
            }
            isInitialized = true;
        }
    }

    private boolean extrasAreValid(AdNetworkInfo info) {
        return info != null
                && info.getCustomData() != null
                && info.getCustomData().containsKey(APP_ID_KEY)
                && info.getCustomData().containsKey(APP_SIGNATURE);
    }

    private void chartboostInit(Activity activity) {

        Chartboost.startWithAppId(activity, appId, appSignature);

        Chartboost.setDelegate(delegate);
        Chartboost.setAutoCacheAds(false);
        Chartboost.onCreate(activity);
        //Chartboost.onStart(((Activity) context));
    }

    @Override
    public void cacheAd(AdConfig adConfig, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
        try {
            //if not already caching for this adconfig. then can cache
            if (AdFormat.VIDEO == adConfig.adFormat) {
                if (!adConfig.isRewarded) {
                    Logger.d(TAG + " Non rewarded video is requested !");
                    callbacks.onFailure("Invalid Ad Config.");
                    return;
                }
                rewardedAdConfigMap.put(adConfig.screenName, adConfig);
                if (Chartboost.hasRewardedVideo(adConfig.screenName)) {
                    callbacks.onSuccess(0.0);
                } else {
                    Chartboost.cacheRewardedVideo(adConfig.screenName);
                    callackMap.put(adConfig, callbacks);
                }
            } else {
                if (adConfig.isRewarded) {
                    Logger.d(TAG + " Rewarded interstitial is requested !");
                    callbacks.onFailure("Invalid Ad Config.");
                    return;
                }
                nonRewardedAdConfigMap.put(adConfig.screenName, adConfig);
                if (Chartboost.hasInterstitial(adConfig.screenName)) {
                    callbacks.onSuccess(0.0);
                } else {
                    Chartboost.cacheInterstitial(adConfig.screenName);
                    callackMap.put(adConfig, callbacks);
                }
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Fetch Ad Failed", e);
            if (AdFormat.VIDEO == adConfig.adFormat) {
                rewardedAdConfigMap.remove(adConfig.screenName);
            } else {
                nonRewardedAdConfigMap.remove(adConfig.screenName);
            }
            stopTimer(adConfig);
            callbacks.onFailure("Caching Failed ");
        }
    }

    private void stopTimer(AdConfig adConfig) {
        Callbacks.WithSuccessAndFailure<Double, String> callback = callackMap.get(adConfig);
        if (callback != null) {
            callackMap.remove(adConfig);
        }
    }

    @Override
    public void showAd(AdConfig adConfig, Callbacks.WithOnlyFailure<String> failureCallback) {
        Logger.d(TAG + " Show Ad Called ");
        try {
            if (AdFormat.VIDEO == adConfig.adFormat) {
                if (!adConfig.isRewarded) {
                    Logger.d(TAG + " Non rewarded video is requested !");
                    requestFailed("Invalid AdConfig !", failureCallback);
                    return;
                }
                if (Chartboost.hasRewardedVideo(adConfig.screenName)) {
                    if (!rewardedAdConfigMap.containsKey(adConfig.screenName)) {
                        rewardedAdConfigMap.put(adConfig.screenName, adConfig);
                    }
                    Chartboost.showRewardedVideo(adConfig.screenName);
                } else {
                    Logger.d(TAG + " No rewarded ad is currently available !");
                    requestFailed("Ad Not Available !", failureCallback);
                }
            } else {
                if (adConfig.isRewarded) {
                    Logger.d(TAG + " Rewarded interstitial is requested !");
                    requestFailed("Invalid AdConfig !", failureCallback);
                    return;
                }
                if (Chartboost.hasInterstitial(adConfig.screenName)) {
                    if (!nonRewardedAdConfigMap.containsKey(adConfig.screenName)) {
                        nonRewardedAdConfigMap.put(adConfig.screenName, adConfig);
                    }
                    Chartboost.showInterstitial(adConfig.screenName);
                } else {
                    Logger.d(TAG + " No interstitial is currently available !");
                    requestFailed("Ad Not Available !", failureCallback);
                }
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Play Ad Failed", e);
            if (AdFormat.VIDEO == adConfig.adFormat) {
                if (rewardedAdConfigMap.containsKey(adConfig.screenName)) {
                    rewardedAdConfigMap.remove(adConfig.screenName);
                }
            } else {
                if (nonRewardedAdConfigMap.containsKey(adConfig.screenName)) {
                    nonRewardedAdConfigMap.remove(adConfig.screenName);
                }
            }
            requestFailed("Ad Not Available !", failureCallback);
        }
    }

    @Override
    public void adClosed(AdConfig adConfig) {

    }

    @Override
    public void checkAdAvailability(AdConfig adConfig, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        try {
            if (AdFormat.VIDEO == adConfig.adFormat) {
                if (adConfig.isRewarded) {
                    if (Chartboost.hasRewardedVideo(adConfig.screenName)) {
                        if (!rewardedAdConfigMap.containsKey(adConfig.screenName)) {
                            rewardedAdConfigMap.put(adConfig.screenName, adConfig);
                        }
                        callbacks.onSuccess(null);
                        return;
                    }
                }
            } else {
                if (!adConfig.isRewarded) {
                    if (Chartboost.hasInterstitial(adConfig.screenName)) {
                        if (!nonRewardedAdConfigMap.containsKey(adConfig.screenName)) {
                            nonRewardedAdConfigMap.put(adConfig.screenName, adConfig);
                        }
                        callbacks.onSuccess(null);
                        return;
                    }
                }
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
            if (AdFormat.VIDEO == adConfig.adFormat) {
                if (rewardedAdConfigMap.containsKey(adConfig.screenName)) {
                    rewardedAdConfigMap.remove(adConfig.screenName);
                }
            } else {
                if (nonRewardedAdConfigMap.containsKey(adConfig.screenName)) {
                    nonRewardedAdConfigMap.remove(adConfig.screenName);
                }
            }
        }
        callbacks.onFailure("Ad Not Available");
    }

    @Override
    public void loadBannerAd(AdConfig adConfig, BannerUnit bannerUnit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        callbacks.onFailure("Invalid Network !");
    }

    @Override
    public AdNetworkInfo getAdNetworkInfo() {
        return adNetworkInfo;
    }

    @Override
    public void destroyBannerAd(AdConfig adConfig) {

    }

    @Override
    public void notifyCachingTimeout(AdConfig adConfig) {
        String reason = " Time Out In Fetching ";
        Logger.e(TAG + reason);
        callackMap.remove(adConfig);
    }

    @Override
    public double getAdVC(AdConfig adConfig) {
        if (adConfig.isRewarded) {
            return rewardValue;
        } else {
            return 0;
        }
    }

    @Override
    public boolean supportsAdConfig(AdConfig config) {
        if (AdFormat.BRANDING == config.adFormat) {
            return false;
        }

        if (AdFormat.INTERSTITIAL == config.adFormat) {
            if (config.isRewarded) {
                return false;
            }
        }
        if (AdFormat.VIDEO == config.adFormat) {
            if (!(config.isRewarded)) {
                return false;
            }
        }
        if (AdFormat.BANNER == config.adFormat) {
            return false;
        }
        return true;
    }

    @Override
    public boolean isInitialised() {
        return isInitialized;
    }

    @Override
    public boolean isPokktNetwork() {
        return false;
    }

    @Override
    public boolean verifyCampaignForAdConfig(AdConfig adConfig, boolean verifyCached) {
        return true;
    }

    private ChartboostDelegate delegate = new ChartboostDelegate() {

        //Called after a rewarded video has been loaded from the Chartboost API servers and cached locally.
        @Override
        public void didCacheRewardedVideo(String location) {
            Logger.d(TAG + String.format(" Chartboost: DID CACHE REWARDED VIDEO: '%s'", (location != null ? location : "null")));
            AdConfig config = rewardedAdConfigMap.get(location);
            if (config != null) {
                if (callackMap.get(config) != null) {
                    callackMap.get(config).onSuccess(0.0);
                    stopTimer(config);
                }
            }
        }

        //Called after a rewarded video has attempted to load from the ChartBoost API servers but failed.
        //Chartboost gives this callback both in caching case and show case.
        //Timer map will be only valid for caching case.
        @Override
        public void didFailToLoadRewardedVideo(String location, CBError.CBImpressionError error) {
            Logger.d(TAG + String.format(" Chartboost: DID FAIL TO LOAD REWARDED VIDEO: '%s', Error:  %s", (location != null ? location : "null"), error.name()));
            AdConfig config = rewardedAdConfigMap.get(location);
            if (callackMap.get(config) != null) {
                callackMap.get(config).onFailure(error.name());
                stopTimer(rewardedAdConfigMap.get(location));
            } else {
                if (config != null) {
                    AdManager.getInstance().adClosed(config, adNetworkInfo);
                }
            }
            rewardedAdConfigMap.remove(location);
        }

        //
        @Override
        public void didFailToLoadInPlay(String location, CBError.CBImpressionError error) {
            Logger.d(TAG + String.format(" Chartboost: DID FAIL TO LOAD REWARDED VIDEO: '%s', Error:  %s", (location != null ? location : "null"), error.name()));
            AdConfig config = rewardedAdConfigMap.get(location);
            if (config != null) {
                AdManager.getInstance().adClosed(config, adNetworkInfo);
            }
            if (callackMap.get(config) != null) {
                stopTimer(config);
            }
            rewardedAdConfigMap.remove(location);
        }

        //Called after a rewarded video has been closed.
        @Override
        public void didCloseRewardedVideo(String location) {
            Logger.d(TAG + String.format(" Chartboost: DID CLOSE REWARDED VIDEO '%s'", (location != null ? location : "null")));
            AdConfig config = rewardedAdConfigMap.get(location);
            if (config != null) {
                AdManager.getInstance().adClosed(config, adNetworkInfo);
            }
            isChartboostPlayed = false;
            rewardedAdConfigMap.remove(location);
        }

        //Called after a rewarded video has been viewed completely and user is eligible for reward.
        @Override
        public void didCompleteRewardedVideo(String location, int reward) {
            Logger.d(TAG + String.format(Locale.US, " Chartboost: DID COMPLETE REWARDED VIDEO '%s' FOR REWARD %d", (location != null ? location : "null"), reward));
            AdConfig config = rewardedAdConfigMap.get(location);
            if (config != null) {
                AdManager.getInstance().adCompleted(config, adNetworkInfo);
                if (config.isRewarded) {
                    AdManager.getInstance().adGratified(config, rewardValue, adNetworkInfo);
                }
            }
        }

        //Called after a rewarded video has been displayed on the screen.
        @Override
        public void didDisplayRewardedVideo(String location) {
            Logger.d(TAG + String.format(" Chartboost: DID DISPLAY REWARDED VIDEO '%s' FOR REWARD", location));
            AdConfig config = rewardedAdConfigMap.get(location);
            if (config != null) {
                AdManager.getInstance().adDisplayed(config, adNetworkInfo);
            }
            isChartboostPlayed = true;
        }

        //Interstitial Ad
        public void didCacheInterstitial(String location) {
            Logger.d(TAG + " didCacheInterstitial: Location- " + location);
            AdConfig config = nonRewardedAdConfigMap.get(location);
            if (config != null) {
                if (callackMap.get(config) != null) {
                    callackMap.get(config).onSuccess(0.0);
                    stopTimer(config);
                }
            }
        }

        public void didFailToLoadInterstitial(String location, CBError.CBImpressionError error) {
            Logger.d(TAG + " didFailToLoadInterstitial: Location- " + location);
            AdConfig config = nonRewardedAdConfigMap.get(location);
            if (callackMap.get(config) != null) {
                callackMap.get(config).onFailure(error.name());
                stopTimer(config);
            } else {
                if (config != null) {
                    AdManager.getInstance().adClosed(config, adNetworkInfo);
                }
            }
            nonRewardedAdConfigMap.remove(location);
        }

        public void didDismissInterstitial(String location) {
            Logger.d(TAG + " didDismissInterstitial: Location- " + location);
            AdConfig config = nonRewardedAdConfigMap.get(location);
            if (config != null) {
                AdManager.getInstance().adClosed(config, adNetworkInfo);
            }
            isChartboostPlayed = false;
            nonRewardedAdConfigMap.remove(location);
        }

        public void didCloseInterstitial(String location) {
            Logger.d(TAG + " didCloseInterstitial: Location- " + location);
            AdConfig config = nonRewardedAdConfigMap.get(location);
            if (config != null) {
                AdManager.getInstance().adCompleted(config, adNetworkInfo);
                AdManager.getInstance().adClosed(config, adNetworkInfo);
            }
            isChartboostPlayed = false;
            nonRewardedAdConfigMap.remove(location);
        }

        public void didDisplayInterstitial(String location) {
            Logger.d(TAG + " didDisplayInterstitial: Location- " + location);
            AdConfig config = nonRewardedAdConfigMap.get(location);
            if (config != null) {
                AdManager.getInstance().adDisplayed(config, adNetworkInfo);
            }
            isChartboostPlayed = true;
        }
    };

    @Override
    public void onActivityCreated(Activity activity, Bundle bundle) {
        Logger.d(TAG + " Created activity " + activity + " with launcher " + activityClassName);
        try {
            //if (activity.getClass().getName().equals(activityClassName)) {
            Logger.d(TAG + " Created ChartBoost");
            //  activityClassName = this.context.getClass().getName();
            if (!(activity instanceof CBImpressionActivity)) {
                chartboostInit(activity);
            }
            //}
        } catch (Throwable e) {
            Logger.printStackTrace(TAG, e);
        }
    }

    @Override
    public void onActivityStarted(Activity activity) {
        Logger.d(TAG + " Started activity " + activity + " with launcher " + activityClassName);
        try {
            // if (activity.getClass().getName().equals(activityClassName)) {
            Logger.d(TAG + " Started ChartBoost");
            if (!(activity instanceof CBImpressionActivity)) {
                Chartboost.onStart(activity);
            }
            //}
        } catch (Throwable e) {
            Logger.printStackTrace(TAG, e);
        }
    }

    @Override
    public void onActivityResumed(Activity activity) {
        Logger.d(TAG + " Resumed activity " + activity + " with launcher " + activityClassName);
        try {
            // if (activity.getClass().getName().equals(activityClassName)) {
            Logger.d(TAG + " Resumed ChartBoost");
            if (!(activity instanceof CBImpressionActivity)) {
                Chartboost.onResume(activity);
            }
            //}
        } catch (Throwable e) {
            Logger.printStackTrace(TAG, e);
        }
    }

    @Override
    public void onActivityPaused(Activity activity) {
        Logger.d(TAG + " Paused activity " + activity + " with launcher " + activityClassName);
        try {
            //if (activity.getClass().getName().equals(activityClassName)) {
            Logger.d(TAG + " Paused ChartBoost");
            if (!(activity instanceof CBImpressionActivity)) {
                Chartboost.onPause(activity);
            }
            //}
        } catch (Throwable e) {
            Logger.printStackTrace(TAG, e);
        }
    }

    @Override
    public void onActivityStopped(Activity activity) {
        Logger.d(TAG + " Stopped activity " + activity + " with launcher " + activityClassName);
        try {
            // if (activity.getClass().getName().equals(activityClassName)) {
            Logger.d(TAG + " Stopped ChartBoost");
            if (!(activity instanceof CBImpressionActivity)) {
                Chartboost.onStop(activity);
            }
            //}
        } catch (Throwable e) {
            Logger.printStackTrace(TAG, e);
        }
    }

    @Override
    public void onActivitySaveInstanceState(Activity activity, Bundle bundle) {
    }

    @Override
    public void onActivityDestroyed(Activity activity) {
        Logger.d(TAG + " Destroyed activity " + activity + " with launcher " + activityClassName);
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Logger.d(TAG + " Destroyed ChartBoost");
                Chartboost.onDestroy(activity);
//                if (!rewardedAdConfigMap.isEmpty()) {
//                    for (String s : rewardedAdConfigMap.keySet()) {
//                        PokktCustomNetworkAdDelegate.clearChartboostCacheAds(rewardedAdConfigMap.get(s));
//                    }
//                }
//                if (!nonRewardedAdConfigMap.isEmpty()) {
//                    for (String s : nonRewardedAdConfigMap.keySet()) {
//                        PokktCustomNetworkAdDelegate.clearChartboostCacheAds(nonRewardedAdConfigMap.get(s));
//                    }
//                }
                nonRewardedAdConfigMap.clear();
                rewardedAdConfigMap.clear();
                callackMap.clear();
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG, e);
        }
    }

    public static boolean handleChartBoostBack() {
        Logger.d(TAG + " Handle back press");
        return isChartboostPlayed && Chartboost.onBackPressed();
    }

    private void requestFailed(String msg, Callbacks.WithOnlyFailure<String> failureCallback) {
        failureCallback.onFailure(msg);
    }

    private boolean sdkEnabled() {
        try {
            Class.forName("com.chartboost.sdk.Chartboost");
            Class.forName("com.chartboost.sdk.ChartboostDelegate");
            Class.forName("com.chartboost.sdk.Libraries.CBLogging");
            return true;
        } catch (Throwable ignore) {
            Logger.e(TAG + " SDK not found");
            return false;
        }
    }

}
