package com.pokkt.thirdparty.Facebook;

import android.app.Activity;
import android.content.Context;
import android.text.TextUtils;

import com.facebook.ads.Ad;
import com.facebook.ads.AdError;
import com.facebook.ads.AdListener;
import com.facebook.ads.AdSettings;
import com.facebook.ads.AdSize;
import com.facebook.ads.AdView;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.banners.PokktBannerView;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;

import org.json.JSONObject;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class FacebookBannerManager {

    private static final String TAG = FacebookBannerManager.class.getName();

    private AdNetworkInfo adNetworkInfo;
    private String testDeviceId = "";
    private Map<AdConfig, FacebookBannerUnit> adMap = new HashMap<AdConfig, FacebookBannerUnit>();//map of ad unit to ad
    private AdConfig facebookAdConfig;
    private Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks;
    private boolean isTimedOut;

    public FacebookBannerManager(AdNetworkInfo info, String testDeviceId) {
        this.adNetworkInfo = info;
        this.testDeviceId = testDeviceId;
    }

    public void loadBanner(AdConfig config, Context context, BannerUnit bannerUnit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {

        FacebookAdListener facebookAdListener = new FacebookAdListener(context);
        try {
            if (this.facebookAdConfig != null && !(config.equals(this.facebookAdConfig))) {
                callbacks.onFailure("Wrong AdConfig");
                return;
            }
            if (this.facebookAdConfig == null) {
                this.facebookAdConfig = config.clone();
            }
            String placementID = getPlacementId(config);
            if (TextUtils.isEmpty(placementID)) {
                callbacks.onFailure("Invalid Ad Unit");
                return;
            }

            FacebookBannerUnit adUnit = adMap.get(config);
            if (adUnit != null) {
                callbacks.onSuccess(null);
                return;
            }

            this.callbacks = callbacks;
            AdView adview = new AdView(context, placementID, getAdSize(config));
            facebookAdListener.setAdConfig(config);
            facebookAdListener.setBannerAd(adview);
            facebookAdListener.setBannerAdView(bannerUnit);
            if (testDeviceId != null && testDeviceId.length() > 0) {
                List<String> testDeviceIdList = Arrays.asList(testDeviceId.split(","));

                for (String deviceId : testDeviceIdList)
                    Logger.d(TAG + " Test Device Id for Facebook Ad: " + deviceId);

                AdSettings.addTestDevices(testDeviceIdList);
            }
//            adview.disableAutoRefresh();
            adview.setAdListener(facebookAdListener);
            adview.loadAd();
            isTimedOut = false;
        } catch (Throwable e) {
            isTimedOut = true;
            Logger.printStackTrace(TAG + " Load banner failed", e);
            callbacks.onFailure(" Ad Caching Failed");
        }
    }

    private String getPlacementId(AdConfig config) {
        String placementId = "";
        if (adNetworkInfo.getCustomData().containsKey("screens_mapping_data")) {
            try {
                JSONObject mappingObject = new JSONObject(adNetworkInfo.getCustomData().get("screens_mapping_data"));
                String mapData = mappingObject.optString(config.screenName);
                if (!TextUtils.isEmpty(mapData)) {
                    JSONObject mapObj = new JSONObject(mapData);
                    int isBanner = mapObj.optInt("non_incent");
                    if (isBanner == 1) {
                        placementId = mapObj.optString("network_screen", "").trim();
                    } else {
                        Logger.d(TAG + " Failed.Requesting rewarded banner from facebook.");
                        return placementId;
                    }
                }
            } catch (Exception e) {
                Logger.printStackTrace(TAG + " Invalid Mapping Json", e);
            }
        }
        if (placementId.length() == 0) {
            placementId = config.screenName;
        }
        return placementId;
    }

    private AdSize getAdSize(AdConfig config) {
        int height = config.sHeight;
        int width = config.sWidth;
        // 320 * 80
        AdSize closestAdSize;
        int heightDiff = 0;// widthDiff = 0;

        // Standard
        heightDiff = 50 - height;
        //widthDiff = 320 - width;
        closestAdSize = AdSize.BANNER_HEIGHT_50;

        // Large Banner

        if (!(heightDiff < (90 - height))) {
            heightDiff = 90 - height;
            closestAdSize = AdSize.BANNER_HEIGHT_90;
        }
        //widthDiff = 320 - width;

        // Medium Rectangle
        if (!(heightDiff < (250 - height))) {
            heightDiff = 250 - height;
            closestAdSize = AdSize.RECTANGLE_HEIGHT_250;
        }
        //widthDiff = 320 - width;
        return closestAdSize;
    }

    public void destroy(Activity activity) {
        Iterator<AdConfig> i = adMap.keySet().iterator();
        while (i.hasNext()) {
            AdConfig key = i.next();
            FacebookBannerUnit unit = adMap.get(key);
            if (unit.activityName.equals(activity.getLocalClassName())) {
                unit.adView.destroy();
                adMap.remove(key);
            }
        }
    }

    public void destroyBanner(AdConfig adConfig) {

        Iterator<AdConfig> i = adMap.keySet().iterator();
        while (i.hasNext()) {
            AdConfig key = i.next();
            if (key.equals(adConfig)) {
                FacebookBannerUnit unit = adMap.get(key);
                unit.adView.destroy();
                adMap.remove(key);
            }
        }
//        if (bannerUnit != null) {
//            List<PokktBannerView> containerList = bannerUnit.getContainerList();
//            for (int j = 0; j < containerList.size(); j++) {
//                PokktBannerView container = containerList.get(j);
//                container.removeAllViews();
//            }
//        }
    }

    //TODO need to check multiple adconfigs handling.
    public void notifyCachingTimeout() {
        isTimedOut = true;
        String reason = " Time Out In Fetching Banner";
        Logger.e(TAG + reason);
    }

    private class FacebookBannerUnit {
        private AdView adView;
        private String activityName;

        FacebookBannerUnit(String name, AdView adView) {
            this.adView = adView;
            this.activityName = name;
        }
    }

    private class FacebookAdListener implements AdListener {

        private final Context context;//TODO shd be activity context
        private AdView adView = null;
        private BannerUnit unit;
        private AdConfig adConfig;

        FacebookAdListener(Context context) {
            this.context = context;
        }

        void setBannerAd(AdView adView) {
            this.adView = adView;
        }

        void setBannerAdView(BannerUnit bannerAdView) {
            this.unit = bannerAdView;
        }

        @Override
        public void onError(Ad ad, AdError adError) {
            if (!isTimedOut) {
                callbacks.onFailure("Ad Load Failed " + adError.toString());
                destroyBanner(facebookAdConfig);
            }
        }

        @Override
        public void onAdLoaded(Ad ad) {
            if (!isTimedOut) {
                if (!(adMap.containsKey(adConfig))) {
                    adMap.put(adConfig, new FacebookBannerUnit((((Activity) context).getClass().getName()), adView));
                }

                List<PokktBannerView> containerList = unit.getContainerList();
//                for (int i = 0; i < containerList.size(); i++) {
                if (containerList.size() > 0) {
                    PokktBannerView container = containerList.get(0);
                    if (container.getChildAt(0) == null) {
                        container.addView(adView);
                    } else {
                        container.removeView(container.getChildAt(0));
                        container.addView(adView);
//                    }
                    }
                    callbacks.onSuccess(null);
                } else {
                    destroyBanner(adConfig);
                }
            }
        }

        @Override
        public void onAdClicked(Ad ad) {
        }

        @Override
        public void onLoggingImpression(Ad ad) {

        }

        void setAdConfig(AdConfig config) {
            this.adConfig = config;
        }
    }
}
