package com.pokkt.thirdparty;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.os.Bundle;
import android.support.annotation.Keep;

import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdFormat;
import com.pokkt.sdk.adnetworks.AdNetwork;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;
import com.pokkt.thirdparty.Facebook.FacebookBannerManager;
import com.pokkt.thirdparty.Facebook.FacebookInterstitialManager;

@Keep
public class FacebookNetwork implements AdNetwork, Application.ActivityLifecycleCallbacks {

    private static final String TAG = FacebookNetwork.class.getName();
    private static final String TEST_DEVICE_ID = "TestDeviceId";

    private Context context = null;
    private AdNetworkInfo adNetworkInfo;
    private boolean isInitialized = false;
    private FacebookBannerManager facebookBannerManager;
    private FacebookInterstitialManager facebookInterstitialManager;

    @Override
    public void initNetwork(final Context context, final AdNetworkInfo networkInfo) {
        Logger.d(TAG + " Init network called !");

        synchronized (FacebookNetwork.class) {
            if (isInitialized) {
                Logger.d(TAG + " Network already initialized !");
                return;
            }

            if (!sdkEnabled()) {
                return;
            }

            if (!(context instanceof Activity)) {
                Logger.d(TAG + " Init Configurations Error ! Needs Activity context");
                return;
            }
//            FacebookSdk.setIsDebugEnabled(PokktAds.Debugging.isEnabled());

            if (extrasAreValid(networkInfo)) {
                this.context = context;
                ((Activity) context).getApplication().registerActivityLifecycleCallbacks(this);
                this.adNetworkInfo = networkInfo;
                String testDeviceId = networkInfo.getCustomData().get(TEST_DEVICE_ID);
                facebookBannerManager = new FacebookBannerManager(networkInfo, testDeviceId);
                facebookInterstitialManager = new FacebookInterstitialManager(networkInfo, testDeviceId);
                Logger.d(TAG + " Network initialized !");
                isInitialized = true;
            } else {
                Logger.d(TAG + " Configurations Error !");
            }
        }
    }

    @Override
    public AdNetworkInfo getAdNetworkInfo() {
        return adNetworkInfo;
    }

    @Override
    public void cacheAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
        facebookInterstitialManager.fetchAd(config, callbacks, context);
    }

    @Override
    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {
        Logger.d(TAG + " Play interstitial called !");
        facebookInterstitialManager.showAd(config, failureCallback);
    }

    @Override
    public void adClosed(AdConfig adConfig) {

    }

    @Override
    public void checkAdAvailability(AdConfig config, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        Logger.d(TAG + " Facebook Check Ad Available called !");

        if (facebookInterstitialManager.isInterstitialAvailable(config)) {
            callbacks.onSuccess(null);
            return;
        }
        callbacks.onFailure("No Ad Available.");
    }

    @Override
    public void loadBannerAd(final AdConfig config, final BannerUnit bannerUnit, final Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        Logger.d(TAG + " Load Banner called");
        if ((context instanceof Activity)) {
            try {
                ((Activity)context).runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        facebookBannerManager.loadBanner(config, context, bannerUnit, callbacks);
                    }
                });
            } catch (Throwable e) {
                callbacks.onFailure(" load banner Failed");
            }
        } else {
            callbacks.onFailure("Banner Load Failed ! Needs Activity context !");
        }
    }

    @Override
    public void destroyBannerAd(AdConfig config) {
        facebookBannerManager.destroyBanner(config);
    }

    @Override
    public void notifyCachingTimeout(AdConfig config) {
        if (AdFormat.INTERSTITIAL == config.adFormat) {
            facebookInterstitialManager.notifyCachingTimeout();
        } else if (AdFormat.BANNER == config.adFormat) {
            facebookBannerManager.notifyCachingTimeout();
        }
    }

    @Override
    public double getAdVC(AdConfig config) {
        return 0.0;
    }

    @Override
    public boolean supportsAdConfig(AdConfig config) {
        if (AdFormat.BRANDING == config.adFormat) {
            return false;
        }

        if (AdFormat.INTERSTITIAL == config.adFormat) {
            if (config.isRewarded) {
                return false;
            }
        }
        if (AdFormat.VIDEO == config.adFormat) {
            return false;
        }
        return true;
    }

    @Override
    public boolean isInitialised() {
        return isInitialized;
    }

    @Override
    public boolean isPokktNetwork() {
        return false;
    }

    @Override
    public boolean verifyCampaignForAdConfig(AdConfig config, boolean verifyCached) {
        return true;
    }

    private boolean extrasAreValid(AdNetworkInfo adNetworkInfo) {
        return adNetworkInfo != null && adNetworkInfo.getCustomData() != null;
    }

    private boolean sdkEnabled() {
        try {
            Class.forName("com.facebook.ads.Ad");
            Class.forName("com.facebook.ads.AdError");
            Class.forName("com.facebook.ads.AdSettings");
            return true;
        } catch (Throwable ignore) {
            Logger.e(TAG + " SDK not found");
            return false;
        }
    }

    @Override
    public void onActivityCreated(Activity activity, Bundle savedInstanceState) {

    }

    @Override
    public void onActivityStarted(Activity activity) {

    }

    @Override
    public void onActivityResumed(Activity activity) {

    }

    @Override
    public void onActivityPaused(Activity activity) {

    }

    @Override
    public void onActivityStopped(Activity activity) {

    }

    @Override
    public void onActivitySaveInstanceState(Activity activity, Bundle outState) {

    }

    @Override
    public void onActivityDestroyed(Activity activity) {
        Logger.d(TAG + " " + activity.getClass().getName() + " destroyed");
        facebookBannerManager.destroy(activity);
    }
}