package com.pokkt.thirdparty.Flurry;

import android.content.Context;
import android.text.TextUtils;

import com.flurry.android.ads.FlurryAdBanner;
import com.flurry.android.ads.FlurryAdBannerListener;
import com.flurry.android.ads.FlurryAdErrorType;
import com.flurry.android.ads.FlurryAdTargeting;
import com.pokkt.PokktAds;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.banners.PokktBannerView;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;

import org.json.JSONObject;

import java.util.HashMap;
import java.util.Map;

public class FlurryBannerManager {

    private static final String TAG = FlurryBannerManager.class.getName();

    private AdNetworkInfo adNetworkInfo;
    private Callbacks.WithSuccessAndFailure<AdCampaign, String> callback;
    private Map<AdConfig, FlurryAdBanner> adMap = new HashMap<>();
    private boolean isTimedOut;

    public FlurryBannerManager(AdNetworkInfo info) {
        this.adNetworkInfo = info;
    }

    public void loadBanner(AdConfig config, BannerUnit bannerUnit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks, Context context) {
        Logger.d(TAG + " cache Ad called !");

        String adUnitId = getAdUnitId(config);
        if (TextUtils.isEmpty(adUnitId)) {
            callbacks.onFailure("Invalid Ad Unit");
            return;
        }

        FlurryBannerListener listener = new FlurryBannerListener(config);

        this.callback = callbacks;
        try {
            FlurryAdBanner adUnit = adMap.get(config);
            if (adUnit != null) {
                callbacks.onSuccess(null);
                return;
            }
            PokktBannerView container = bannerUnit.getContainerList().get(0);
            FlurryAdBanner flurryAdBanner = new FlurryAdBanner(context, container, adUnitId);
            FlurryAdTargeting flurryAdTargeting = new FlurryAdTargeting();
            flurryAdTargeting.setEnableTestAds(PokktAds.Debugging.isEnabled());
//            flurryAdTargeting.setEnableTestAds(false);//TODO
            flurryAdBanner.setTargeting(flurryAdTargeting);
            flurryAdBanner.setListener(listener);
            isTimedOut = false;
            flurryAdBanner.fetchAd();

        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Load banner failed", e);
            isTimedOut = true;
            callbacks.onFailure(" Ad Caching Failed");
        }
    }

    private String getAdUnitId(AdConfig config) {
        String adUnitId = "";
        if (adNetworkInfo.getCustomData().containsKey("screens_mapping_data")) {
            try {
                JSONObject mappingObject = new JSONObject(adNetworkInfo.getCustomData().get("screens_mapping_data"));
                String mapData = mappingObject.optString(config.screenName);
                if (!TextUtils.isEmpty(mapData)) {
                    JSONObject mapObj = new JSONObject(mapData);
                    int isBanner = mapObj.optInt("non_incent");
                    if (isBanner == 1) {
                        adUnitId = mapObj.optString("network_screen", "").trim();
                    } else {
                        Logger.d(TAG + " Failed.Requesting rewarded banner.");
                        return adUnitId;
                    }
                }
            } catch (Exception e) {
                Logger.printStackTrace(TAG + " Invalid Mapping Json", e);
            }
        }
//        if (adUnitId.length() == 0) {
//            adUnitId = config.screenName;
//        }
        return adUnitId;
    }

    public void destroyBanner(AdConfig adConfig) {
        FlurryAdBanner unit = adMap.get(adConfig);
        if (unit != null) {
            unit.destroy();
            adMap.remove(adConfig);
        }
    }

    public void notifyCachingTimeout() {
        isTimedOut = true;
        String reason = " Time Out In Fetching Ad";
        Logger.e(TAG + reason);
    }

    private class FlurryBannerListener implements FlurryAdBannerListener {

        private final AdConfig adConfig;

        FlurryBannerListener(AdConfig config) {
            this.adConfig = config;
        }

        @Override
        public void onFetched(FlurryAdBanner flurryAdBanner) {
            Logger.d(TAG + " onFetched ! ");
            if (!isTimedOut) {
                isTimedOut = true;
                callback.onSuccess(null);
                adMap.put(adConfig, flurryAdBanner);
                flurryAdBanner.displayAd();
            }
        }

        @Override
        public void onRendered(FlurryAdBanner flurryAdBanner) {
            Logger.d(TAG + " onRendered ! ");
        }

        @Override
        public void onShowFullscreen(FlurryAdBanner flurryAdBanner) {
            Logger.d(TAG + " onShowFullscreen ! ");
        }

        @Override
        public void onCloseFullscreen(FlurryAdBanner flurryAdBanner) {
            Logger.d(TAG + " onCloseFullscreen ! ");
        }

        @Override
        public void onAppExit(FlurryAdBanner flurryAdBanner) {
            Logger.d(TAG + " banner leave app ");
            destroyBanner(adConfig);

        }

        @Override
        public void onClicked(FlurryAdBanner flurryAdBanner) {
            Logger.d(TAG + " onClicked !");
        }

        @Override
        public void onVideoCompleted(FlurryAdBanner flurryAdBanner) {
            Logger.d(TAG + " onVideoCompleted ! ");
        }

        @Override
        public void onError(FlurryAdBanner flurryAdBanner, FlurryAdErrorType flurryAdErrorType, int i) {
            Logger.d(TAG + " banner onError !! " + i + " : " + flurryAdErrorType);
            if (!(isTimedOut) && FlurryAdErrorType.FETCH == flurryAdErrorType) {
                isTimedOut = true;
                callback.onFailure("Ad Load Failed " + i);
                destroyBanner(adConfig);
            }
        }
    }
}