package com.pokkt.thirdparty.Flurry;

import android.content.Context;
import android.text.TextUtils;

import com.flurry.android.ads.FlurryAdErrorType;
import com.flurry.android.ads.FlurryAdInterstitial;
import com.flurry.android.ads.FlurryAdInterstitialListener;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.debugging.Logger;

import org.json.JSONObject;

import java.util.HashMap;
import java.util.Map;

public class FlurryInterstitialManager {

    private static final String TAG = FlurryInterstitialManager.class.getName();
    private double amount = 0;

    private AdNetworkInfo adNetworkInfo;
    private Map<String, FlurryAdInterstitial> adMap = new HashMap<String, FlurryAdInterstitial>();//map of ad unit to ad
    private AdConfig flurryAdConfig = null;
    private Callbacks.WithSuccessAndFailure<Double, String> callback;
    private boolean isTimedOut = false;

    public FlurryInterstitialManager(AdNetworkInfo info, double amount) {
        this.adNetworkInfo = info;
        this.amount = amount;
    }

    public void fetchAd(final AdConfig config, final Callbacks.WithSuccessAndFailure<Double, String> callbacks, Context context) {
        Logger.i(TAG + " cache Ad called !");

        FlurryAdListener listener = new FlurryAdListener(config);

        try {
            String adUnitId = getAdUnitId(config);
            if (TextUtils.isEmpty(adUnitId)) {
                callbacks.onFailure("Ad Caching Failed");
                return;
            }
            if (this.flurryAdConfig != null && !(config.equals(this.flurryAdConfig))) {
                callbacks.onFailure("Fetch Ad Failed. Wrong AdConfig.");
                return;
            }
            if (this.flurryAdConfig == null) {
                this.flurryAdConfig = config.clone();
            }
            FlurryAdInterstitial ad = adMap.get(getAdUnitId(config));
            if (ad != null && ad.isReady()) {
                callbacks.onSuccess(0.0);
                return;
            }

            this.callback = callbacks;
            FlurryAdInterstitial interstitialAd = new FlurryAdInterstitial(context, adUnitId);
            listener.setAdUnitID(adUnitId);
            listener.setInterstitialAd(interstitialAd);
            interstitialAd.setListener(listener);
            interstitialAd.fetchAd();
            isTimedOut = false;
        } catch (Throwable e) {
            this.flurryAdConfig = null;
            Logger.printStackTrace(TAG + " Cache Ad failed ", e);
            isTimedOut = true;
            callbacks.onFailure("Ad Caching Failed");
        }
    }

    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback, Context context) {

        Logger.i(TAG + " show Interstitial called !");
        String adUnitId = "";
        try {
            adUnitId = getAdUnitId(config);
            if (TextUtils.isEmpty(adUnitId)) {
                adMap.remove(adUnitId);
                requestFailed("Ad Not Available !", failureCallback);
                return;
            }
            if (this.flurryAdConfig != null && !(config.equals(this.flurryAdConfig))) {
                failureCallback.onFailure("Fetch Ad Failed. Wrong AdConfig.");
                return;
            }
            if (this.flurryAdConfig == null) {
                this.flurryAdConfig = config.clone();
            }
            FlurryAdInterstitial ad = adMap.get(getAdUnitId(config));
            if (ad != null && ad.isReady()) {
                ad.displayAd();
            } else {
                Logger.i(TAG + " No Ad is currently available!!");
                adMap.remove(adUnitId);
                requestFailed("Ad Not Available !", failureCallback);
            }
        } catch (Throwable e) {
            adMap.remove(adUnitId);
            requestFailed("Ad Not Available !", failureCallback);
        }
    }

    private void requestFailed(String msg, Callbacks.WithOnlyFailure<String> failureCallback) {
        this.flurryAdConfig = null;
        failureCallback.onFailure(msg);
    }

    public boolean isInterstitialAvailable(AdConfig config) {
        try {
            String zoneId = getAdUnitId(config);
            if (!TextUtils.isEmpty(zoneId)) {
                if (this.flurryAdConfig != null && !(config.equals(this.flurryAdConfig))) {
                    Logger.i(TAG + " Fetch Ad Failed. Wrong AdConfig.");
                    return false;
                }
            }
            FlurryAdInterstitial ad = adMap.get(getAdUnitId(config));
            if (ad != null && ad.isReady()) {
                Logger.i(TAG + " ad is available and ready !!");
                return true;
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        return false;
    }

    private String getAdUnitId(AdConfig config) {
        String adUnitId = "";
        if (adNetworkInfo.getCustomData().containsKey("screens_mapping_data")) {
            try {
                JSONObject mappingObject = new JSONObject(adNetworkInfo.getCustomData().get("screens_mapping_data"));
                String mapData = mappingObject.optString(config.screenName);
                if (!TextUtils.isEmpty(mapData)) {
                    JSONObject mapObj = new JSONObject(mapData);
                    adUnitId = mapObj.optString("network_screen", "").trim();
                }
            } catch (Exception e) {
                Logger.printStackTrace(TAG + " Invalid Mapping Json", e);
            }
        }
        if (adUnitId.length() == 0) {
            adUnitId = config.screenName;
        }
        return adUnitId;
    }

    public void notifyCachingTimeout() {
        isTimedOut = true;
        String reason = " Time Out In Fetching Ad";
        flurryAdConfig = null;
        Logger.e(TAG + reason);
    }

    private class FlurryAdListener implements FlurryAdInterstitialListener {
        private AdConfig adConfig;
        private String adUnitID = "";
        private FlurryAdInterstitial interstitialAd = null;

        void setAdUnitID(String adUnitId) {
            this.adUnitID = adUnitId;
        }

        void setInterstitialAd(FlurryAdInterstitial interstitialAd) {
            this.interstitialAd = interstitialAd;
        }

        FlurryAdListener(AdConfig config) {
            this.adConfig = config;
        }

        @Override
        public void onFetched(FlurryAdInterstitial flurryAdInterstitial) {
            Logger.i(TAG + " onAdLoadSucceeded !");
            adMap.put(adUnitID, interstitialAd);

            if (!isTimedOut) {
                isTimedOut = true;
                callback.onSuccess(0.0);
            }
        }

        @Override
        public void onRendered(FlurryAdInterstitial flurryAdInterstitial) {

        }

        @Override
        public void onDisplay(FlurryAdInterstitial flurryAdInterstitial) {
            Logger.i(TAG + " onAdOpened !");
            AdManager.getInstance().adDisplayed(adConfig, adNetworkInfo);
        }

        @Override
        public void onClose(FlurryAdInterstitial flurryAdInterstitial) {
            Logger.i(TAG + " onAdClosed !");
            adMap.remove(adUnitID);
            AdManager.getInstance().adClosed(adConfig, adNetworkInfo);
            flurryAdConfig = null;
        }

        @Override
        public void onAppExit(FlurryAdInterstitial flurryAdInterstitial) {

        }

        @Override
        public void onClicked(FlurryAdInterstitial flurryAdInterstitial) {

        }

        @Override
        public void onVideoCompleted(FlurryAdInterstitial flurryAdInterstitial) {
            Logger.i(TAG + " onVideoCompleted !");
            AdManager.getInstance().adCompleted(adConfig, adNetworkInfo);
            if (adConfig.isRewarded) {
                AdManager.getInstance().adGratified(adConfig, amount, adNetworkInfo);
            }
        }

        @Override
        public void onError(FlurryAdInterstitial flurryAdInterstitial, FlurryAdErrorType flurryAdErrorType, int i) {
            Logger.i(TAG + " onError ! " + i);
            if (!isTimedOut && FlurryAdErrorType.FETCH == flurryAdErrorType) {
                isTimedOut = true;
                callback.onFailure("Ad Load Failed ! " + i);
                flurryAdConfig = null;
            } else if (FlurryAdErrorType.RENDER == flurryAdErrorType) {
                AdManager.getInstance().adClosed(adConfig, adNetworkInfo);
                flurryAdConfig = null;
            }
        }
    }
}