package com.pokkt.thirdparty;

import android.content.Context;
import android.support.annotation.Keep;

import com.flurry.android.FlurryAgent;
import com.pokkt.PokktAds;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdFormat;
import com.pokkt.sdk.adnetworks.AdNetwork;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;
import com.pokkt.thirdparty.Flurry.FlurryBannerManager;
import com.pokkt.thirdparty.Flurry.FlurryInterstitialManager;

@Keep
public class FlurryNetwork implements AdNetwork {

    private static final String TAG = FlurryNetwork.class.getName();

    private Context context = null;
    private AdNetworkInfo adNetworkInfo;
    private boolean isInitialized = false;
    private FlurryInterstitialManager flurryInterstitialManager;
    private FlurryBannerManager flurryBannerManager;
    private static final String API_KEY = "ApiKey";
    private double amount = 0;

    @Override
    public void initNetwork(final Context context, final AdNetworkInfo info) {
        Logger.d(TAG + " init network called !");

        synchronized (FlurryNetwork.class) {
            if (isInitialized) {
                Logger.d(TAG + " Network already initialized !");
                return;
            }
            this.context = context;

            if (!sdkEnabled()) {
                Logger.d(TAG + " Init Configurations Error !");
                return;
            }

            if (extrasAreValid(info)) {
                this.adNetworkInfo = info;
                String apiKey = info.getCustomData().get(API_KEY);
                Logger.d(TAG + " init network initialized !!");

                if (info.getCustomData().containsKey("amount")) {
                    try {
                        amount = Float.parseFloat(info.getCustomData().get("amount"));
                    } catch (Exception e) {
                        Logger.printStackTrace(TAG + " Could not parse amount", e);
                        //do nothing
                    }
                }
                FlurryAgent.setLogEnabled(PokktAds.Debugging.isEnabled());
                // init Flurry
                FlurryAgent.init(context, apiKey);
                if (!FlurryAgent.isSessionActive()) {
                    FlurryAgent.onStartSession(context, apiKey);
                }
                flurryInterstitialManager = new FlurryInterstitialManager(info, amount);
                flurryBannerManager = new FlurryBannerManager(info);
                isInitialized = true;
            } else {
                Logger.d(TAG + " Init Configurations Error or SDK Error !");
            }
        }
    }

    @Override
    public AdNetworkInfo getAdNetworkInfo() {
        return adNetworkInfo;
    }

    private boolean extrasAreValid(AdNetworkInfo info) {
        return info != null && info.getCustomData() != null && info.getCustomData().containsKey(API_KEY);
    }

    @Override
    public void cacheAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
        if (AdFormat.INTERSTITIAL == config.adFormat) {
            flurryInterstitialManager.fetchAd(config, callbacks, context);
        } else {
            callbacks.onFailure("Invalid Ad Format");
        }
    }

    @Override
    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {
        if (AdFormat.INTERSTITIAL == config.adFormat) {
            flurryInterstitialManager.showAd(config, failureCallback, context);
        } else {
            failureCallback.onFailure("Invalid Ad Format");
        }
    }

    @Override
    public void adClosed(AdConfig adConfig) {

    }

    @Override
    public void checkAdAvailability(AdConfig config, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        try {
            if (AdFormat.INTERSTITIAL == config.adFormat) {
                if (flurryInterstitialManager.isInterstitialAvailable(config)) {
                    callbacks.onSuccess(null);
                    return;
                }
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        callbacks.onFailure("Ad Not Available");
    }

    @Override
    public void loadBannerAd(AdConfig adConfig, BannerUnit bannerUnit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        flurryBannerManager.loadBanner(adConfig, bannerUnit, callbacks, context);
    }

    @Override
    public void destroyBannerAd(AdConfig adConfig) {
        flurryBannerManager.destroyBanner(adConfig);
    }

    @Override
    public void notifyCachingTimeout(AdConfig adConfig) {
        if (AdFormat.INTERSTITIAL == adConfig.adFormat) {
            flurryInterstitialManager.notifyCachingTimeout();
        } else if (AdFormat.BANNER == adConfig.adFormat) {
            flurryBannerManager.notifyCachingTimeout();
        }
    }

    @Override
    public double getAdVC(AdConfig adConfig) {
        if (adConfig.isRewarded) {
            return amount;
        } else {
            return 0.0;
        }
    }

    @Override
    public boolean supportsAdConfig(AdConfig config) {
        if (AdFormat.BRANDING == config.adFormat) {
            return false;
        }

        if (AdFormat.INTERSTITIAL == config.adFormat) {
            if (config.isRewarded) {
                return false;
            }
        }
        if (AdFormat.VIDEO == config.adFormat) {
            return false;
        }
        return true;
    }

    @Override
    public boolean isInitialised() {
        return isInitialized;
    }

    @Override
    public boolean isPokktNetwork() {
        return false;
    }

    @Override
    public boolean verifyCampaignForAdConfig(AdConfig adConfig, boolean verifyCached) {
        return true;
    }

    private boolean sdkEnabled() {
        try {
            Class.forName("com.flurry.android.FlurryAgent");
            return true;
        } catch (Throwable ignore) {
            Logger.e(TAG + " SDK not found");
            return false;
        }
    }
}