package com.pokkt.thirdparty.Fyber;

import android.app.Activity;
import android.content.Context;
import android.support.annotation.Keep;

import com.fyber.ads.banners.BannerAd;
import com.fyber.ads.banners.BannerAdListener;
import com.fyber.ads.banners.BannerAdView;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.banners.PokktBannerView;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Keep
public class FyberBannerManager {

    public static final String TAG = FyberBannerManager.class.getName();

    private Callbacks.WithSuccessAndFailure<AdCampaign, String> callback;
    private Map<AdConfig, BannerAd> adMap = new HashMap<>();//map of ad unit to ad
    private boolean isTimedOut;

    public void loadBanner(final AdConfig config, final BannerUnit unit, final Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks, final Context context) {
        Logger.d(TAG + " Cache Ad Called");

        this.callback = callbacks;

        ((Activity) context).runOnUiThread(new Runnable() {
            @Override
            public void run() {
                try {
                    FyberListener listener = new FyberListener(unit, config);
                    BannerAd adUnit = adMap.get(config);
                    if (adUnit != null) {
                        callbacks.onSuccess(null);
                        return;
                    }
                    BannerAdView bannerAdView = new BannerAdView((Activity) context).withListener(listener);
//                    .withPlacementId("ca-app-pub-9403393692924373/3838929242")
//                    .withNetworkSize(AdMobNetworkBannerSizes.BANNER);
//                    .withNetworkSize(bannerSize);
                    listener.setFyberBanner(bannerAdView);
                    bannerAdView.load();
                    isTimedOut = false;
                } catch (Throwable e) {
                    Logger.printStackTrace(TAG + " Load banner failed", e);
                    isTimedOut = true;
                    callbacks.onFailure(" Ad Caching Failed");
                }
            }
        });
    }

    public void destroyBanner(AdConfig adConfig) {
        BannerAd unit = adMap.get(adConfig);
        if (unit != null) {
            unit.destroy();
            adMap.remove(adConfig);
        }
    }

    public void notifyCachingTimeout() {
        isTimedOut = true;
        String reason = " Time Out In Fetching Ad";
        Logger.e(TAG + reason);
    }

    private class FyberListener implements BannerAdListener {

        private final AdConfig adConfig;
        private BannerUnit bannerUnit;
        private BannerAdView fyberBanner;

        FyberListener(BannerUnit unit, AdConfig config) {
            this.bannerUnit = unit;
            this.adConfig = config;
        }

        @Override
        public void onAdError(BannerAd bannerAd, String s) {
            Logger.d(TAG + " Something went wrong with the request: " + s);
            if (!isTimedOut) {
                Logger.d(TAG + " banner load failed " + s);
                isTimedOut = true;
                callback.onFailure("Ad Load Failed " + s);
                if (bannerAd != null) {
                    bannerAd.destroy();
                }
            }
        }

        @Override
        public void onAdLoaded(BannerAd bannerAd) {
            if (!isTimedOut) {
                isTimedOut = true;
                adMap.put(adConfig, bannerAd);
                List<PokktBannerView> containerList = bannerUnit.getContainerList();
//                for (int i = 0; i < containerList.size(); i++) {
                if (containerList.size() > 0) {
                    PokktBannerView container = containerList.get(0);
                    if (container.getChildAt(0) == null) {
                        container.addView(fyberBanner);
                    } else {
                        container.removeView(container.getChildAt(0));
                        container.addView(fyberBanner);
                    }
                    callback.onSuccess(null);
                } else {
                    destroyBanner(adConfig);
                }
//                }
            }
        }

        @Override
        public void onAdClicked(BannerAd bannerAd) {
        }

        @Override
        public void onAdLeftApplication(BannerAd bannerAd) {
            Logger.d(TAG + " banner leave app ");
            if (bannerAd != null) {
                bannerAd.destroy();
            }
        }

        void setFyberBanner(BannerAdView fyberBanner) {
            this.fyberBanner = fyberBanner;
        }
    }
}