package com.pokkt.thirdparty.Fyber;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.support.annotation.Keep;

import com.fyber.ads.AdFormat;
import com.fyber.requesters.InterstitialRequester;
import com.fyber.requesters.RequestCallback;
import com.fyber.requesters.RequestError;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.thirdparty.FyberActivity;
import com.pokkt.thirdparty.FyberNetwork;

@Keep
public class FyberInterstitialManager extends FyberNetwork {

    public static final String TAG = FyberInterstitialManager.class.getName();

    private Intent intent;

    private AdNetworkInfo adNetworkInfo;
    private AdConfig fyberAdConfig;
    private Callbacks.WithOnlyFailure<String> showfailCallback;
    private Callbacks.WithSuccessAndFailure<Double, String> cacheCallback;
    private boolean isTimedOut;

    public FyberInterstitialManager(AdNetworkInfo info) {
        this.adNetworkInfo = info;
    }

    public Intent getIntent() {
        return intent;
    }

    public void setIntent(Intent intent) {
        this.intent = intent;
        if (intent == null) {
            fyberAdConfig = null;
        }
    }

    public void fetchAd(final AdConfig config, final Callbacks.WithSuccessAndFailure<Double, String> callbacks, Context context) {
        Logger.d(TAG + " Cache Ad Called");
        AdRequestCallback interstitialRequestCallback = null;
        try {
            if (fyberAdConfig != null && !fyberAdConfig.equals(config)) {
                callbacks.onFailure("Ad Caching Failed");
                return;
            }
            if (intent != null) {
                Logger.d(TAG + " Ad is cached since last call !!");
                callbacks.onSuccess(0.0);
                return;
            }

            if (fyberAdConfig == null) {
                fyberAdConfig = config.clone();
            }
            this.cacheCallback = callbacks;
            interstitialRequestCallback = new AdRequestCallback();
            performRequest(interstitialRequestCallback, context);
            isTimedOut = false;
        } catch (Exception e) {
            Logger.printStackTrace(TAG + " Cache Ad Failed", e);
            isTimedOut = true;
            callbacks.onFailure("Ad Caching Failed");
            fyberAdConfig = null;
        }
    }

    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback, Context context) {
        Logger.d(TAG + " Play Ad Called");

        showfailCallback = failureCallback;
        if (fyberAdConfig != null && fyberAdConfig.equals(config) && intent != null) {

            try {
                Intent fyberIntent = new Intent(context, FyberActivity.class);
                fyberIntent.putExtra("adRequestInfo", config);
                fyberIntent.putExtra("network", adNetworkInfo);
                context.startActivity(fyberIntent);
            } catch (Throwable e) {
                Logger.printStackTrace(TAG + " Play Ad Failed", e);
                requestFailed("Play Ad Failed !");
            }
        } else {
            requestFailed("Ad Not Available !");
        }
    }

    public boolean checkAdAvailable(AdConfig adConfig) {
        try {
            if (intent != null && fyberAdConfig != null && fyberAdConfig.equals(adConfig)) {
                return true;
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        return false;
    }

    private void performRequest(final AdRequestCallback adRequestCallback, final Context context) {
        ((Activity) context).runOnUiThread(new Runnable() {
            @Override
            public void run() {
                try {
                    InterstitialRequester.create(adRequestCallback).request(context);
                } catch (Throwable e) {
                    cacheCallback.onFailure("Ad Caching Failed");
                    fyberAdConfig = null;
                }
            }
        });
    }

    public void notifyCachingTimeout() {
        isTimedOut = true;
        String reason = " Time Out In Fetching Ad";
        Logger.e(TAG + reason);
        fyberAdConfig = null;
    }

    private class AdRequestCallback implements RequestCallback {

        AdRequestCallback() {
        }

        @Override
        public void onAdAvailable(Intent intent) {
            setIntent(intent);
            if (!isTimedOut) {
                isTimedOut = true;
                cacheCallback.onSuccess(0.0);
            }
        }

        @Override
        public void onAdNotAvailable(AdFormat adFormat) {
            Logger.d(TAG + " " + adFormat.name() + " Not available");
            if (!isTimedOut) {
                isTimedOut = true;
                cacheCallback.onFailure("No Ad Available");
                fyberAdConfig = null;
            }
        }

        @Override
        public void onRequestError(RequestError requestError) {
            Logger.d(TAG + " Something went wrong with the request: " + requestError.getDescription());
            if (!isTimedOut) {
                isTimedOut = true;
                cacheCallback.onFailure("No Ad Available");
                fyberAdConfig = null;
            }
        }
    }

    private void requestFailed(String msg) {
        intent = null;
        fyberAdConfig = null;
        showfailCallback.onFailure(msg);
    }
}