package com.pokkt.thirdparty.Fyber;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;

import com.fyber.ads.AdFormat;
import com.fyber.currency.VirtualCurrencyErrorResponse;
import com.fyber.currency.VirtualCurrencyResponse;
import com.fyber.requesters.RequestCallback;
import com.fyber.requesters.RequestError;
import com.fyber.requesters.RewardedVideoRequester;
import com.fyber.requesters.VirtualCurrencyCallback;
import com.fyber.requesters.VirtualCurrencyRequester;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.thirdparty.FyberActivity;
import com.pokkt.thirdparty.FyberNetwork;


public class FyberVideoAdManager extends FyberNetwork {

    public static final String TAG = FyberVideoAdManager.class.getName();

    private Intent intent;

    private AdNetworkInfo adNetworkInfo;
    private AdConfig fyberAdConfig;
    private Callbacks.WithSuccessAndFailure<Double, String> cacheCallback;
    private boolean isTimedOut;
    private Callbacks.WithOnlyFailure<String> showfailCallback;

    public Intent getIntent() {
        return intent;
    }

    public void setIntent(Intent intent) {
        this.intent = intent;
        if (intent == null) {
            fyberAdConfig = null;
        }
    }

    public FyberVideoAdManager(AdNetworkInfo info) {
        this.adNetworkInfo = info;
    }

    public void fetchAd(final AdConfig config, final Callbacks.WithSuccessAndFailure<Double, String> callbacks, Context context) {
        Logger.d(TAG+ " Cache Ad Called");
        AdRequestCallback videoRequestCallback;
        try {

            if (fyberAdConfig != null && !fyberAdConfig.equals(config)) {
                callbacks.onFailure("Ad Caching Failed");
                return;
            }

            if (intent != null) {
                Logger.d(TAG+ " Ad is cached since last call !!");
                callbacks.onSuccess(0.0);
                return;
            }
            if (fyberAdConfig == null) {
                fyberAdConfig = config.clone();
            }
            this.cacheCallback = callbacks;
            videoRequestCallback = new AdRequestCallback(config);

            performRequest(videoRequestCallback, context);
            isTimedOut = false;
        } catch (Throwable e) {
            isTimedOut = true;
            callbacks.onFailure("Ad Caching Failed");
            fyberAdConfig = null;
        }
    }

    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback, Context context) {
        Logger.d(TAG+ " Play Ad Called");

        showfailCallback = failureCallback;
        if (fyberAdConfig != null && fyberAdConfig.equals(config) && intent != null) {

            try {
                Intent fyberIntent = new Intent(context, FyberActivity.class);
                fyberIntent.putExtra("adRequestInfo", config);
                fyberIntent.putExtra("network", adNetworkInfo);
                context.startActivity(fyberIntent);
            } catch (Throwable e) {
                Logger.printStackTrace(TAG+ " Play Ad Failed", e);
                requestFailed("Play Ad Failed !");
            }
        } else {
            requestFailed("Ad Not Available !");
        }
    }

    public boolean checkAdAvailable(AdConfig adConfig) {
        try {
            if (intent != null && fyberAdConfig != null && fyberAdConfig.equals(adConfig)) {
                return true;
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG+ " Failed to find availability", e);
        }
        return false;
    }

    private VirtualCurrencyRequester getVirtualCurrencyRequester(AdRequestCallback adRequestCallback) {
        return VirtualCurrencyRequester.create(adRequestCallback)
                // user will get a toast notification upon reward
                .notifyUserOnReward(false);
//				this is the currency id for RV ad format
//				 you can refer to this -- http://developer.fyber.com/content/android/basics/rewarding-the-user/vcs/
//				.forCurrencyId("gold")
    }

    private void performRequest(final AdRequestCallback adRequestCallback, final Context context) {

        //Requesting a rewarded video ad
        ((Activity) context).runOnUiThread(new Runnable() {
            @Override
            public void run() {
                try {
                    RewardedVideoRequester.create(adRequestCallback)
                            .withVirtualCurrencyRequester(getVirtualCurrencyRequester(adRequestCallback))
                            .request(context);
                } catch (Throwable e) {
                    cacheCallback.onFailure("Ad Caching Failed");
                    fyberAdConfig = null;
                }
            }
        });
    }

    public void notifyCachingTimeout() {
        isTimedOut = true;
        String reason = " Time Out In Fetching Ad";
        Logger.e(TAG+ reason);
        fyberAdConfig = null;
    }

    private class AdRequestCallback implements RequestCallback, VirtualCurrencyCallback {

        private AdConfig adConfig;

        AdRequestCallback(AdConfig config) {
            this.adConfig = config;
        }

        @Override
        public void onAdAvailable(Intent intent) {
//            setIntent(intent, adConfig.isRewarded());
            setIntent(intent);
            if (!isTimedOut) {
                isTimedOut = true;
                cacheCallback.onSuccess(0.0);
            }
        }

        @Override
        public void onAdNotAvailable(AdFormat adFormat) {
            Logger.d(TAG+ " "+ adFormat.name() + " Not available");
            if (!isTimedOut) {
                isTimedOut = true;
                cacheCallback.onFailure("No Ad Available");
                fyberAdConfig = null;
            }
        }

        @Override
        public void onRequestError(RequestError requestError) {
            Logger.d(TAG+ " Something went wrong with the request: " + requestError.getDescription());
            if (!isTimedOut) {
                isTimedOut = true;
                cacheCallback.onFailure("No Ad Available");
                fyberAdConfig = null;
            }
        }

        @Override
        public void onError(VirtualCurrencyErrorResponse virtualCurrencyErrorResponse) {
            Logger.d(TAG+ " "+ virtualCurrencyErrorResponse.getErrorMessage());
            requestFailed(virtualCurrencyErrorResponse.getErrorMessage());
        }

        @Override
        public void onSuccess(VirtualCurrencyResponse virtualCurrencyResponse) {
            Logger.d(TAG+ " onRewardedVideoCompleted. Rewarded amount: " + virtualCurrencyResponse.getDeltaOfCoins());
            AdManager.getInstance().adGratified(adConfig, virtualCurrencyResponse.getDeltaOfCoins(), adNetworkInfo);
        }
    }

    private void requestFailed(String msg) {
        intent = null;
        fyberAdConfig = null;
        showfailCallback.onFailure(msg);
    }
}