package com.pokkt.thirdparty;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;

import com.fyber.ads.videos.RewardedVideoActivity;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.debugging.Logger;

public class FyberActivity extends Activity {

    private final static String TAG = FyberActivity.class.getName();
    private AdConfig adConfig;
    private AdNetworkInfo adNetworkInfo;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        boolean isRewarded = false;
        try {
            adConfig = (AdConfig) getIntent().getSerializableExtra("adRequestInfo");
            adNetworkInfo = (AdNetworkInfo) getIntent().getSerializableExtra("network");
            this.startActivityForResult(FyberNetwork.getIntent(adConfig), FyberNetwork.REWARDED_VIDEO_REQUEST_CODE);
        } catch (Exception e) {
            Logger.printStackTrace(TAG, e);
            FyberNetwork.setIntent(null, adConfig);
            AdManager.getInstance().adClosed(adConfig, adNetworkInfo);
            finish();
        }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        try {
            if (resultCode == Activity.RESULT_OK && requestCode == FyberNetwork.REWARDED_VIDEO_REQUEST_CODE) {
                String engagementStatus = data.getStringExtra(RewardedVideoActivity.ENGAGEMENT_STATUS);
                if (RewardedVideoActivity.REQUEST_STATUS_PARAMETER_FINISHED_VALUE.equals(engagementStatus)) {
                    Logger.d(TAG + " The video has finished after completing. The user will be rewarded.");
                    AdManager.getInstance().adCompleted(adConfig, adNetworkInfo);

                } else if (RewardedVideoActivity.REQUEST_STATUS_PARAMETER_ABORTED_VALUE.equals(engagementStatus)) {
                    Logger.d(TAG + " Video has been skipped");
                    AdManager.getInstance().adSkipped(adConfig, adNetworkInfo);

                } else if (RewardedVideoActivity.REQUEST_STATUS_PARAMETER_ERROR.equals(engagementStatus)) {
                    Logger.d(TAG + " The video was interrupted or failed to play due to an error.");
                }
            }
        } catch (Exception e) {
            Logger.printStackTrace(TAG, e);
        } finally {
            FyberNetwork.setIntent(null, adConfig);
            AdManager.getInstance().adClosed(adConfig, adNetworkInfo);
            finish();
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        FyberNetwork.setIntent(null, adConfig);
    }
}
