package com.pokkt.thirdparty;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.support.annotation.Keep;

import com.fyber.Fyber;
import com.fyber.utils.FyberLogger;
import com.pokkt.PokktAds;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdFormat;
import com.pokkt.sdk.adnetworks.AdNetwork;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;
import com.pokkt.thirdparty.Fyber.FyberBannerManager;
import com.pokkt.thirdparty.Fyber.FyberInterstitialManager;
import com.pokkt.thirdparty.Fyber.FyberVideoAdManager;

import java.io.Serializable;

@Keep
public class FyberNetwork implements AdNetwork, Serializable {

    static final int REWARDED_VIDEO_REQUEST_CODE = 8796;

    public static final String TAG = FyberNetwork.class.getName();

    private static final String APP_ID = "AppKey";
    private static final String SECURITY_TOKEN = "security_token";

    private Context context = null;
    private boolean isInitialized = false;

    private AdNetworkInfo adNetworkInfo;
    private double amount = 0;
    private static FyberVideoAdManager fyberVideoAdManager;
    private static FyberInterstitialManager fyberInterstitialManager;
    private FyberBannerManager fyberBannerManager;

    static Intent getIntent(AdConfig adConfig) {
        if (AdFormat.VIDEO == adConfig.adFormat) {
            return fyberVideoAdManager.getIntent();
        } else {
            return fyberInterstitialManager.getIntent();
        }
    }

    static void setIntent(Intent intent, AdConfig adConfig) {
        if (AdFormat.VIDEO == adConfig.adFormat) {
            fyberVideoAdManager.setIntent(intent);
        } else {
            fyberInterstitialManager.setIntent(intent);
        }
    }

    @Override
    public void initNetwork(Context context, AdNetworkInfo info) {

        synchronized (FyberNetwork.class) {
            if (isInitialized) {
                Logger.d(TAG + " Network already initialized !");
                return;
            }

            if (!(context instanceof Activity)) {
                Logger.d(TAG + " Init Configurations Error ! Needs Activity context");
                return;
            }


            if (!sdkEnabled()) {
                Logger.d(TAG + " Init Error !");
                return;
            }
            this.context = context;
            this.adNetworkInfo = info;
            FyberLogger.enableLogging(PokktAds.Debugging.isEnabled());

            if (info != null && info.getCustomData() != null && (info.getCustomData().containsKey(APP_ID) || info.getCustomData().containsKey("appKey")) && info.getCustomData().containsKey(SECURITY_TOKEN)) {
                String appId = info.getCustomData().get(APP_ID);
                if (appId == null) {
                    appId = info.getCustomData().get("appKey");
                }
                String securityToken = info.getCustomData().get(SECURITY_TOKEN);
                String amt = info.getCustomData().get("amount");
                try {
                    if (amt != null) {
                        amount = Float.parseFloat(amt);
                    }
                } catch (Exception e) {
                    //do nothing
                    Logger.printStackTrace(TAG + " Failed to find currency ", e);
                }
                try {
                    Fyber.Settings fyberSettings = Fyber
                            .with(appId, ((Activity) context))
                            .withSecurityToken(securityToken)
                            .withUserId(AdManager.getInstance().getThirdPartyUserId())
                            .start();
                    fyberVideoAdManager = new FyberVideoAdManager(info);
                    fyberInterstitialManager = new FyberInterstitialManager(info);
                    fyberBannerManager = new FyberBannerManager();
                    isInitialized = true;
                } catch (Throwable e) {
                    Logger.printStackTrace(TAG + " Init Configurations Error !", e);
                }
            } else {
                Logger.e(TAG + " Init Configurations Error or SDK Error!");
            }
        }
    }

    //User Settings to customise Fyber SDK behaviour
    private void customiseFyberSettings(Fyber.Settings fyberSettings) {
        fyberSettings.notifyUserOnReward(false).notifyUserOnCompletion(true);
    }

    @Override
    public void cacheAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
        Logger.d(TAG + " Cache Ad Called");

        if (AdFormat.VIDEO == config.adFormat) {
            fyberVideoAdManager.fetchAd(config, callbacks, context);
        } else {
            fyberInterstitialManager.fetchAd(config, callbacks, context);
        }
    }

    @Override
    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {
        Logger.d(TAG + " Play Ad Called");

        if (AdFormat.VIDEO == config.adFormat) {
            fyberVideoAdManager.showAd(config, failureCallback, context);
        } else {
            fyberInterstitialManager.showAd(config, failureCallback, context);
        }
    }

    @Override
    public void adClosed(AdConfig adConfig) {

    }

    @Override
    public void checkAdAvailability(AdConfig adConfig, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        try {
            if (AdFormat.VIDEO == adConfig.adFormat) {
                if (fyberVideoAdManager.checkAdAvailable(adConfig)) {
                    callbacks.onSuccess(null);
                    return;
                } else {
                    callbacks.onFailure("Ad Not Available");
                    return;
                }
            } else if (AdFormat.INTERSTITIAL == adConfig.adFormat) {
                if (fyberInterstitialManager.checkAdAvailable(adConfig)) {
                    callbacks.onSuccess(null);
                    return;
                } else {
                    callbacks.onFailure("Ad Not Available");
                    return;
                }
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        callbacks.onFailure("Ad Not Available");
    }

    @Override
    public void loadBannerAd(AdConfig adConfig, BannerUnit bannerUnit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        fyberBannerManager.loadBanner(adConfig, bannerUnit, callbacks, context);
    }

    @Override
    public void destroyBannerAd(AdConfig adConfig) {
        fyberBannerManager.destroyBanner(adConfig);
    }

    @Override
    public void notifyCachingTimeout(AdConfig adConfig) {
        if (AdFormat.VIDEO == adConfig.adFormat) {
            fyberVideoAdManager.notifyCachingTimeout();
        } else if (AdFormat.INTERSTITIAL == adConfig.adFormat) {
            fyberInterstitialManager.notifyCachingTimeout();
        } else if (AdFormat.BANNER == adConfig.adFormat) {
            fyberBannerManager.notifyCachingTimeout();
        }
    }

    @Override
    public double getAdVC(AdConfig adConfig) {
        if (adConfig.isRewarded) {
            return amount;
        } else {
            return 0.0;
        }
    }

    @Override
    public boolean supportsAdConfig(AdConfig config) {
        if (AdFormat.BRANDING == config.adFormat) {
            return false;
        }

        if (AdFormat.INTERSTITIAL == config.adFormat) {
            if (config.isRewarded) {
                return false;
            }
        }
        if (AdFormat.VIDEO == config.adFormat) {
            if (!(config.isRewarded)) {
                return false;
            }
        }
        return true;
    }

    @Override
    public boolean isInitialised() {
        return isInitialized;
    }

    @Override
    public boolean isPokktNetwork() {
        return false;
    }

    @Override
    public boolean verifyCampaignForAdConfig(AdConfig adConfig, boolean verifyCached) {
        return true;
    }

    private boolean sdkEnabled() {
        try {
            Class.forName("com.fyber.ads.AdFormat");
            Class.forName("com.fyber.requesters.RequestError");
            Class.forName("com.fyber.requesters.RequestCallback");
            return true;
        } catch (Throwable ignore) {
            Logger.e(TAG + " SDK not found");
            return false;
        }
    }

    @Override
    public AdNetworkInfo getAdNetworkInfo() {
        return adNetworkInfo;
    }
}