package com.pokkt.thirdparty;

import android.app.Activity;
import android.content.Context;
import android.support.annotation.Keep;
import android.text.TextUtils;

import com.hyprmx.mediate.HyprMediate;
import com.hyprmx.mediate.HyprMediateError;
import com.hyprmx.mediate.HyprMediateListenerWithInit;
import com.hyprmx.mediate.HyprMediateReward;
import com.pokkt.app.pokktsdk.util.PokktStorage;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdFormat;
import com.pokkt.sdk.adnetworks.AdNetwork;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;

@Keep
public class HyprMXNetwork implements AdNetwork {

    private static final String TAG = HyprMXNetwork.class.getName();
    private static final String API_TOKEN = "apiToken";
    private static final String CURRENCY_KEY = "amount";
    private boolean isInitialized = false;
    private AdConfig adConfig;
    private AdNetworkInfo adNetworkInfo;
    private double rewardAmount = 0;
    private boolean isAdAvailable = false;
    private AdListener listener;
    private boolean isTimedOut = false;
    private Callbacks.WithSuccessAndFailure<Double, String> callback;

    @Override
    public void initNetwork(Context context, AdNetworkInfo info) {
        this.adNetworkInfo = info;

        synchronized (HyprMXNetwork.class) {
            if (isInitialized) {
                Logger.d(TAG + " init network already initialized!!");
                return;
            }

            if (!(context instanceof Activity)) {
                Logger.d(TAG + " Init Configurations Error!. Needs Activity context");
                return;
            }

            if (!sdkEnabled()) {
                Logger.d(TAG + " Init Configurations Error!");
                return;
            }
            String hyprAPIToken = "";
            if (adNetworkInfo != null && adNetworkInfo.getCustomData() != null && adNetworkInfo.getCustomData().containsKey(API_TOKEN)) {
                hyprAPIToken = adNetworkInfo.getCustomData().get(API_TOKEN);
                if (adNetworkInfo.getCustomData().containsKey(CURRENCY_KEY)) {
                    try {
                        rewardAmount = Double.parseDouble(adNetworkInfo.getCustomData().get(CURRENCY_KEY));
                    } catch (Exception e) {
                        Logger.printStackTrace(TAG + "Could not parse amount", e);
                        //do nothing
                    }
                }
            } else {
                Logger.e(TAG + " Init Configurations Error !");
            }
            String hyprUserId = AdManager.getInstance().getThirdPartyUserId();
            if (TextUtils.isEmpty(hyprUserId)) {
                hyprUserId = PokktStorage.getStore(context).getAdId();
            }
            try {
                listener = new AdListener();
                HyprMediate.getInstance().initialize((Activity) context, hyprAPIToken, hyprUserId, listener);
            } catch (Throwable e) {
                Logger.printStackTrace(TAG + " SDK Init Error", e);
            }
        }
    }

    @Override
    public AdNetworkInfo getAdNetworkInfo() {
        return adNetworkInfo;
    }

    @Override
    public void cacheAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
        Logger.d(TAG + " Cache Ad Called");

        try {
            if (adConfig != null && !adConfig.equals(config)) {
                callbacks.onFailure("Cache Ad Failed !");
                return;
            }

            if (adConfig == null) {
                adConfig = config.clone();
            }
            this.callback = callbacks;
            HyprMediate.getInstance().checkInventory();
            listener.setAdConfig(config);
            isTimedOut = false;
        } catch (Throwable e) {
            adConfig = null;
            isTimedOut = true;
            Logger.printStackTrace(TAG + e.getLocalizedMessage(), e);
            callbacks.onFailure("Ad Caching Failed");
        }
    }

    @Override
    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {
        Logger.d(TAG + " Play Ad Called");

        try {
            if (this.adConfig != null && !(config.equals(this.adConfig))) {
                failureCallback.onFailure("Fetch Ad Failed. Wrong AdConfig.");
                return;
            }
            if (adConfig == null) {
                adConfig = config.clone();
            }
            if (isAdAvailable) {
                HyprMediate.getInstance().showAd();
            } else {
                Logger.d(TAG + "Rewarded Ad is currently not available!!");
                requestFailed("Ad Not Available !", failureCallback);
            }
        } catch (Throwable e) {
            requestFailed("Ad Not Available !", failureCallback);
        }
    }

    @Override
    public void adClosed(AdConfig adConfig) {

    }

    private void requestFailed(String msg, Callbacks.WithOnlyFailure<String> failureCallback) {
        adConfig = null;
        failureCallback.onFailure(msg);
    }

    @Override
    public void checkAdAvailability(AdConfig config, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        try {
            if (AdFormat.INTERSTITIAL == config.adFormat) {
                callbacks.onFailure("Interstitial is not supported.");
                return;
            }

            if (this.adConfig != null && !(config.equals(this.adConfig))) {
                Logger.e(TAG + " Fetch Ad Failed. Wrong AdConfig.");
                callbacks.onFailure("Fetch Ad Failed. Wrong AdConfig.");
                return;
            }

            if (isAdAvailable) {
                callbacks.onSuccess(null);
                return;
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + "Failed to find availability", e);
        }
        callbacks.onFailure("Ad Not Available");
    }

    public void loadBannerAd(AdConfig adConfig, BannerUnit bannerUnit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        callbacks.onFailure("Invalid Network !");
    }

    @Override
    public void destroyBannerAd(AdConfig adConfig) {

    }

    @Override
    public void notifyCachingTimeout(AdConfig adConfig) {
        isTimedOut = true;
        String reason = " Time Out In Fetching Ad";
        this.adConfig = null;
        Logger.e(TAG + reason);
    }

    @Override
    public double getAdVC(AdConfig adConfig) {
        if (adConfig.isRewarded) {
            return rewardAmount;
        } else {
            return 0;
        }
    }

    @Override
    public boolean supportsAdConfig(AdConfig config) {
        return AdFormat.VIDEO == config.adFormat;
    }

    @Override
    public boolean isInitialised() {
        return isInitialized;
    }

    @Override
    public boolean isPokktNetwork() {
        return false;
    }

    @Override
    public boolean verifyCampaignForAdConfig(AdConfig adConfig, boolean verifyCached) {
        return true;
    }

    private boolean sdkEnabled() {
        try {
            Class.forName("com.hyprmx.mediate.HyprMediate");
            Class.forName("com.hyprmx.mediate.HyprMediateListener");
            return true;
        } catch (Throwable ignore) {
            Logger.e(TAG + " SDK not found");
            return false;
        }
    }

    private class AdListener implements HyprMediateListenerWithInit {
        private AdConfig config;

        @Override
        public void hyprMediateCanShowAd(boolean available) {
            isAdAvailable = available;
            if (!isTimedOut) {
                isTimedOut = true;
                if (callback != null) {
                    if (isAdAvailable) {
                        callback.onSuccess(0.0);
                    } else {
                        callback.onFailure("No Ad Available");
                        adConfig = null;
                    }
                }
            }
        }

        @Override
        public void hyprMediateRewardDelivered(HyprMediateReward hyprMediateReward) {
            Logger.d(TAG + " Ad Rewarded");
            if (config.isRewarded) {
                rewardAmount = hyprMediateReward.virtualCurrencyAmount();
                AdManager.getInstance().adGratified(config, rewardAmount, adNetworkInfo);
            }
        }

        @Override
        public void hyprMediateErrorOccurred(HyprMediateError hyprMediateError) {
            Logger.d(TAG + " Ad Closed : Error");
            AdManager.getInstance().adClosed(config, adNetworkInfo);
            isAdAvailable = false;
            adConfig = null;
        }

        @Override
        public void hyprMediateStartedDisplaying() {
            Logger.d(TAG + " Ad Displayed");
            AdManager.getInstance().adDisplayed(config, adNetworkInfo);
        }

        @Override
        public void hyprMediateFinishedDisplaying() {
            Logger.d(TAG + " Ad Closed");
            AdManager.getInstance().adCompleted(config, adNetworkInfo);
            AdManager.getInstance().adClosed(config, adNetworkInfo);
            isAdAvailable = false;
            adConfig = null;
        }

        public void setAdConfig(AdConfig adConfig) {
            this.config = adConfig;
        }

        @Override
        public void hyprMediateInitializationComplete() {
            isInitialized = true;
        }
    }
}