package com.pokkt.thirdparty;

import android.content.Context;
import android.support.annotation.Keep;
import android.text.TextUtils;

import com.inmobi.ads.InMobiAdRequestStatus;
import com.inmobi.ads.InMobiInterstitial;
import com.inmobi.sdk.InMobiSdk;
import com.pokkt.PokktAds;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdFormat;
import com.pokkt.sdk.adnetworks.AdNetwork;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;

import org.json.JSONObject;

import java.util.HashMap;
import java.util.Map;

@Keep
public class InMobiNetwork implements AdNetwork {

    private static final String TAG = InMobiNetwork.class.getName();
    private boolean isInitialized = false;
    private AdNetworkInfo adNetworkInfo = null;
    private Map<Long, InMobiInterstitial> adMap = new HashMap<>();
    private Map<AdConfig, InMobiAdListener> listenerMap = new HashMap<>();
    private static final String ACCOUNT_ID_KEY = "AccountId";
    private static final String CURRENCY_KEY = "coins";
    private double amount = 0;
    private AdConfig inmobiAdConfig;

    @Override
    public void initNetwork(Context context, AdNetworkInfo info) {
        Logger.i(TAG + " init network called !");

        synchronized (InMobiNetwork.class) {
            if (isInitialized) {
                Logger.i(TAG + " init network already initialized !");
                return;
            }
//            if (!(context instanceof Activity)) {
//                Logger.i(TAG+" Init Configurations Error ! Needs Activity context");
//                return;
//            }
            if (!sdkEnabled()) {
                return;
            }
            if (PokktAds.Debugging.isEnabled()) {
                InMobiSdk.setLogLevel(InMobiSdk.LogLevel.DEBUG);
            } else {
                InMobiSdk.setLogLevel(InMobiSdk.LogLevel.NONE);
            }
            this.adNetworkInfo = info;
            if (extrasAreValid()) {
                String accountId = adNetworkInfo.getCustomData().get(ACCOUNT_ID_KEY);
                if (adNetworkInfo.getCustomData().containsKey(CURRENCY_KEY)) {
                    try {
                        amount = Double.parseDouble(adNetworkInfo.getCustomData().get(CURRENCY_KEY));
                    } catch (Exception e) {
                        Logger.printStackTrace(TAG + " Could not parse currency", e);
                    }
                }
                try {
                    InMobiSdk.init(context, accountId);
                    Logger.i(TAG + " init network initialized !!");
                    Logger.i(TAG + " SDK version " + InMobiSdk.getVersion());
                    isInitialized = true;
                } catch (Throwable e) {
                    Logger.printStackTrace(TAG + " Init Configurations Error !", e);
                }
            } else {
                Logger.i(TAG + " Init Configurations Error !");
            }
        }
    }

    @Override
    public AdNetworkInfo getAdNetworkInfo() {
        return adNetworkInfo;
    }

    private boolean sdkEnabled() {
        try {
            Class.forName("com.inmobi.ads.InMobiAdRequestStatus");
            Class.forName("com.inmobi.ads.InMobiInterstitial");
            Class.forName("com.inmobi.sdk.InMobiSdk");
            return true;
        } catch (Throwable ignore) {
            Logger.e(TAG + " SDK not found");
            return false;
        }
    }

    private boolean extrasAreValid() {
        return adNetworkInfo != null
                && adNetworkInfo.getCustomData() != null
                && adNetworkInfo.getCustomData().containsKey(ACCOUNT_ID_KEY);
    }

    private class Placement {
        String placementId = "";
        boolean isRewarded;
        boolean isNonRewarded;
    }

    private Placement getPlacement(AdConfig config) {
        Placement placement = new Placement();
        if (adNetworkInfo.getCustomData().containsKey("screens_mapping_data")) {
            try {
                JSONObject mappingObject = new JSONObject(adNetworkInfo.getCustomData().get("screens_mapping_data"));
                String mapData = mappingObject.optString(config.screenName);
                if (!TextUtils.isEmpty(mapData)) {
                    JSONObject mapObj = new JSONObject(mapData);
                    placement.placementId = mapObj.optString("network_screen");
                    placement.isRewarded = mapObj.optInt("incent") == 1;
                    placement.isNonRewarded = mapObj.optInt("non_incent") == 1;
                }
            } catch (Exception e) {
                Logger.printStackTrace(TAG + " Invalid Mapping Json", e);
            }
        }
        return placement;
    }

    @Override
    public void cacheAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
        Logger.i(TAG + " caching called ! ");
        if (inmobiAdConfig != null && !inmobiAdConfig.equals(config)) {
            callbacks.onFailure("Cache Ad Failed !");
            return;
        }

        //check if custom data has got the mapping for passed screen name
        try {
            if (inmobiAdConfig == null) {
                inmobiAdConfig = config.clone();
            }
            Placement placement = getPlacement(config);
            if (placement.placementId.length() > 0) {
                if (config.isRewarded == placement.isRewarded || !(config.isRewarded) == placement.isNonRewarded) {//todo
                    long placementId = Long.parseLong(placement.placementId);
                    if (adMap.containsKey(placementId)) {
                        callbacks.onSuccess(0.0);
                    } else {
                        startFetchingAd(config, placementId, callbacks);
                    }
                } else {
                    Logger.i(TAG + " Invalid Placement Id");
                    callbacks.onFailure("Invalid Placement Id: Caching Failed ");
                }
            }
            //if there is no mapping and developer directly passes placement id
            else {
                long placementId = Long.parseLong(config.screenName);
                if (adMap.containsKey(placementId)) {
                    callbacks.onSuccess(0.0);
                } else {
                    startFetchingAd(config, placementId, callbacks);
                }
            }
        } catch (Exception e) {
            inmobiAdConfig = null;
            Logger.printStackTrace(TAG + " Cache Ad failed", e);
            callbacks.onFailure("Caching Failed ");
        }
    }

    @Override
    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {
        Logger.i(TAG + " InMobi show called !");
        try {
            if (inmobiAdConfig == null || inmobiAdConfig.equals(config)) {
                if (inmobiAdConfig == null) {
                    inmobiAdConfig = config.clone();
                }
            }
            Placement placement = getPlacement(config);
            if (placement.placementId.length() == 0) {
                placement.placementId = config.screenName;
            }
            long placementValue = Long.parseLong(placement.placementId);
            InMobiInterstitial interstitial = adMap.get(placementValue);
            if (interstitial != null && interstitial.isReady()) {
                interstitial.show();
            } else {
                Logger.i(TAG + " No interstitial is currently available!!");
                requestFailed("Ad Not Available !", failureCallback);
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Play Ad Failed", e);
            requestFailed("Play Ad Failed !", failureCallback);
        }
    }

    @Override
    public void adClosed(AdConfig adConfig) {

    }

    @Override
    public void checkAdAvailability(AdConfig config, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        try {
            if (inmobiAdConfig == null || inmobiAdConfig.equals(config)) {
                Placement placement = getPlacement(config);
                if (placement.placementId.length() == 0) {
                    placement.placementId = config.screenName;
                }
                long placementValue = Long.parseLong(placement.placementId);
                InMobiInterstitial interstitial = adMap.get(placementValue);
                if (interstitial != null && interstitial.isReady()) {
                    Logger.i(TAG + " ad is available and ready !");
                    callbacks.onSuccess(null);
                    return;
                }
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        callbacks.onFailure("Ad Not Available.");
    }

    @Override
    public void loadBannerAd(AdConfig adConfig, BannerUnit bannerUnit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        callbacks.onFailure("Invalid Network !");
    }

    @Override
    public void destroyBannerAd(AdConfig adConfig) {

    }

    @Override
    public void notifyCachingTimeout(AdConfig adConfig) {
        if (listenerMap.get(adConfig) != null) {
            listenerMap.get(adConfig).notifyTimeout();
        }
    }

    @Override
    public double getAdVC(AdConfig adConfig) {
        if (adConfig.isRewarded) {
            return amount;
        } else {
            return 0;
        }
    }

    @Override
    public boolean supportsAdConfig(AdConfig config) {
        return AdFormat.INTERSTITIAL == config.adFormat;
    }

    @Override
    public boolean isInitialised() {
        return isInitialized;
    }

    @Override
    public boolean isPokktNetwork() {
        return false;
    }

    @Override
    public boolean verifyCampaignForAdConfig(AdConfig adConfig, boolean verifyCached) {
        return true;
    }

    private void requestFailed(String msg, Callbacks.WithOnlyFailure<String> failureCallback) {
        inmobiAdConfig = null;
        failureCallback.onFailure(msg);
    }

    private void startFetchingAd(AdConfig config, long placement, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {

        try {
            InMobiAdListener listener = new InMobiAdListener(config, placement, callbacks);
            InMobiInterstitial interstitial = new InMobiInterstitial(AdManager.getInstance().getAssignedActivity(), placement, listener);
            interstitial.load();
            listenerMap.put(config, listener);
        } catch (Throwable e) {
            inmobiAdConfig = null;
            Logger.printStackTrace(TAG + " Cache Ad Failed ", e);
            callbacks.onFailure("Caching Failed ");
            listenerMap.remove(config);
        }
    }

    private class InMobiAdListener implements InMobiInterstitial.InterstitialAdListener2 {
        private final long placement;
        private AdConfig adConfig;
        private boolean isTimedOut = false;
        private Callbacks.WithSuccessAndFailure<Double, String> callback;

        InMobiAdListener(AdConfig config, long value, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
            this.adConfig = config;
            this.placement = value;
            this.callback = callbacks;
        }

        void notifyTimeout() {
            isTimedOut = true;
            String reason = " Time Out In Fetching Ad";
            Logger.e(TAG + reason);
            listenerMap.remove(adConfig);
        }

        @Override
        public void onAdRewardActionCompleted(InMobiInterstitial inMobiInterstitial, Map<Object, Object> map) {
            Logger.i(TAG + " onAdRewardActionCompleted !");
            if (adConfig.isRewarded) {
                AdManager.getInstance().adCompleted(adConfig, adNetworkInfo);
                try {
                    amount = Double.parseDouble((String) map.get(CURRENCY_KEY));
                } catch (Exception e) {
                    Logger.printStackTrace(TAG + " Could not parse currency", e);
                }
                AdManager.getInstance().adGratified(adConfig, amount, adNetworkInfo);
            }
        }

        @Override
        public void onAdDisplayFailed(InMobiInterstitial inMobiInterstitial) {
            Logger.i(TAG + " onAdDisplayFailed !");
            AdManager.getInstance().adClosed(adConfig, adNetworkInfo);
            inmobiAdConfig = null;
            adMap.remove(placement);
        }

        @Override
        public void onAdWillDisplay(InMobiInterstitial inMobiInterstitial) {

        }

        @Override
        public void onAdDisplayed(InMobiInterstitial inMobiInterstitial) {
            Logger.i(TAG + " onAdDisplayed !");
            AdManager.getInstance().adDisplayed(adConfig, adNetworkInfo);
        }

        @Override
        public void onAdDismissed(InMobiInterstitial inMobiInterstitial) {
            Logger.i(TAG + " onAdDismissed !");
            AdManager.getInstance().adClosed(adConfig, adNetworkInfo);
            inmobiAdConfig = null;
            adMap.remove(placement);
        }

        @Override
        public void onAdInteraction(InMobiInterstitial inMobiInterstitial, Map<Object, Object> map) {
            Logger.i(TAG + " onAdInteraction !");
        }

        @Override
        public void onAdLoadSucceeded(InMobiInterstitial inMobiInterstitial) {
            Logger.i(TAG + " onAdLoadSucceeded !");
            if (!isTimedOut) {
                isTimedOut = true;
                listenerMap.remove(adConfig);
                callback.onSuccess(0.0);
            }
            adMap.put(placement, inMobiInterstitial);
        }

        @Override
        public void onAdLoadFailed(InMobiInterstitial inMobiInterstitial, InMobiAdRequestStatus inMobiAdRequestStatus) {
            Logger.i(TAG + " Request status: " + inMobiAdRequestStatus.getMessage() + "code: " + inMobiAdRequestStatus.getStatusCode());
            if (!isTimedOut) {
                inmobiAdConfig = null;
                isTimedOut = true;
                listenerMap.remove(adConfig);
                callback.onFailure(inMobiAdRequestStatus.getMessage());
            }
        }

        @Override
        public void onAdReceived(InMobiInterstitial inMobiInterstitial) {
            Logger.i(TAG + " onAdReceived !");
        }

        @Override
        public void onUserLeftApplication(InMobiInterstitial inMobiInterstitial) {
            AdManager.getInstance().adSkipped(adConfig, adNetworkInfo);
        }
    }
}