package com.pokkt.thirdparty;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.os.Build;
import android.os.Bundle;
import android.support.annotation.Keep;
import android.support.annotation.RequiresApi;
import android.text.TextUtils;

import com.ironsource.mediationsdk.IronSource;
import com.ironsource.mediationsdk.logger.IronSourceError;
import com.ironsource.mediationsdk.model.Placement;
import com.ironsource.mediationsdk.sdk.InterstitialListener;
import com.ironsource.mediationsdk.sdk.RewardedVideoListener;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdFormat;
import com.pokkt.sdk.adnetworks.AdNetwork;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;

@RequiresApi(api = Build.VERSION_CODES.ICE_CREAM_SANDWICH)
@Keep
public class IronSourceNetwork implements AdNetwork, Application.ActivityLifecycleCallbacks {

    private String activityClassName = null;
    private Context context = null;

    private static final String TAG = IronSourceNetwork.class.getName();
    private static final String APP_KEY = "appKey";
    private boolean isInitialized = false;
    private AdConfig adConfig;
    private AdConfig adConfigInterstitial;
    private AdNetworkInfo adNetworkInfo;
    private String currentScreen = "";
    private String currentInterstitialScreen = "";
    private double rewardAmount;

    @Override
    public void initNetwork(Context context, AdNetworkInfo info) {

        synchronized (IronSourceNetwork.class) {
            if (isInitialized) {
                Logger.i(TAG + " init network already initialized !");
                return;
            }

            if (!(context instanceof Activity)) {
                Logger.i(TAG + " Init Configurations Error. Needs Activity context");
                return;
            }

            this.context = context;
            this.adNetworkInfo = info;

            if (!sdkEnabled()) {
                Logger.i(TAG + " Init Configurations Error");
            }
            this.activityClassName = ((Activity) context).getClass().getName();
            Activity launcherActivity = ((Activity) context);
            launcherActivity.getApplication().registerActivityLifecycleCallbacks(this);

            String appKey = "";
            if (adNetworkInfo != null && adNetworkInfo.getCustomData() != null && adNetworkInfo.getCustomData().containsKey(APP_KEY)) {
                appKey = adNetworkInfo.getCustomData().get(APP_KEY);
                if (adNetworkInfo.getCustomData().containsKey("amount")) {
                    try {
                        rewardAmount = Double.parseDouble(adNetworkInfo.getCustomData().get("amount"));
                    } catch (Throwable e) {
                        Logger.printStackTrace(TAG + " Could not parse amount", e);
                        //do nothing
                    }
                }
            } else {
                Logger.i(TAG + " Init Configurations Error");
                return;
            }

            String userId = AdManager.getInstance().getThirdPartyUserId();

            try {
                if (TextUtils.isEmpty(userId)) {
                    userId = IronSource.getAdvertiserId(context);
                }
                IronSource.setUserId(userId);

                IronSource.init((Activity) context, appKey, IronSource.AD_UNIT.INTERSTITIAL, IronSource.AD_UNIT.REWARDED_VIDEO);

                IronSource.setRewardedVideoListener(mRewardedVideoListener);
                IronSource.setInterstitialListener(interstitialListener);
                isInitialized = true;
            } catch (Throwable e) {
                Logger.printStackTrace(TAG + "  SDK Init Error", e);
                isInitialized = false;
            }
        }
    }

    @Override
    public AdNetworkInfo getAdNetworkInfo() {
        return adNetworkInfo;
    }

    @Override
    public void cacheAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
        Logger.i(TAG + " Cache Ad Called");

        try {

            if (AdFormat.VIDEO == config.adFormat) {
                if (currentScreen.length() != 0 && !currentScreen.equalsIgnoreCase(config.screenName)) {
                    callbacks.onFailure("Ad Caching Failed !");
                    return;
                }

                if (IronSource.isRewardedVideoAvailable()) {
                    this.adConfig = config;
                    currentScreen = config.screenName;

                    callbacks.onSuccess(0.0);
                } else {
                    callbacks.onFailure("Ad Caching Failed !");
                }
            } else {
                if (currentInterstitialScreen.length() != 0 && !currentInterstitialScreen.equalsIgnoreCase(config.screenName)) {
                    callbacks.onFailure("Ad Caching Failed !");
                    return;
                }

                if (IronSource.isInterstitialReady()) {
                    this.adConfigInterstitial = config;
                    currentInterstitialScreen = config.screenName;
                    callbacks.onSuccess(0.0);
                } else {
                    callbacks.onFailure("Ad Caching Failed !");
                }
            }
        } catch (Throwable e) {
            if (AdFormat.VIDEO == config.adFormat) {
                currentScreen = "";
            } else if (AdFormat.INTERSTITIAL == config.adFormat) {
                currentInterstitialScreen = "";
            }
            Logger.printStackTrace(TAG + " Cache Ad Failed ", e);
            callbacks.onFailure("Ad Caching Failed");
        }
    }

    @Override
    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {
        Logger.i(TAG + " Play Ad Called");

        if (AdFormat.VIDEO == config.adFormat) {
            if (!config.isRewarded) {
                Logger.i(TAG + " Non rewarded video is requested !");
                requestFailed("Invalid Ad Config !", failureCallback);
                return;
            }
            if (currentScreen.length() == 0 || currentScreen.equalsIgnoreCase(config.screenName)) {
                this.adConfig = config;
                try {
                    if (IronSource.isRewardedVideoAvailable()) {
                        IronSource.showRewardedVideo();
                    } else {
                        Logger.i(TAG + " Rewarded Ad is currently not available!!");
                        currentScreen = "";
                        requestFailed("Ad Not Available !", failureCallback);
                    }
                } catch (Throwable e) {
                    currentScreen = "";
                    requestFailed("Ad Not Available !", failureCallback);
                }
            } else {
                requestFailed("Ad Not Available !", failureCallback);
            }
        } else {
            if (config.isRewarded) {
                Logger.i(TAG + " Rewarded interstitial is requested !");
                requestFailed("Invalid Ad Config !", failureCallback);
                return;
            }
            if (currentInterstitialScreen.length() == 0 || currentInterstitialScreen.equalsIgnoreCase(config.screenName)) {
                this.adConfigInterstitial = config;
                try {
                    if (IronSource.isInterstitialReady()) {
                        IronSource.showInterstitial();
                    } else {
                        Logger.i(TAG + " Ad is currently not available!!");
                        currentInterstitialScreen = "";
                        requestFailed("Ad Not Available !", failureCallback);
                    }
                } catch (Throwable e) {
                    currentInterstitialScreen = "";
                    requestFailed("Ad Not Available !", failureCallback);
                }
            } else {
                requestFailed("Ad Not Available !", failureCallback);
            }
        }
    }

    @Override
    public void adClosed(AdConfig adConfig) {

    }

    private void requestFailed(String msg, Callbacks.WithOnlyFailure<String> failureCallback) {
        failureCallback.onFailure(msg);
    }

    @Override
    public void checkAdAvailability(AdConfig adConfig, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        try {
            if (AdFormat.VIDEO == adConfig.adFormat) {
                if (currentScreen.length() == 0) {//caching is not called or caching cycle completed
                    if (IronSource.isRewardedVideoAvailable()) {
                        callbacks.onSuccess(null);
                        return;
                    }
                } else {
                    if (currentScreen.equalsIgnoreCase(adConfig.screenName)) {
                        if (IronSource.isRewardedVideoAvailable()) {
                            callbacks.onSuccess(null);
                            return;
                        }
                    }
                }
            } else {
                if (currentInterstitialScreen.length() == 0) {//caching is not called or caching cycle completed
                    if (IronSource.isInterstitialReady()) {
                        callbacks.onSuccess(null);
                        return;
                    }
                } else {
                    if (currentInterstitialScreen.equalsIgnoreCase(adConfig.screenName)) {
                        if (IronSource.isInterstitialReady()) {
                            callbacks.onSuccess(null);
                            return;
                        }
                    }
                }
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        callbacks.onFailure("Ad not available");
    }

    @Override
    public void loadBannerAd(AdConfig adConfig, BannerUnit bannerUnit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        callbacks.onFailure("Invalid Network !");
    }

    @Override
    public void destroyBannerAd(AdConfig adConfig) {

    }

    @Override
    public void notifyCachingTimeout(AdConfig adConfig) {

    }

    @Override
    public double getAdVC(AdConfig adConfig) {
        if (adConfig.isRewarded) {
            return rewardAmount;
        } else {
            return 0;
        }
    }

    @Override
    public boolean supportsAdConfig(AdConfig config) {
        if (AdFormat.BRANDING == config.adFormat) {
            return false;
        }

        if (AdFormat.INTERSTITIAL == config.adFormat) {
            if (config.isRewarded) {
                return false;
            }
        }
        if (AdFormat.VIDEO == config.adFormat) {
            if (!(config.isRewarded)) {
                return false;
            }
        }
        if (AdFormat.BANNER == config.adFormat) {
            return false;
        }
        return true;
    }

    @Override
    public boolean isInitialised() {
        return isInitialized;
    }

    @Override
    public boolean isPokktNetwork() {
        return false;
    }

    @Override
    public boolean verifyCampaignForAdConfig(AdConfig adConfig, boolean verifyCached) {
        return true;
    }

    @Override
    public void onActivityCreated(Activity activity, Bundle bundle) {

    }

    @Override
    public void onActivityStarted(Activity activity) {
    }

    @Override
    public void onActivityResumed(Activity activity) {
        try {
            if (activity.getClass().getName().equals(context.getClass().getName())) {
                Logger.i(TAG + " Resumed ironsource");
                IronSource.onResume(((Activity) context));
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG, e);
        }
    }

    @Override
    public void onActivityPaused(Activity activity) {
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Logger.i(TAG + " Paused ironsource");
                IronSource.onPause(activity);
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG, e);
        }
    }

    @Override
    public void onActivityStopped(Activity activity) {
    }

    @Override
    public void onActivitySaveInstanceState(Activity activity, Bundle bundle) {

    }

    @Override
    public void onActivityDestroyed(Activity activity) {
    }

    //Rewarded video listener section
    private RewardedVideoListener mRewardedVideoListener = new RewardedVideoListener() {

        @Override
        public void onRewardedVideoAdOpened() {
            //Invoked when the RewardedVideo ad view has opened.
            Logger.i(TAG + " onRewardedVideoAdOpened");
            AdManager.getInstance().adDisplayed(adConfig, adNetworkInfo);
        }

        @Override
        public void onRewardedVideoAdClosed() {
            //Invoked when the user is about to return to the application after closing the rewarded video
            Logger.i(TAG + " onRewardedVideoAdClosed");
            AdManager.getInstance().adClosed(adConfig, adNetworkInfo);
            currentScreen = "";
        }

        @Override
        public void onRewardedVideoAvailabilityChanged(boolean b) {
            Logger.i(TAG + " onRewardedVideoAvailabilityChanged " + b);
        }

        @Override
        public void onRewardedVideoAdStarted() {
            Logger.i(TAG + " onRewardedVideoAdStarted");
        }

        @Override
        public void onRewardedVideoAdEnded() {
            Logger.i(TAG + " onVideoEnd");
            AdManager.getInstance().adCompleted(adConfig, adNetworkInfo);
            AdManager.getInstance().adClosed(adConfig, adNetworkInfo);
        }

        @Override
        public void onRewardedVideoAdRewarded(Placement placement) {
            // Invoked when the user get rewarded from watching the video.
            // @param - Placement

            // Get the information of the placement
            if (adConfig.isRewarded) {
                String rewardName = placement.getRewardName();
                int rewardAmount = placement.getRewardAmount();
                Logger.i(TAG + " onRewardedVideoAdRewarded: rewardName=" + rewardName + ", rewardAmount=" + rewardAmount);
                AdManager.getInstance().adGratified(adConfig, rewardAmount, adNetworkInfo);
            }
        }

        @Override
        public void onRewardedVideoAdShowFailed(IronSourceError ironSourceError) {
            Logger.i(TAG + " " + ironSourceError.getErrorMessage());
            AdManager.getInstance().adClosed(adConfig, adNetworkInfo);
            currentScreen = "";
        }
    };

    private InterstitialListener interstitialListener = new InterstitialListener() {

        @Override
        public void onInterstitialAdReady() {
            Logger.i(TAG + " onInterstitialAdReady");
        }

        @Override
        public void onInterstitialAdLoadFailed(IronSourceError ironSourceError) {
            Logger.i(TAG + " onInterstitialAdLoadFailed");
        }

        @Override
        public void onInterstitialAdOpened() {
            Logger.i(TAG + " onInterstitialAdOpened");
        }

        @Override
        public void onInterstitialAdClosed() {
            Logger.i(TAG + " onInterstitialAdClosed");
            AdManager.getInstance().adCompleted(adConfigInterstitial, adNetworkInfo);
            AdManager.getInstance().adClosed(adConfigInterstitial, adNetworkInfo);
            currentInterstitialScreen = "";
        }

        @Override
        public void onInterstitialAdShowSucceeded() {
            Logger.i(TAG + " onInterstitialShowSuccess");
            AdManager.getInstance().adDisplayed(adConfigInterstitial, adNetworkInfo);
        }

        @Override
        public void onInterstitialAdShowFailed(IronSourceError ironSourceError) {
            Logger.i(TAG + " onInterstitialShowFail");
            AdManager.getInstance().adClosed(adConfigInterstitial, adNetworkInfo);
            currentInterstitialScreen = "";
        }

        @Override
        public void onInterstitialAdClicked() {
            Logger.i(TAG + " onInterstitialAdClicked");
        }
    };

    private boolean sdkEnabled() {
        try {
            Class.forName("com.ironsource.mediationsdk.IronSource");
            Class.forName("com.ironsource.mediationsdk.model.Placement");
            Class.forName("com.ironsource.mediationsdk.sdk.InterstitialListener");
            Class.forName("com.ironsource.mediationsdk.sdk.RewardedVideoListener");
            return true;
        } catch (Throwable ignore) {
            Logger.e(TAG + "  SDK not found");
            return false;
        }
    }
}
