package com.pokkt.thirdparty.LoopMe;

import android.app.Activity;
import android.content.Context;

import com.loopme.AdController;
import com.loopme.LoopMeBanner;
import com.loopme.LoopMeBannerView;
import com.loopme.common.LoopMeError;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.banners.PokktBannerView;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;

import java.util.List;

public class LoopMeBannerManager {

    private static final String TAG = LoopMeBannerManager.class.getName();
    private final String appKey;

    private LoopMeBanner mBanner;
    private String callingActivity;
    private Callbacks.WithSuccessAndFailure<AdCampaign, String> callback;
    private boolean isTimedOut;
    private boolean isStaleBanner = false;

    public LoopMeBannerManager(String appkey) {
        this.appKey = appkey;
    }

    public void loadBanner(final AdConfig config, final BannerUnit unit, final Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks, final Context context) {
        Logger.d(TAG + " Load Banner called !");

        if (mBanner != null && mBanner.isViewBinded()) {
            callbacks.onFailure(" Banner Already showing");
            return;
        }
        final LoopMeBannerListener listener = new LoopMeBannerListener();
        this.callback = callbacks;

        ((Activity) context).runOnUiThread(new Runnable() {
            @Override
            public void run() {
                try {
                    isStaleBanner = false;
                    LoopMeBannerView adView = new LoopMeBannerView(context);
                    mBanner = LoopMeBanner.getInstance(appKey, (Activity) context);
                    mBanner.setListener(listener);
                    mBanner.bindView(adView);
                    mBanner.load();
                    listener.setbannerAd(adView);
                    listener.setBannerAdView(unit);
                    isTimedOut = false;
                } catch (Throwable e) {
                    Logger.printStackTrace(TAG + " Load banner failed", e);
                    isTimedOut = true;
                    callbacks.onFailure(" Ad Caching Failed");
                }
            }
        });
    }

    public void resume() {

        if (mBanner != null) {
            Logger.d(TAG + " Banner resumed");
            mBanner.resume();
            if(isStaleBanner) {
                Logger.d(TAG + " Banner destroy");
                mBanner.destroy();
                mBanner = null;
                isStaleBanner = false;
            }
        }
    }

    public void pause() {
        if (mBanner != null) {
            Logger.d(TAG + " Banner Paused");
            mBanner.pause();
        }
    }

    public void destroy() {
        //we will not destroy the banner if in fullscreen mode, rather mark the banner as stale so that
        //when the banner comes back to normal mode (in resume) we will destroy it.
        if (mBanner != null) {
            if(mBanner.getAdController().getCurrentDisplayMode() == 100) {//102 is for fullscreen and 100 for normal
                Logger.d(TAG + " Banner destroyed");
                mBanner.destroy();
                mBanner = null;
            } else {
                isStaleBanner = true;
            }
        }
    }

    public void setCallingActivity(String callingActivity) {
        this.callingActivity = callingActivity;
    }

    public String getCallingActivity() {
        return callingActivity;
    }

    public void destroyBanner(AdConfig adConfig) {
        destroy();
    }

    public void notifyCachingTimeout() {
        isTimedOut = true;
        String reason = " Time Out In Fetching Ad";
        callingActivity = "";
        Logger.d(TAG + reason);
    }

    private class LoopMeBannerListener implements LoopMeBanner.Listener {
        private LoopMeBannerView loopMeBannerView = null;
        private BannerUnit bannerUnit;

        void setbannerAd(LoopMeBannerView banner) {
            this.loopMeBannerView = banner;
        }

        LoopMeBannerListener() {
        }

        void setBannerAdView(BannerUnit unit) {
            this.bannerUnit = unit;
        }

        @Override
        public void onLoopMeBannerLoadSuccess(LoopMeBanner banner) {
            Logger.d(TAG + " onAdLoadSucceeded !");

            if (!isTimedOut) {

                isTimedOut = true;
                List<PokktBannerView> containerList = bannerUnit.getContainerList();
//                for (int i = 0; i < containerList.size(); i++) {
                if (containerList.size() > 0) {
                    PokktBannerView container = containerList.get(0);

                    if (container.getChildAt(0) == null) {
                        container.addView(loopMeBannerView);
                    } else {
                        container.removeView(container.getChildAt(0));
                        container.addView(loopMeBannerView);
                    }
//                }
                    banner.show();
                    callback.onSuccess(null);
                } else {
                    destroy();
                }
            }
        }

        @Override
        public void onLoopMeBannerLoadFail(LoopMeBanner banner, LoopMeError error) {
            if (!isTimedOut) {
                Logger.d(TAG + " banner load failed " + error.getMessage());
                callingActivity = "";
                isTimedOut = true;
                callback.onFailure("Ad Load Failed " + error.getMessage());
            }
        }

        @Override
        public void onLoopMeBannerShow(LoopMeBanner banner) {
            Logger.d(TAG + " banner shown");
        }

        @Override
        public void onLoopMeBannerHide(LoopMeBanner banner) {
            Logger.d(TAG + " banner hidden");
        }

        @Override
        public void onLoopMeBannerClicked(LoopMeBanner banner) {
            Logger.d(TAG + " banner clicked");
        }

        @Override
        public void onLoopMeBannerLeaveApp(LoopMeBanner banner) {
            Logger.d(TAG + " banner leave app");
            banner.destroy();
        }

        @Override
        public void onLoopMeBannerVideoDidReachEnd(LoopMeBanner banner) {
            Logger.d(TAG + " banner video ended");
        }

        @Override
        public void onLoopMeBannerExpired(LoopMeBanner banner) {
            Logger.d(TAG + " banner expired");
        }
    }
}