package com.pokkt.thirdparty;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.os.Bundle;
import android.support.annotation.Keep;

import com.loopme.LoopMeBanner;
import com.loopme.LoopMeInterstitial;
import com.pokkt.PokktAds;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdFormat;
import com.pokkt.sdk.adnetworks.AdNetwork;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;
import com.pokkt.thirdparty.LoopMe.LoopMeBannerManager;
import com.pokkt.thirdparty.LoopMe.LoopMeInterstitialManager;

@Keep
public class LoopMeNetwork implements AdNetwork, Application.ActivityLifecycleCallbacks {

    private static final String TAG = LoopMeNetwork.class.getName();
    private Context context = null;
    private boolean isInitialized = false;
    private AdNetworkInfo adNetworkInfo = null;
    private static final String APP_KEY = "AppKey";
    private LoopMeBannerManager bannerManager;
    private LoopMeInterstitialManager interstitialManager;

    @Override
    public void initNetwork(Context context, AdNetworkInfo info) {
        Logger.d(TAG+ " init network called!!");

        synchronized (LoopMeNetwork.class) {
            if (isInitialized) {
                Logger.d(TAG+ " init network already initialized !");
                return;
            }
            if (!sdkEnabled()) {
                Logger.d(TAG+ " Init Error!");
                return;
            }
            if (!(context instanceof Activity)) {
                Logger.d(TAG+ " Init Configurations Error . Needs Activity context");
                return;
            }
            this.context = context;
            this.adNetworkInfo = info;
            ((Activity) context).getApplication().registerActivityLifecycleCallbacks(this);
            if (extrasAreValid()) {
                String appKey = adNetworkInfo.getCustomData().get(APP_KEY);
                if (PokktAds.Debugging.isEnabled()) {
                    appKey = LoopMeBanner.TEST_MPU_BANNER;
                }
                bannerManager = new LoopMeBannerManager(appKey);
                interstitialManager = new LoopMeInterstitialManager(adNetworkInfo, context);
            } else {
                Logger.d(TAG+ " Init Configurations Error.");
            }
            isInitialized = true;
        }
    }

    private boolean sdkEnabled() {
        try {
            Class.forName("com.loopme.LoopMeInterstitial");
            Class.forName("com.loopme.LoopMeBanner");
            return true;
        } catch (Throwable ignore) {
            Logger.e(TAG+ " SDK not found");
            return false;
        }
    }

    private boolean extrasAreValid() {
        return adNetworkInfo != null
                && adNetworkInfo.getCustomData() != null
                && adNetworkInfo.getCustomData().containsKey(APP_KEY);
    }

    @Override
    public void cacheAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
        if (AdFormat.INTERSTITIAL == config.adFormat) {
            String appKey = adNetworkInfo.getCustomData().get(APP_KEY);
            if (PokktAds.Debugging.isEnabled()) {
                appKey = LoopMeInterstitial.TEST_LAND_INTERSTITIAL;
            }
            interstitialManager.fetchAd(config, callbacks, appKey);
        } else {
            callbacks.onFailure("Invalid Ad Format. Ad Not Available.");
        }
    }

    @Override
    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {
        if (AdFormat.INTERSTITIAL == config.adFormat) {
            interstitialManager.showAd(config, failureCallback);
        } else {
            failureCallback.onFailure("Invalid Ad Format. Ad Not Available.");
        }
    }

    @Override
    public void adClosed(AdConfig adConfig) {

    }

    @Override
    public void checkAdAvailability(AdConfig config, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        if (AdFormat.INTERSTITIAL == config.adFormat) {
            if (interstitialManager.isInterstitialAvailable(config)) {
                callbacks.onSuccess(null);
                return;
            }
        } else {
            callbacks.onFailure("Invalid Ad Format. Ad Not Available.");
            return;
        }
        callbacks.onFailure("Ad Not Available.");
    }

    @Override
    public void loadBannerAd(AdConfig adConfig, BannerUnit bannerUnit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        if ((context instanceof Activity)) {
            bannerManager.setCallingActivity(((Activity) context).getClass().getName());
            bannerManager.loadBanner(adConfig, bannerUnit, callbacks, context);
        } else {
            callbacks.onFailure("Activity Needed.");
        }
    }


    @Override
    public AdNetworkInfo getAdNetworkInfo() {
        return adNetworkInfo;
    }

    @Override
    public void destroyBannerAd(AdConfig adConfig) {
        bannerManager.destroyBanner(adConfig);
    }

    @Override
    public void notifyCachingTimeout(AdConfig adConfig) {
        if (AdFormat.INTERSTITIAL == adConfig.adFormat) {
            interstitialManager.notifyCachingTimeout();
        } else if (AdFormat.BANNER == adConfig.adFormat) {
            bannerManager.notifyCachingTimeout();
        }
    }

    @Override
    public double getAdVC(AdConfig adConfig) {
        return 0;
    }

    @Override
    public boolean supportsAdConfig(AdConfig config) {
        return AdFormat.INTERSTITIAL == config.adFormat || AdFormat.BANNER == config.adFormat;
    }

    @Override
    public boolean isInitialised() {
        return isInitialized;
    }

    @Override
    public boolean isPokktNetwork() {
        return false;
    }

    @Override
    public boolean verifyCampaignForAdConfig(AdConfig adConfig, boolean verifyCached) {
        return true;
    }

    @Override
    public void onActivityCreated(Activity activity, Bundle savedInstanceState) {

    }

    @Override
    public void onActivityStarted(Activity activity) {

    }

    @Override
    public void onActivityResumed(Activity activity) {
        if (activity.getClass().getName().equals(bannerManager.getCallingActivity())) {
            bannerManager.resume();
        }
    }

    @Override
    public void onActivityPaused(Activity activity) {
        if (activity.getClass().getName().equals(bannerManager.getCallingActivity())) {
            bannerManager.pause();
        }
    }

    @Override
    public void onActivityStopped(Activity activity) {

    }

    @Override
    public void onActivitySaveInstanceState(Activity activity, Bundle outState) {

    }

    @Override
    public void onActivityDestroyed(Activity activity) {
        if (activity.getClass().getName().equals(bannerManager.getCallingActivity())) {
            bannerManager.destroy();
        }
    }
}