package com.pokkt.thirdparty.Millennial;

import android.view.ViewGroup;

import com.millennialmedia.InlineAd;
import com.millennialmedia.InlineAd.AdSize;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.banners.PokktBannerView;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;

import java.util.HashMap;
import java.util.Map;

public class MillennialBannerManager {

    private static final String TAG = MillennialBannerManager.class.getName();
    private Map<AdConfig, InlineAd> bannerMap = new HashMap<AdConfig, InlineAd>();
    private Map<AdConfig, BannerListener> listenerMap = new HashMap<AdConfig, BannerListener>();
    private Callbacks.WithSuccessAndFailure<AdCampaign, String> callback;


    public void loadBanner(AdConfig config, final BannerUnit unit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks, String placementId) {
        Logger.i(TAG + " Load Banner called !");

        if (bannerMap.get(config) != null) {
            callbacks.onFailure(" Banner Already showing");
            return;
        }
        BannerListener listener = new BannerListener(config);
        this.callback = callbacks;

        try {
            PokktBannerView container = unit.getContainerList().get(0);
            InlineAd inlineAd = InlineAd.createInstance(placementId, (ViewGroup) container);
            inlineAd.setListener(listener);
            final InlineAd.InlineAdMetadata inlineAdMetadata = new InlineAd.InlineAdMetadata().
                    setAdSize(getBannerSize(config));
            inlineAd.request(inlineAdMetadata);
            listenerMap.put(config, listener);
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Load banner failed", e);
            listenerMap.remove(config);
            callbacks.onFailure("Banner Caching Failed");
        }
    }

    private AdSize getBannerSize(AdConfig view) {
        int height = view.sHeight;
        int width = view.sWidth;
        if (height >= AdSize.BANNER.height && width >= AdSize.BANNER.width)
            return AdSize.BANNER;
        if (height >= AdSize.FULL_BANNER.height && width >= AdSize.FULL_BANNER.width)
            return AdSize.FULL_BANNER;
        if (height >= AdSize.LARGE_BANNER.height && width >= AdSize.LARGE_BANNER.width)
            return AdSize.LARGE_BANNER;
        if (height >= AdSize.LEADERBOARD.height && width >= AdSize.LEADERBOARD.width)
            return AdSize.LEADERBOARD;
        if (height >= AdSize.MEDIUM_RECTANGLE.height && width >= AdSize.MEDIUM_RECTANGLE.width)
            return AdSize.MEDIUM_RECTANGLE;
        if (height >= AdSize.SMART_BANNER.height && width >= AdSize.SMART_BANNER.width)
            return AdSize.SMART_BANNER;
        return AdSize.SMART_BANNER;
    }

    public void destroyBanner(AdConfig config) {
        if (bannerMap.get(config) != null) {
            Logger.i(TAG + " Banner destroyed");
            bannerMap.remove(config);
            listenerMap.remove(config);
        }
    }

    public void notifyCachingTimeout(AdConfig config) {
        String reason = " Time Out In Fetching Banner";
        Logger.e(TAG + reason);
        listenerMap.get(config).notifyTimeout();

    }

    private class BannerListener implements InlineAd.InlineListener {

        private final AdConfig config;
        private boolean isTimedOut;

        BannerListener(AdConfig config) {
            this.config = config;
        }

        @Override
        public void onRequestSucceeded(InlineAd inlineAd) {
            Logger.i(TAG + " Banner Fetched !");
            if (!isTimedOut) {
                isTimedOut = true;
                callback.onSuccess(null);
                bannerMap.put(config, inlineAd);
            }
        }

        @Override
        public void onRequestFailed(InlineAd inlineAd, InlineAd.InlineErrorStatus inlineErrorStatus) {
            if (!isTimedOut) {
                Logger.i(TAG + " Banner load failed " + inlineErrorStatus.getDescription());
                isTimedOut = true;
                callback.onFailure("Banner Load Failed " + inlineErrorStatus.getDescription());
            }
        }

        @Override
        public void onClicked(InlineAd inlineAd) {

        }

        @Override
        public void onResize(InlineAd inlineAd, int i, int i1) {

        }

        @Override
        public void onResized(InlineAd inlineAd, int i, int i1, boolean b) {

        }

        @Override
        public void onExpanded(InlineAd inlineAd) {

        }

        @Override
        public void onCollapsed(InlineAd inlineAd) {

        }

        @Override
        public void onAdLeftApplication(InlineAd inlineAd) {
            destroyBanner(config);
        }

        void notifyTimeout() {
            isTimedOut = true;
        }
    }
}