package com.pokkt.thirdparty.Millennial;

import android.content.Context;
import android.text.TextUtils;

import com.millennialmedia.InterstitialAd;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.debugging.Logger;

import java.util.HashMap;
import java.util.Map;

public class MillennialInterstitialManager {

    private static final String TAG = MillennialInterstitialManager.class.getName();

    private AdNetworkInfo adNetworkInfo;
    private Map<AdConfig, InterstitialAd> adMap = new HashMap<>();
    private AdConfig mmAdConfig;
    private Callbacks.WithSuccessAndFailure<Double, String> callback;
    private boolean isTimedOut;

    public MillennialInterstitialManager(AdNetworkInfo network) {
        this.adNetworkInfo = network;
    }

    public void fetchAd(final AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks, Context context, String placementId) {
        Logger.i(TAG + " cache Interstitial called !");

        AdListener listener = new AdListener(config);
        try {
            if (this.mmAdConfig != null && !(config.equals(this.mmAdConfig))) {
                callbacks.onFailure("Fetch Ad Failed. Wrong AdConfig.");
                return;
            }
            if (this.mmAdConfig == null) {
                this.mmAdConfig = config.clone();
            }
            if (TextUtils.isEmpty(placementId)) {
                callbacks.onFailure("Ad Caching Failed");
                return;
            }
            InterstitialAd ad = adMap.get(config);
            if (ad != null && ad.isReady() && !(ad.hasExpired())) {
                callbacks.onSuccess(0.0);
                return;
            }

            this.callback = callbacks;

            final InterstitialAd interstitialAd = InterstitialAd.createInstance(placementId);
            interstitialAd.setListener(listener);
            interstitialAd.load(context, null);
            isTimedOut = false;

        } catch (Throwable e) {
            mmAdConfig = null;
            isTimedOut = true;
            Logger.printStackTrace(TAG + " Cache Ad Failed", e);
            callbacks.onFailure("Cache Ad Failed");
        }
    }

    public void showAd(final AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback, Context context, String placementId) {
        Logger.i(TAG + " Show interstitial called !");

        try {
            if (TextUtils.isEmpty(placementId)) {
                adMap.remove(config);
                requestFailed("interstitial Not Available !", failureCallback);
                return;
            }
            if (this.mmAdConfig != null && !(config.equals(this.mmAdConfig))) {
                failureCallback.onFailure("Fetch interstitial Failed. Wrong AdConfig.");
                return;
            }
            if (this.mmAdConfig == null) {
                this.mmAdConfig = config.clone();
            }
            InterstitialAd interstitialAd = adMap.get(config);
            if (interstitialAd != null && interstitialAd.isReady() && !(interstitialAd.hasExpired())) {
                interstitialAd.show(context);
            } else {
                Logger.i(TAG + " No interstitial is currently available !");
                adMap.remove(config);
                requestFailed("Show interstitial failed", failureCallback);
            }
        } catch (Throwable e) {
            adMap.remove(config);
            requestFailed("Show interstitial failed", failureCallback);
        }
    }

    private void requestFailed(String msg, Callbacks.WithOnlyFailure<String> failureCallback) {
        mmAdConfig = null;
        failureCallback.onFailure(msg);
    }

    public boolean isInterstitialAvailable(AdConfig config, String placementId) {
        try {
            if (!TextUtils.isEmpty(placementId)) {
                if (this.mmAdConfig != null && !(config.equals(this.mmAdConfig))) {
                    Logger.i(TAG + " Fetch Interstitial Failed. Wrong AdConfig.");
                    return false;
                }
            }
            InterstitialAd interstitialAd = adMap.get(config);
            if (interstitialAd != null && interstitialAd.isReady() && !(interstitialAd.hasExpired())) {
                Logger.i(TAG + " Interstitial is available and ready !");
                return true;
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        return false;
    }

    public void notifyCachingTimeout() {
        isTimedOut = true;
        String reason = " Time Out In Fetching Interstitial";
        Logger.e(TAG + reason);
        mmAdConfig = null;
    }

    private class AdListener implements InterstitialAd.InterstitialListener {

        private final AdConfig adConfig;

        AdListener(AdConfig config) {
            this.adConfig = config;
        }

        @Override
        public void onLoaded(InterstitialAd interstitialAd) {
            Logger.i(TAG + " Interstitial Loaded !");
            adMap.put(adConfig, interstitialAd);

            if (!isTimedOut) {
                isTimedOut = true;
                callback.onSuccess(0.0);
            }

        }

        @Override
        public void onLoadFailed(InterstitialAd interstitialAd, InterstitialAd.InterstitialErrorStatus interstitialErrorStatus) {
            Logger.i(TAG + " Interstitial Load Failed " + interstitialErrorStatus.getDescription());
            if (!isTimedOut) {
                isTimedOut = true;
                callback.onFailure(interstitialErrorStatus.getDescription());
                mmAdConfig = null;
            }
        }

        @Override
        public void onShown(InterstitialAd interstitialAd) {
            Logger.i(TAG + " Interstitial Displayed !");
            AdManager.getInstance().adDisplayed(adConfig, adNetworkInfo);

        }

        @Override
        public void onShowFailed(InterstitialAd interstitialAd, InterstitialAd.InterstitialErrorStatus interstitialErrorStatus) {

        }

        @Override
        public void onClosed(InterstitialAd interstitialAd) {
            Logger.i(TAG + " Interstitial Closed ");
            AdManager.getInstance().adCompleted(adConfig, adNetworkInfo);
            AdManager.getInstance().adClosed(adConfig, adNetworkInfo);
            adMap.remove(adConfig);
            mmAdConfig = null;
        }

        @Override
        public void onClicked(InterstitialAd interstitialAd) {

        }

        @Override
        public void onAdLeftApplication(InterstitialAd interstitialAd) {

        }

        @Override
        public void onExpired(InterstitialAd interstitialAd) {

        }
    }
}