package com.pokkt.thirdparty;

import android.app.Activity;
import android.content.Context;
import android.support.annotation.Keep;
import android.text.TextUtils;

import com.millennialmedia.MMSDK;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdFormat;
import com.pokkt.sdk.adnetworks.AdNetwork;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;
import com.pokkt.thirdparty.Millennial.MillennialBannerManager;
import com.pokkt.thirdparty.Millennial.MillennialInterstitialManager;

import org.json.JSONObject;

@Keep
public class MillennialNetwork implements AdNetwork {

    private static final String TAG = MillennialNetwork.class.getName();

    private Context context = null;
    private boolean isInitialized = false;
    private AdNetworkInfo adNetworkInfo;
    private MillennialInterstitialManager interstitialManager;
    private MillennialBannerManager bannerManager;

    @Override
    public void initNetwork(final Context context, AdNetworkInfo info) {
        this.adNetworkInfo = info;

        synchronized (MillennialNetwork.class) {
            if (isInitialized) {
                Logger.d(TAG+ " Network already initialized !");
                return;
            }

            if (!(context instanceof Activity)) {
                Logger.d(TAG+ " Init Configurations Error. Needs Activity context");
                return;
            }

            this.context = context;

            if (!sdkEnabled()) {
                Logger.d(TAG+ " Init Configurations Error!");
                return;
            }

            ((Activity) context).runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    try {
                        MMSDK.initialize(((Activity) context).getApplication());
                        interstitialManager = new MillennialInterstitialManager(adNetworkInfo);
                        bannerManager = new MillennialBannerManager();
                        isInitialized = true;
                    } catch (Throwable e) {
                        Logger.d(TAG+ " Init Configurations Error or SDK Error!");
                    }
                }
            });
        }
    }

    @Override
    public AdNetworkInfo getAdNetworkInfo() {
        return adNetworkInfo;
    }

    @Override
    public void cacheAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
        if (AdFormat.INTERSTITIAL == config.adFormat) {
            interstitialManager.fetchAd(config, callbacks, context, getPlacementId(config));
        } else {
            callbacks.onFailure("Invalid Ad Format. Ad Not Available.");
        }
    }

    @Override
    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {
        Logger.d(TAG+ " Play Ad Called");
        if (AdFormat.INTERSTITIAL == config.adFormat) {
            interstitialManager.showAd(config, failureCallback, context, getPlacementId(config));
        } else {
            failureCallback.onFailure("Invalid Ad Format. Ad Not Available.");
        }
    }

    @Override
    public void adClosed(AdConfig adConfig) {

    }

    @Override
    public void checkAdAvailability(AdConfig config, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        if (AdFormat.INTERSTITIAL == config.adFormat) {
            if (interstitialManager.isInterstitialAvailable(config, getPlacementId(config))) {
                callbacks.onSuccess(null);
                return;
            }
        } else {
            callbacks.onFailure("Invalid Ad Format. Ad Not Available.");
            return;
        }
        callbacks.onFailure("No Ad Available.");
    }

    @Override
    public void loadBannerAd(AdConfig adConfig, BannerUnit bannerUnit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        bannerManager.loadBanner(adConfig, bannerUnit, callbacks, getPlacementId(adConfig));
    }

    @Override
    public void destroyBannerAd(AdConfig adConfig) {
        bannerManager.destroyBanner(adConfig);
    }

    @Override
    public void notifyCachingTimeout(AdConfig config) {
        if (AdFormat.INTERSTITIAL == config.adFormat) {
            interstitialManager.notifyCachingTimeout();
        }
    }

    @Override
    public double getAdVC(AdConfig adConfig) {
        return 0;
    }

    @Override
    public boolean supportsAdConfig(AdConfig config) {
        if (AdFormat.INTERSTITIAL == config.adFormat) {
            if (!(config.isRewarded)) {
                return true;
            }
        } else if (AdFormat.BANNER == config.adFormat) {
            return true;
        }

        return false;
    }

    @Override
    public boolean isInitialised() {
        return isInitialized;
    }

    @Override
    public boolean isPokktNetwork() {
        return false;
    }

    @Override
    public boolean verifyCampaignForAdConfig(AdConfig adConfig, boolean verifyCached) {
        return true;
    }

    private String getPlacementId(AdConfig config) {
        String placementId = "";
        if (adNetworkInfo.getCustomData().containsKey("screens_mapping_data")) {
            try {
                JSONObject mappingObject = new JSONObject(adNetworkInfo.getCustomData().get("screens_mapping_data"));
                String mapData = mappingObject.optString(config.screenName);
                if (!TextUtils.isEmpty(mapData)) {
                    JSONObject mapObj = new JSONObject(mapData);
                    int isInterstitial = mapObj.optInt("non_incent");
                    if (isInterstitial == 1) {
                        placementId = mapObj.optString("network_screen", "").trim();
                    } else {
                        Logger.d(TAG+ " Failed.Requesting rewarded interstitial from facebook.");
                        return placementId;
                    }
                }
            } catch (Exception e) {
                Logger.printStackTrace(TAG+ " Invalid Mapping Json", e);
            }
        }
        if (placementId.length() == 0) {
            placementId = config.screenName;
        }
        return placementId;
    }

    private boolean sdkEnabled() {
        try {
            Class.forName("com.millennialmedia.MMSDK");
            Class.forName("com.millennialmedia.InterstitialAd");
            return true;
        } catch (Throwable ignore) {
            Logger.e(TAG+ " SDK not found");
            return false;
        }
    }
}