package com.pokkt.thirdparty;

import android.app.Activity;
import android.content.Context;
import android.support.annotation.Keep;
import android.text.TextUtils;

import com.nativex.monetization.MonetizationManager;
import com.nativex.monetization.business.reward.Reward;
import com.nativex.monetization.communication.RedeemRewardData;
import com.nativex.monetization.enums.AdEvent;
import com.nativex.monetization.listeners.OnAdEventV2;
import com.nativex.monetization.listeners.RewardListener;
import com.nativex.monetization.listeners.SessionListener;
import com.nativex.monetization.mraid.AdInfo;
import com.pokkt.PokktAds;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdFormat;
import com.pokkt.sdk.adnetworks.AdNetwork;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;

import org.json.JSONObject;

import java.util.HashMap;
import java.util.Map;

@Keep
public class NativeXNetwork implements AdNetwork, SessionListener {

    private static final String TAG = NativeXNetwork.class.getName();
    private static final String APP_ID_KEY = "appId";
    private static final String CURRENCY_KEY = "amount";
    private Context context = null;
    private int initialized = -1;//-1 not initialized, 0 initializing , 1 initialized
    private AdNetworkInfo adNetworkInfo = null;
    private double amount = 0;
    private AdConfig adConfig;
    private Map<AdConfig, AdListener> listenerMap = new HashMap<>();

    @Override
    public void initNetwork(Context context, AdNetworkInfo info) {
        Logger.i(TAG + " init network called !");

        synchronized (NativeXNetwork.class) {
            if (initialized == 1 || initialized == 0) {
                Logger.i(TAG + " network already initialized or initializing !");
                return;
            }
            if (!sdkEnabled()) {
                Logger.i(TAG + "  Init Error !");
                return;
            }

            this.context = context;

            if (getSuitableActivity() == null) {
                Logger.i(TAG + " Activity Init Error!");
                return;
            }

            MonetizationManager.enableLogging(PokktAds.Debugging.isEnabled());

            this.adNetworkInfo = info;
            if (extrasAreValid()) {
                String appId = adNetworkInfo.getCustomData().get(APP_ID_KEY);
                if (adNetworkInfo.getCustomData().containsKey(CURRENCY_KEY)) {
                    try {
                        amount = Double.parseDouble(adNetworkInfo.getCustomData().get(CURRENCY_KEY));
                    } catch (Exception e) {
                        Logger.printStackTrace(TAG + " Could not parse currency", e);
                    }
                }
                try {
                    MonetizationManager.createSession(context, appId, this);
                } catch (Throwable e) {
                    Logger.printStackTrace(TAG + " Init Configurations Error ", e);
                    return;
                }
                Logger.i(TAG + " network initialized !");
                Logger.i(TAG + " SDK version " + MonetizationManager.getSDKVersion());
                initialized = 0;
            } else {
                Logger.e(TAG + " Init Configurations Error !");
            }
        }
    }

    private Activity getSuitableActivity() {
        if (AdManager.getInstance().getAssignedActivity() != null) {
            return AdManager.getInstance().getAssignedActivity();
        } else if (context instanceof Activity) {
            return (Activity) context;
        }
        return null;
    }

    @Override
    public AdNetworkInfo getAdNetworkInfo() {
        return adNetworkInfo;
    }

    private boolean sdkEnabled() {
        try {
            Class.forName("com.nativex.monetization.MonetizationManager");
            Class.forName("com.nativex.monetization.enums.AdEvent");
            Class.forName("com.nativex.monetization.enums.NativeXAdPlacement");
            Class.forName("com.google.gson.Gson");
            return true;
        } catch (Throwable ignore) {
            Logger.e(TAG + " SDK not found");
            return false;
        }
    }

    private boolean extrasAreValid() {
        return adNetworkInfo != null
                && adNetworkInfo.getCustomData() != null
                && adNetworkInfo.getCustomData().containsKey(APP_ID_KEY);
    }

    @Override
    public void createSessionCompleted(boolean success, boolean isOfferWallEnabled, String message) {
        if (success) {
            initialized = 1;
        }
    }

    private String getPlacement(AdConfig config) {
        String placement = "";
        if (adNetworkInfo.getCustomData().containsKey("screens_mapping_data")) {
            try {
                JSONObject mappingObject = new JSONObject(adNetworkInfo.getCustomData().get("screens_mapping_data"));
                String mapData = mappingObject.optString(config.screenName, "");
                if (!TextUtils.isEmpty(mapData)) {
                    JSONObject mapObj = new JSONObject(mapData);
                    int isNonIncent = mapObj.optInt("non_incent");
                    int isIncent = mapObj.optInt("incent");
                    if ((isNonIncent == 1 && !config.isRewarded || (isIncent == 1 && config.isRewarded))) {
                        placement = mapObj.optString("network_screen", "").trim();
                    } else {
                        return placement;
                    }
                }
            } catch (Exception e) {
                Logger.printStackTrace(TAG + " Invalid Mapping Json", e);
            }

        }
        if (placement.length() == 0) {
            placement = config.screenName;
        }
        return placement;
    }

    @Override
    public void cacheAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
        Logger.i(TAG + " caching called !");
        try {
            if (getSuitableActivity() != null) {
                String placement = getPlacement(config);
                if (!TextUtils.isEmpty(placement)) {
                    if (MonetizationManager.isAdReady(placement)) {
                        if (adConfig == null || config.equals(adConfig)) {
                            callbacks.onSuccess(0.0);
                            if (adConfig == null) {
                                adConfig = config.clone();
                            }
                        } else {
                            callbacks.onFailure("Caching Failed.");
                        }
                    } else {
                        if (adConfig == null) {
                            adConfig = config.clone();
                        }
                        AdListener listener = new AdListener(config, callbacks);
                        listenerMap.put(config, listener);
                        MonetizationManager.fetchAd(getSuitableActivity(), placement, listener);
                    }
                } else {
                    callbacks.onFailure("Caching Failed. Invalid placement.");
                }
            } else {
                callbacks.onFailure("Caching Failed. Activity needed.");
            }
        } catch (Throwable e) {
            adConfig = null;
            Logger.printStackTrace(TAG + " cache ad failed ", e);
            listenerMap.remove(config);
            callbacks.onFailure("Caching Failed");
        }
    }

    @Override
    public void showAd(final AdConfig config, Callbacks.
            WithOnlyFailure<String> failureCallback) {
        Logger.i(TAG + "  show called !");
        try {
            if (getSuitableActivity() != null) {
                String placement = getPlacement(config);
                if (!TextUtils.isEmpty(placement)) {
                    if (MonetizationManager.isAdReady(placement) && config.equals(adConfig)) {
                        MonetizationManager.setRewardListener(new RewardListener() {
                            @Override
                            public void onRedeem(RedeemRewardData redeemRewardData) {
                                if (config.isRewarded) {
                                    amount = 0;
                                    for (Reward reward : redeemRewardData.getRewards()) {
                                        amount = reward.getAmount();
                                    }
                                    AdManager.getInstance().adGratified(config, amount, adNetworkInfo);
                                }
                            }
                        });
                        MonetizationManager.showReadyAd(getSuitableActivity(), placement, new OnAdEventV2() {
                            @Override
                            public void onEvent(AdEvent adEvent, AdInfo adInfo, String s) {
                                {
                                    switch (adEvent) {
                                        case ALREADY_SHOWN:
                                            break;
                                        case BEFORE_DISPLAY:
                                            break;
                                        case DISMISSED:
                                            Logger.i(TAG + " onAdDismissed !");
                                            adConfig = null;
                                            AdManager.getInstance().adClosed(config, adNetworkInfo);
                                            break;
                                        case DISPLAYED:
                                            Logger.i(TAG + " onAdDisplayed !");
                                            AdManager.getInstance().adDisplayed(config, adNetworkInfo);
                                            break;
                                        case DOWNLOADING:
                                            break;
                                        case ERROR:
                                            Logger.i(TAG + " Error !");
                                            adConfig = null;
                                            AdManager.getInstance().adClosed(config, adNetworkInfo);
                                            break;
                                        case EXPIRED:
                                            Logger.i(TAG + " Expired !");
                                            adConfig = null;
                                            AdManager.getInstance().adClosed(config, adNetworkInfo);
                                            break;
                                        case USER_NAVIGATES_OUT_OF_APP:
                                            break;
                                        case IMPRESSION_CONFIRMED:
                                            break;
                                        case AD_CONVERTED:
                                            Logger.i(TAG + " AD CONVERTED !");
                                            adConfig = null;
//                                            PokktCustomNetworkAdDelegate.onRewardedAdGratified(context, amount, network, config);
                                            AdManager.getInstance().adCompleted(config, adNetworkInfo);
                                            break;
                                    }
                                }
                            }
                        });
                    } else {
                        requestFailed("Show Failed.Ad not available.", failureCallback);
                    }
                } else {
                    requestFailed("Show Failed.Invalid Placement.", failureCallback);
                }
            } else {
                requestFailed("Show Failed.Activity needed.", failureCallback);
            }
        } catch (Throwable e) {
            adConfig = null;
            Logger.printStackTrace(TAG + " Play Ad Failed", e);
            requestFailed("Play Ad Failed !", failureCallback);
        }
    }

    @Override
    public void adClosed(AdConfig adConfig) {

    }

    @Override
    public void checkAdAvailability(AdConfig config, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        try {
            if (getSuitableActivity() != null) {
                String placement = getPlacement(config);
                if (!TextUtils.isEmpty(placement)) {
                    if (MonetizationManager.isAdReady(placement) && config.equals(adConfig)) {
                        callbacks.onSuccess(null);
                    }
                }
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        callbacks.onFailure("Ad Not Available");
    }

    @Override
    public void loadBannerAd(AdConfig adConfig, BannerUnit bannerUnit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        callbacks.onFailure("Invalid Network !");
    }

    @Override
    public void destroyBannerAd(AdConfig adConfig) {

    }

    @Override
    public void notifyCachingTimeout(AdConfig adConfig) {
        if (listenerMap.get(adConfig) != null) {
            listenerMap.get(adConfig).notifyTimeout();
        }
    }

    @Override
    public double getAdVC(AdConfig adConfig) {
        if (adConfig.isRewarded) {
            return amount;
        } else {
            return 0;
        }
    }

    @Override
    public boolean supportsAdConfig(AdConfig config) {
        if (AdFormat.BRANDING == config.adFormat) {
            return false;
        }

        if (AdFormat.BANNER == config.adFormat) {
            return false;
        }
        return true;
    }

    @Override
    public boolean isInitialised() {
        return initialized == 1;
    }

    @Override
    public boolean isPokktNetwork() {
        return false;
    }

    @Override
    public boolean verifyCampaignForAdConfig(AdConfig adConfig, boolean verifyCached) {
        return true;
    }

    private void requestFailed(String msg, Callbacks.WithOnlyFailure<String> failureCallback) {
        failureCallback.onFailure(msg);
    }

    private class AdListener implements OnAdEventV2 {

        private boolean isTimedOut;
        private AdConfig config;
        private Callbacks.WithSuccessAndFailure<Double, String> callback;

        AdListener(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
            this.config = config;
            this.callback = callbacks;
        }

        void notifyTimeout() {
            isTimedOut = true;
            String reason = " Time Out In Fetching Ad";
            Logger.i(TAG + reason);
            listenerMap.remove(config);
            adConfig = null;
        }

        @Override
        public void onEvent(AdEvent adEvent, AdInfo adInfo, String s) {
            switch (adEvent) {
                case ALREADY_FETCHED:
                    Logger.i(TAG + " already fetched !");
                    if (!isTimedOut) {
                        isTimedOut = true;
                        if ((config.adFormat == AdFormat.VIDEO && adInfo.willPlayAudio()) || (config.adFormat == AdFormat.INTERSTITIAL)) {
                            callback.onSuccess(0.0);
                            listenerMap.remove(config);
                        } else {
                            callback.onFailure("No Ad Available. Invalid Ad Format");
                            adConfig = null;
                            listenerMap.remove(config);
                        }
                    }
                    break;
                case DOWNLOADING:
                    break;
                case FETCHED:
                    Logger.i(TAG + " onAdLoadSucceeded !");
                    if (!isTimedOut) {
                        isTimedOut = true;
                        if ((config.adFormat == AdFormat.VIDEO && adInfo.willPlayAudio()) || (config.adFormat == AdFormat.INTERSTITIAL)) {
                            callback.onSuccess(0.0);
                            listenerMap.remove(config);
                        } else {
                            callback.onFailure("No Ad Available.  Invalid Ad Format");
                            adConfig = null;
                            listenerMap.remove(config);
                        }
                    }
                    break;
                case NO_AD:
                case ERROR:
                    if (!isTimedOut) {
                        isTimedOut = true;
                        adConfig = null;
                        callback.onFailure("No Ad Available");
                        listenerMap.remove(config);
                    }
                    break;
            }
        }
    }
}