package com.pokkt.thirdparty.RevMob;

import android.app.Activity;
import android.text.TextUtils;

import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.banners.PokktBannerView;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;

import org.json.JSONObject;

import java.util.List;

import rm.com.android.sdk.Rm;
import rm.com.android.sdk.RmListener;
import rm.com.android.sdk.ads.banner.RMBannerView;

public class RevMobBannerManager {

    private static final String TAG = RevMobBannerManager.class.getName();
    private AdNetworkInfo adNetworkInfo;
    private Callbacks.WithSuccessAndFailure<AdCampaign, String> callback;
    private RMBannerView banner = null;
    private boolean isTimedOut;
    private BannerUnit bannerUnit;
    private AdConfig revMobAdConfig;

    public RevMobBannerManager(AdNetworkInfo info) {
        this.adNetworkInfo = info;
    }

    private String getPlacement(AdConfig info) {
        String placement = "";
        if (adNetworkInfo.getCustomData().containsKey("screens_mapping_data")) {
            try {
                JSONObject mappingObject = new JSONObject(adNetworkInfo.getCustomData().get("screens_mapping_data"));
                String mapData = mappingObject.optString(info.screenName);
                if (!TextUtils.isEmpty(mapData)) {
                    JSONObject mapObj = new JSONObject(mapData);
                    int isBanner = mapObj.optInt("non_incent");
                    if (isBanner == 1) {
                        placement = mapObj.optString("network_screen", "").trim();
                    } else {
                        Logger.d(TAG + " Failed.Requesting rewarded banner from RevMob.");
                        return placement;
                    }
                }
            } catch (Exception e) {
                Logger.printStackTrace(TAG + " Invalid Mapping Json", e);
            }
        }
        if (placement.length() == 0) {
            placement = info.screenName;
        }
        return placement;
    }

    public void loadBanner(AdConfig config, final BannerUnit unit, final Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks, final Activity activity) {
        Logger.d(TAG + " Load banner called !");
        try {
            final String placement = getPlacement(config);
            if (TextUtils.isEmpty(placement)) {
                callbacks.onFailure("Fetch Ad Failed. Invalid AdConfig");
                return;
            }
            if (this.revMobAdConfig != null && !(config.equals(this.revMobAdConfig))) {
                callbacks.onFailure("Wrong AdConfig");
                return;
            }
            if (this.revMobAdConfig == null) {
                this.revMobAdConfig = config.clone();
            }
            if (banner != null) {
                callbacks.onSuccess(null);
                return;
            }
            this.callback = callbacks;
            this.bannerUnit = unit;
            Rm.cacheBanner(placement, new RmListener.Cache() {
                @Override
                public void onRmAdReceived() {
                    Logger.d(TAG + " Banner Loaded !");
                    if (Rm.isBannerLoaded(placement)) {
                        banner = Rm.getBanner(activity, placement, new Get() {
                            @Override
                            public void onRmAdDisplayed() {
                                Logger.d(TAG + " Banner Displayed");
                            }

                            @Override
                            public void onRmAdFailed(String s) {

                            }

                            @Override
                            public void onRmAdDismissed() {

                            }

                            @Override
                            public void onRmAdClicked() {

                            }
                        });


                        if (!isTimedOut) {
                            isTimedOut = true;
                            activity.runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    List<PokktBannerView> containerList = bannerUnit.getContainerList();
                                    if (containerList.size() > 0) {
                                        PokktBannerView container = containerList.get(0);
                                        if (container.getChildAt(0) == null) {
                                            container.addView(banner);
                                        } else {
                                            container.removeView(container.getChildAt(0));
                                            container.addView(banner);
                                        }
                                        banner.show();
                                        callback.onSuccess(null);
                                    } else {
                                        destroyBanner();
                                    }
                                }
                            });
                        }
                    }
                }

                @Override
                public void onRmAdNotReceived(String message) {
                    Logger.d(TAG + " onError !! " + message);
                    if (!isTimedOut) {
                        isTimedOut = true;
                        callback.onFailure("Banner Load Failed ! " + message);
                        activity.runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                destroyBanner();
                            }
                        });
                    }
                }
            });
            isTimedOut = false;
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Load banner failed", e);
            banner = null;
            revMobAdConfig = null;
            callbacks.onFailure("Banner Loading Failed");
        }
    }

    public void destroyBanner() {
        if (banner != null) {
            banner.release();
            banner = null;
            revMobAdConfig = null;
        }
    }

    public void notifyCachingTimeout() {
        String reason = " Time Out In Fetching Banner";
        Logger.e(TAG + reason);
        isTimedOut = true;
        revMobAdConfig = null;
    }
}
