package com.pokkt.thirdparty.RevMob;

import android.app.Activity;
import android.content.Context;
import android.text.TextUtils;

import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.debugging.Logger;

import org.json.JSONObject;

import rm.com.android.sdk.Rm;
import rm.com.android.sdk.RmListener;

public class RevMobInterstitialManager {

    private static final String TAG = RevMobInterstitialManager.class.getName();
    private final AdListener listener;
    private AdNetworkInfo adNetworkInfo;
    private AdConfig adConfig = null;
    private Callbacks.WithSuccessAndFailure<Double, String> callback;
    private boolean isTimedOut = false;

    public RevMobInterstitialManager(AdNetworkInfo info) {
        this.adNetworkInfo = info;
        listener = new AdListener();
    }

    private String getPlacement(AdConfig info) {
        String placement = "";
        if (adNetworkInfo.getCustomData().containsKey("screens_mapping_data")) {
            try {
                JSONObject mappingObject = new JSONObject(adNetworkInfo.getCustomData().get("screens_mapping_data"));
                String mapData = mappingObject.optString(info.screenName);
                if (!TextUtils.isEmpty(mapData)) {
                    JSONObject mapObj = new JSONObject(mapData);
                    int isInterstitial = mapObj.optInt("non_incent");
                    if (isInterstitial == 1) {
                        placement = mapObj.optString("network_screen", "").trim();
                    } else {
                        Logger.d(TAG + " Failed.Requesting rewarded interstitial from RevMob.");
                        return placement;
                    }
                }
            } catch (Exception e) {
                Logger.printStackTrace(TAG + " Invalid Mapping Json", e);
            }
        }
        if (placement.length() == 0) {
            placement = info.screenName;
        }
        return placement;
    }

    public void fetchAd(final AdConfig config, final Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
        Logger.d(TAG + " cache Interstitial called !");

        try {
            String placement = getPlacement(config);
            if (TextUtils.isEmpty(placement)) {
                callbacks.onFailure("Fetch Ad Failed. Invalid AdConfig");
                return;
            }
            if (this.adConfig != null && !(config.equals(this.adConfig))) {
                callbacks.onFailure("Fetch Interstitial Failed. Wrong AdConfig.");
                return;
            }
            if (this.adConfig == null) {
                this.adConfig = config.clone();
            }
            if (Rm.isInterstitialLoaded(placement)) {
                callbacks.onSuccess(0.0);
                return;
            }
            this.callback = callbacks;
            listener.setAdConfig(config);
            Rm.cacheInterstitial(placement, listener);
            isTimedOut = false;
        } catch (Throwable e) {
            this.adConfig = null;
            Logger.printStackTrace(TAG + " Cache Ad failed", e);
            isTimedOut = true;
            callbacks.onFailure("Interstitial Caching Failed");
        }
    }

    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback, Context context) {

        Logger.d(TAG + " show Interstitial called !");
        try {
            String placement = getPlacement(config);
            if (TextUtils.isEmpty(placement)) {
                Logger.d(TAG + " Play Ad Failed. Invalid placement.");
                requestFailed("Play Ad Failed. Invalid placement.", failureCallback);
                return;
            }
            if (this.adConfig != null && !(config.equals(this.adConfig))) {
                failureCallback.onFailure("Fetch Ad Failed. Wrong AdConfig.");
                return;
            }
            if (this.adConfig == null) {
                this.adConfig = config.clone();
            }
            if (Rm.isInterstitialLoaded(placement) && context instanceof Activity) {
                Rm.showInterstitial((Activity) context, placement, listener);
            } else {
                Logger.d(TAG + " No Interstitial is currently available !");
                requestFailed("Interstitial Not Available !", failureCallback);
            }
        } catch (Throwable e) {
            requestFailed("Interstitial Not Available !", failureCallback);
        }
    }

    private void requestFailed(String msg, Callbacks.WithOnlyFailure<String> failureCallback) {
        this.adConfig = null;
        failureCallback.onFailure(msg);
    }

    public boolean isInterstitialAvailable(AdConfig config) {
        try {
            if (this.adConfig != null && !(config.equals(this.adConfig))) {
                return false;
            }
            if (Rm.isInterstitialLoaded(getPlacement(config))) {
                Logger.d(TAG + " Interstitial is available and ready !");
                return true;
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        return false;
    }

    public void notifyCachingTimeout() {
        isTimedOut = true;
        String reason = " Time Out In Fetching Interstitial";
        adConfig = null;
        Logger.e(TAG + reason);
    }

    private class AdListener implements RmListener.Cache, RmListener.Show {
        private AdConfig config;

        public void setAdConfig(AdConfig c) {
            this.config = c;
        }

        @Override
        public void onRmAdReceived() {
            Logger.d(TAG + " Fullscreen loaded");
            if (!isTimedOut) {
                isTimedOut = true;
                callback.onSuccess(0.0);
            }
        }

        @Override
        public void onRmAdNotReceived(String message) {
            Logger.d(TAG + " Fullscreen not received." + message);
            if (!isTimedOut) {
                isTimedOut = true;
                callback.onFailure("Ad Load Failed ! " + message);
                adConfig = null;
            }
        }

        @Override
        public void onRmAdDismissed() {
            Logger.d(TAG + " Fullscreen dismissed.");
            AdManager.getInstance().adCompleted(config, adNetworkInfo);
            AdManager.getInstance().adClosed(config, adNetworkInfo);
            adConfig = null;
        }

        @Override
        public void onRmAdClicked() {
            Logger.d(TAG + " Fullscreen clicked.");
        }

        @Override
        public void onRmAdDisplayed() {
            Logger.d(TAG + " Fullscreen displayed.");
            AdManager.getInstance().adDisplayed(config, adNetworkInfo);
        }

        @Override
        public void onRmAdFailed(String s) {
            Logger.d(TAG + " Fullscreen failed.");
            AdManager.getInstance().adClosed(config, adNetworkInfo);
            adConfig = null;
        }
    }
}