package com.pokkt.thirdparty.RevMob;

import android.app.Activity;
import android.content.Context;
import android.text.TextUtils;

import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.debugging.Logger;

import org.json.JSONObject;

import rm.com.android.sdk.Rm;
import rm.com.android.sdk.RmListener;

public class RevMobNonRewardedVideoManager {

    private static final String TAG = RevMobNonRewardedVideoManager.class.getName();
    private final AdListener listener;

    private AdNetworkInfo adNetworkInfo;
    private AdConfig adConfig;
    private Callbacks.WithSuccessAndFailure<Double, String> callback;
    private boolean isTimedOut;

    public RevMobNonRewardedVideoManager(AdNetworkInfo network) {
        this.adNetworkInfo = network;
        listener = new AdListener();
    }

    private String getPlacement(AdConfig info) {
        String placement = "";
        if (adNetworkInfo.getCustomData().containsKey("screens_mapping_data")) {
            try {
                JSONObject mappingObject = new JSONObject(adNetworkInfo.getCustomData().get("screens_mapping_data"));
                String mapData = mappingObject.optString(info.screenName);
                if (!TextUtils.isEmpty(mapData)) {
                    JSONObject mapObj = new JSONObject(mapData);
                    int isInterstitial = mapObj.optInt("non_incent");
                    if (isInterstitial == 1) {
                        placement = mapObj.optString("network_screen", "").trim();
                    } else {
                        Logger.d(TAG + " Failed.Requesting rewarded interstitial from RevMob.");
                        return placement;
                    }
                }
            } catch (Exception e) {
                Logger.printStackTrace(TAG + " Invalid Mapping Json", e);
            }
        }
        if (placement.length() == 0) {
            placement = info.screenName;
        }
        return placement;
    }

    public void fetchAd(final AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
        Logger.d(TAG + " cache Video called !");

        try {
            String placement = getPlacement(config);
            if (TextUtils.isEmpty(placement)) {
                callbacks.onFailure("Fetch Ad Failed. Invalid AdConfig");
                return;
            }
            if (this.adConfig != null && !(config.equals(this.adConfig))) {
                callbacks.onFailure("Fetch Video Failed. Wrong AdConfig.");
                return;
            }
            if (this.adConfig == null) {
                this.adConfig = config.clone();
            }

            if (Rm.isInterstitialLoaded(placement)) {
                callbacks.onSuccess(0.0);
                return;
            }

            this.callback = callbacks;
            listener.setAdConfig(config);
            Rm.cacheInterstitial(placement, listener);
            isTimedOut = false;
        } catch (Throwable e) {
            adConfig = null;
            isTimedOut = true;
            Logger.printStackTrace(TAG + " Cache Ad Failed", e);
            callbacks.onFailure("Cache Video Failed");
        }
    }

    public void showAd(final AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback, Context context) {
        Logger.d(TAG + " Show Video called !");

        try {
            String placement = getPlacement(config);
            if (TextUtils.isEmpty(placement)) {
                Logger.d(TAG + " Play Ad Failed. Invalid placement.");
                requestFailed("Play Ad Failed. Invalid placement.", failureCallback);
                return;
            }
            if (this.adConfig != null && !(config.equals(this.adConfig))) {
                failureCallback.onFailure("Fetch Video Failed. Wrong AdConfig.");
                return;
            }
            if (this.adConfig == null) {
                this.adConfig = config.clone();
            }
            if (Rm.isInterstitialLoaded(placement) && context instanceof Activity) {
                Rm.showInterstitial((Activity) context, placement, listener);
            } else {
                Logger.d(TAG + " No Video is currently available !");
                requestFailed("Show Video failed", failureCallback);
            }
        } catch (Throwable e) {
            requestFailed("Show Video failed", failureCallback);
        }
    }

    private void requestFailed(String msg, Callbacks.WithOnlyFailure<String> failureCallback) {
        adConfig = null;
        failureCallback.onFailure(msg);
    }

    public boolean isVideoAvailable(AdConfig config) {
        try {
            if (this.adConfig != null && !(config.equals(this.adConfig))) {
                return false;
            }
            if (Rm.isInterstitialLoaded(getPlacement(config))) {
                Logger.d(TAG + " Video is available and ready !");
                return true;
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        return false;
    }

    public void notifyCachingTimeout() {
        isTimedOut = true;
        String reason = " Time Out In Fetching Video";
        Logger.e(TAG + reason);
        adConfig = null;
    }

    private class AdListener implements RmListener.Cache, RmListener.Show {

        private AdConfig config;

        public void setAdConfig(AdConfig c) {
            this.config = c;
        }

        @Override
        public void onRmAdDismissed() {
            Logger.d(TAG + " Video Closed");
            AdManager.getInstance().adCompleted(config, adNetworkInfo);
            AdManager.getInstance().adClosed(config, adNetworkInfo);
            adConfig = null;
        }

        @Override
        public void onRmAdDisplayed() {
            Logger.d(TAG + " Video Displayed");
            AdManager.getInstance().adDisplayed(config, adNetworkInfo);
        }

        @Override
        public void onRmAdClicked() {

        }

        @Override
        public void onRmAdFailed(String s) {
            Logger.d(TAG + " Video has not been completely loaded. " + s);
            //You tried to show it but it's not loaded yet
            AdManager.getInstance().adClosed(config, adNetworkInfo);
            adConfig = null;
        }

        @Override
        public void onRmAdReceived() {
            Logger.d(TAG + " Video Loaded");
            if (!isTimedOut) {
                isTimedOut = true;
                callback.onSuccess(0.0);
            }

        }

        @Override
        public void onRmAdNotReceived(String message) {
            Logger.d(TAG + " Video Load Failed ");
            if (!isTimedOut) {
                isTimedOut = true;
                callback.onFailure("Video load failed");
                adConfig = null;
            }
        }
    }
}