package com.pokkt.thirdparty.StartApp;

import android.content.Context;
import android.view.View;

import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.banners.PokktBannerView;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;
import com.startapp.android.publish.ads.banner.Banner;
import com.startapp.android.publish.ads.banner.BannerListener;

import java.util.List;

public class StartAppBannerManager {

    private static final String TAG = StartAppBannerManager.class.getName();
    private final Context context;
    private Callbacks.WithSuccessAndFailure<AdCampaign, String> callback;
    private boolean isTimedOut;
    private Banner startAppBanner;
    private BannerUnit bannerUnit;

    public StartAppBannerManager(Context ctx) {
        this.context = ctx;
    }


    public void loadBanner(final BannerUnit unit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        Logger.d(TAG + " Load Banner called !");

        if (startAppBanner != null) {
            callbacks.onFailure(" Banner Already showing");
            return;
        }
        this.bannerUnit = unit;
        Listener listener = new Listener();
        this.callback = callbacks;

        try {
            startAppBanner = new Banner(context, listener);
            isTimedOut = false;
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Load banner failed", e);
            isTimedOut = true;
            startAppBanner = null;
            callbacks.onFailure("Banner Caching Failed");
        }
    }

    public void destroyBanner() {
        if (startAppBanner != null) {
            Logger.d(TAG + " Banner destroyed");
            isTimedOut = true;
            startAppBanner = null;
        }
    }

    public void notifyCachingTimeout() {
        String reason = " Time Out In Fetching Banner";
        Logger.e(TAG + reason);
        isTimedOut = true;
        startAppBanner.hideBanner();
        startAppBanner = null;

    }

    private class Listener implements BannerListener {

        @Override
        public void onReceiveAd(View view) {
            Logger.d(TAG + " Banner Fetched");
            if (!isTimedOut) {
                isTimedOut = true;
                List<PokktBannerView> containerList = bannerUnit.getContainerList();
                if (containerList.size() > 0) {
                    PokktBannerView container = containerList.get(0);

                    if (container.getChildAt(0) == null) {
                        container.addView(view);
                    } else {
                        container.removeView(container.getChildAt(0));
                        container.addView(view);
                    }

                    callback.onSuccess(null);
                    startAppBanner.showBanner();
                } else {
                    startAppBanner.hideBanner();
                    startAppBanner = null;
                }
            }
        }

        @Override
        public void onFailedToReceiveAd(View view) {
            if (!isTimedOut) {
                Logger.d(TAG + " Banner load failed ");
                isTimedOut = true;
                callback.onFailure("Banner Load Failed ");
            }
        }

        @Override
        public void onClick(View view) {

        }
    }
}