package com.pokkt.thirdparty.StartApp;

import android.content.Context;

import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.debugging.Logger;
import com.startapp.android.publish.adsCommon.Ad;
import com.startapp.android.publish.adsCommon.StartAppAd;
import com.startapp.android.publish.adsCommon.adListeners.AdDisplayListener;
import com.startapp.android.publish.adsCommon.adListeners.AdEventListener;

public class StartAppInterstitialManager {

    private static final String TAG = StartAppInterstitialManager.class.getName();

    private AdNetworkInfo adNetworkInfo;
    private Ad interstitialAd = null;
    private AdConfig adConfig;
    private Callbacks.WithSuccessAndFailure<Double, String> callback;
    private boolean isTimedOut;
    private StartAppAd startAppAd;

    public StartAppInterstitialManager(AdNetworkInfo network, Context ctx) {
        this.adNetworkInfo = network;
        startAppAd = new StartAppAd(ctx);
    }

    public void fetchAd(final AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
        Logger.d(TAG + " cache Interstitial called !");

        try {
            if (this.adConfig != null && !(config.equals(this.adConfig))) {
                callbacks.onFailure("Fetch Interstitial Failed. Wrong AdConfig.");
                return;
            }
            if (this.adConfig == null) {
                this.adConfig = config.clone();
            }

            if (interstitialAd != null && interstitialAd.isReady()) {
                callbacks.onSuccess(0.0);
                return;
            }

            this.callback = callbacks;
            startAppAd.loadAd(StartAppAd.AdMode.FULLPAGE, new AdListener(config));
            isTimedOut = false;

        } catch (Throwable e) {
            adConfig = null;
            isTimedOut = true;
            Logger.printStackTrace(TAG + " Cache Ad Failed", e);
            callbacks.onFailure("Cache Interstitial Failed");
        }
    }

    public void showAd(final AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {
        Logger.d(TAG + " Show interstitial called !");

        try {
            if (this.adConfig != null && !(config.equals(this.adConfig))) {
                failureCallback.onFailure("Fetch interstitial Failed. Wrong AdConfig.");
                return;
            }
            if (this.adConfig == null) {
                this.adConfig = config.clone();
            }
            if (interstitialAd != null && interstitialAd.isReady()) {
                startAppAd.showAd(new AdListener(config));
            } else {
                Logger.d(TAG + " No interstitial is currently available !");
                requestFailed("Show interstitial failed", failureCallback);
            }
        } catch (Throwable e) {
            requestFailed("Show interstitial failed", failureCallback);
        }
    }

    private void requestFailed(String msg, Callbacks.WithOnlyFailure<String> failureCallback) {
        adConfig = null;
        interstitialAd = null;
        failureCallback.onFailure(msg);
    }

    public boolean isInterstitialAvailable(AdConfig config) {
        try {
            if (this.adConfig != null && !(config.equals(this.adConfig))) {
                return false;
            }
            if (interstitialAd != null && interstitialAd.isReady()) {
                Logger.d(TAG + " Interstitial is available and ready !");
                return true;
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        return false;
    }

    public void notifyCachingTimeout() {
        isTimedOut = true;
        String reason = " Time Out In Fetching Interstitial";
        Logger.e(TAG + reason);
        adConfig = null;
        interstitialAd = null;
    }

    private class AdListener implements AdDisplayListener, AdEventListener {
        private final AdConfig config;

        public AdListener(AdConfig c) {
            this.config = c;
        }

        @Override
        public void adHidden(Ad ad) {
            Logger.d(TAG + " Interstitial Closed");
            AdManager.getInstance().adClosed(config, adNetworkInfo);
            interstitialAd = null;
            adConfig = null;
        }

        @Override
        public void adDisplayed(Ad ad) {
            Logger.d(TAG + " Interstitial Displayed");
            AdManager.getInstance().adDisplayed(config, adNetworkInfo);
        }

        @Override
        public void adClicked(Ad ad) {

        }

        @Override
        public void adNotDisplayed(Ad ad) {
            Logger.d(TAG + " Interstitial Not Displayed");
            AdManager.getInstance().adClosed(config, adNetworkInfo);
            interstitialAd = null;
            adConfig = null;
        }

        @Override
        public void onReceiveAd(Ad ad) {
            Logger.d(TAG + " Interstitial Received");
            interstitialAd = ad;
            if (!isTimedOut) {
                isTimedOut = true;
                callback.onSuccess(0.0);
            }

        }

        @Override
        public void onFailedToReceiveAd(Ad ad) {
            Logger.d(TAG + " Interstitial Load Failed");
            if (!isTimedOut) {
                isTimedOut = true;
                callback.onFailure("Interstitial load failed");
                adConfig = null;
            }
        }
    }
}