package com.pokkt.thirdparty.StartApp;

import android.content.Context;

import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.debugging.Logger;
import com.startapp.android.publish.adsCommon.Ad;
import com.startapp.android.publish.adsCommon.StartAppAd;
import com.startapp.android.publish.adsCommon.VideoListener;
import com.startapp.android.publish.adsCommon.adListeners.AdDisplayListener;
import com.startapp.android.publish.adsCommon.adListeners.AdEventListener;

public class StartAppVideoManager {

    private static final String TAG = StartAppVideoManager.class.getName();
    private AdNetworkInfo adNetworkInfo;
    private Ad videoAd = null;
    private AdConfig adConfig;
    private Callbacks.WithSuccessAndFailure<Double, String> callback;
    private boolean isTimedOut;
    private StartAppAd startAppAd;

    public StartAppVideoManager(AdNetworkInfo network, Context ctx) {
        this.adNetworkInfo = network;
        startAppAd = new StartAppAd(ctx);
    }

    public void fetchAd(final AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
        Logger.d(TAG + " cache Video called !");

        try {
            if (this.adConfig != null && !(config.equals(this.adConfig))) {
                callbacks.onFailure("Fetch Video Failed. Wrong AdConfig.");
                return;
            }
            if (this.adConfig == null) {
                this.adConfig = config.clone();
            }

            if (videoAd != null && videoAd.isReady()) {
                callbacks.onSuccess(0.0);
                return;
            }

            this.callback = callbacks;
            AdListener listener = new AdListener(config);
            if (config.isRewarded) {
                startAppAd.loadAd(StartAppAd.AdMode.REWARDED_VIDEO, listener);
            } else {
                startAppAd.loadAd(StartAppAd.AdMode.VIDEO, listener);
            }
            startAppAd.setVideoListener(listener);
            isTimedOut = false;

        } catch (Throwable e) {
            adConfig = null;
            isTimedOut = true;
            Logger.printStackTrace(TAG + " Cache Ad Failed", e);
            callbacks.onFailure("Cache Video Failed");
        }
    }

    public void showAd(final AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {
        Logger.d(TAG + " Show video called !");

        try {
            if (this.adConfig != null && !(config.equals(this.adConfig))) {
                failureCallback.onFailure("Fetch video Failed. Wrong AdConfig.");
                return;
            }
            if (this.adConfig == null) {
                this.adConfig = config.clone();
            }
            if (videoAd != null && videoAd.isReady()) {
                startAppAd.showAd(new AdListener(config));
            } else {
                Logger.d(TAG + " No video is currently available !");
                requestFailed("Show video failed", failureCallback);
            }
        } catch (Throwable e) {
            requestFailed("Show video failed", failureCallback);
        }
    }

    private void requestFailed(String msg, Callbacks.WithOnlyFailure<String> failureCallback) {
        adConfig = null;
        videoAd = null;
        failureCallback.onFailure(msg);
    }

    public boolean isVideoAvailable(AdConfig config) {
        try {
            if (this.adConfig != null && !(config.equals(this.adConfig))) {
                return false;
            }
            if (videoAd != null && videoAd.isReady()) {
                Logger.d(TAG + " Video is available and ready !");
                return true;
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        return false;
    }

    public void notifyCachingTimeout() {
        isTimedOut = true;
        String reason = "Time Out In Fetching Video";
        Logger.e(TAG + reason);
        adConfig = null;
    }

    private class AdListener implements AdDisplayListener, AdEventListener, VideoListener {
        private final AdConfig config;

        public AdListener(AdConfig c) {
            this.config = c;
        }

        @Override
        public void adHidden(Ad ad) {
            Logger.d(TAG + " Video Closed");
            AdManager.getInstance().adClosed(config, adNetworkInfo);
            videoAd = null;
            adConfig = null;
        }

        @Override
        public void adDisplayed(Ad ad) {
            Logger.d(TAG + " Video Displayed");
            AdManager.getInstance().adDisplayed(config, adNetworkInfo);
        }

        @Override
        public void adClicked(Ad ad) {

        }

        @Override
        public void adNotDisplayed(Ad ad) {
            Logger.d(TAG + " Video Not Displayed");
            AdManager.getInstance().adClosed(config, adNetworkInfo);
            videoAd = null;
            adConfig = null;
        }

        @Override
        public void onReceiveAd(Ad ad) {
            Logger.d(TAG + " Video Loaded");
            videoAd = ad;
            if (!isTimedOut) {
                isTimedOut = true;
                callback.onSuccess(0.0);
            }

        }

        @Override
        public void onFailedToReceiveAd(Ad ad) {
            Logger.d(TAG + " Video Load Failed");
            if (!isTimedOut) {
                isTimedOut = true;
                callback.onFailure("Video load failed");
                adConfig = null;
            }
        }

        @Override
        public void onVideoCompleted() {
            Logger.d(TAG + " Video Completed");
            AdManager.getInstance().adCompleted(config, adNetworkInfo);
            if (config.isRewarded) {
                AdManager.getInstance().adGratified(config, 0.0, adNetworkInfo);
            }
        }
    }
}