package com.pokkt.thirdparty;

import android.app.Activity;
import android.content.Context;
import android.support.annotation.Keep;

import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdFormat;
import com.pokkt.sdk.adnetworks.AdNetwork;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;
import com.pokkt.thirdparty.StartApp.StartAppInterstitialManager;
import com.pokkt.thirdparty.StartApp.StartAppVideoManager;
import com.startapp.android.publish.adsCommon.StartAppAd;
import com.startapp.android.publish.adsCommon.StartAppSDK;

@Keep
public class StartAppNetwork implements AdNetwork {

    private static final String TAG = StartAppNetwork.class.getName();

    private boolean isInitialized = false;
    private AdNetworkInfo adNetworkInfo;
    private StartAppInterstitialManager interstitialManager;
    private StartAppVideoManager videoManager;
//    private StartAppBannerManager bannerManager;
    private static final String APP_ID = "appId";
    private Context context;

    @Override
    public void initNetwork(Context context, AdNetworkInfo info) {
        this.adNetworkInfo = info;

        synchronized (StartAppNetwork.class) {
            if (isInitialized) {
                Logger.d(TAG+ " Already initialized !");
                return;
            }

            if (!(context instanceof Activity)) {
                Logger.d(TAG+ " Init Configurations Error. Needs Activity context");
                return;
            }

            if (!sdkEnabled()) {
                Logger.d(TAG+ " Init Configurations Error !");
                return;
            }

            this.context = context;
            try {
                if (extrasAreValid(info)) {
                    this.adNetworkInfo = info;
                    String appId = info.getCustomData().get(APP_ID);
//                    String appId = "202425051";//TODO change this
                    StartAppSDK.init((Activity) context, appId, false);
                    StartAppAd.disableSplash();
                    interstitialManager = new StartAppInterstitialManager(adNetworkInfo, context);
                    videoManager = new StartAppVideoManager(adNetworkInfo, context);
//                    bannerManager = new StartAppBannerManager(context);
                    isInitialized = true;
                } else {
                    Logger.d(TAG+ " Init Configurations Error");
                }
            } catch (Throwable e) {
                Logger.printStackTrace(TAG+ " Init Configurations Error or SDK Error !",e);
            }
        }
    }

    private boolean extrasAreValid(AdNetworkInfo info) {
        return info != null && info.getCustomData() != null && info.getCustomData().containsKey(APP_ID);
    }

    @Override
    public AdNetworkInfo getAdNetworkInfo() {
        return adNetworkInfo;
    }

    @Override
    public void cacheAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
        Logger.d(TAG+ " Cache Ad called !");

        if (AdFormat.INTERSTITIAL == config.adFormat) {
            interstitialManager.fetchAd(config, callbacks);
        } else if (AdFormat.VIDEO == config.adFormat) {
            videoManager.fetchAd(config, callbacks);
        } else {
            callbacks.onFailure("Invalid Ad Format. Ad Not Available.");
        }
    }

    @Override
    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {
        Logger.d(TAG+ " Play Ad Called");
        if (AdFormat.INTERSTITIAL == config.adFormat) {
            interstitialManager.showAd(config, failureCallback);
        } else if (AdFormat.VIDEO == config.adFormat) {
            videoManager.showAd(config, failureCallback);
        } else {
            failureCallback.onFailure("Invalid Ad Format. Ad Not Available.");
        }
    }

    @Override
    public void adClosed(AdConfig adConfig) {

    }

    @Override
    public void checkAdAvailability(AdConfig config, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        if (AdFormat.INTERSTITIAL == config.adFormat) {
            if (interstitialManager.isInterstitialAvailable(config)) {
                callbacks.onSuccess(null);
            } else {
                callbacks.onFailure("No Ad Available.");
            }
        } else if (AdFormat.VIDEO == config.adFormat) {
            if (videoManager.isVideoAvailable(config)) {
                callbacks.onSuccess(null);
            } else {
                callbacks.onFailure("No Ad Available.");
            }
        } else {
            callbacks.onFailure("Invalid Ad Format.");
        }
    }

    @Override
    public void loadBannerAd(AdConfig adConfig, BannerUnit bannerUnit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        callbacks.onFailure("Invalid ad format");
        //bannerManager.loadBanner(bannerUnit, callbacks);
    }

    @Override
    public void destroyBannerAd(AdConfig adConfig) {
//        bannerManager.destroyBanner();
    }

    @Override
    public void notifyCachingTimeout(AdConfig config) {
        if (AdFormat.INTERSTITIAL == config.adFormat) {
            interstitialManager.notifyCachingTimeout();
        }
        if (AdFormat.VIDEO == config.adFormat) {
            videoManager.notifyCachingTimeout();
        }
//        if (AdFormat.BANNER == config.adFormat) {
//            bannerManager.notifyCachingTimeout();
//        }
    }

    @Override
    public double getAdVC(AdConfig adConfig) {
        return 0;
    }

    @Override
    public boolean supportsAdConfig(AdConfig config) {
        if (AdFormat.INTERSTITIAL == config.adFormat) {
            if (!(config.isRewarded)) {
                return true;
            }
        } else if (AdFormat.VIDEO == config.adFormat) {
            return true;
        } else if (AdFormat.BANNER == config.adFormat) {
            return false;
        }
        return false;
    }

    @Override
    public boolean isInitialised() {
        return isInitialized;
    }

    @Override
    public boolean isPokktNetwork() {
        return false;
    }

    @Override
    public boolean verifyCampaignForAdConfig(AdConfig adConfig, boolean verifyCached) {
        return true;
    }

    private boolean sdkEnabled() {
        try {
            Class.forName("com.startapp.android.publish.adsCommon.StartAppSDK");
            return true;
        } catch (Throwable ignore) {
            Logger.e(TAG+ " SDK not found");
            return false;
        }
    }

    public void handleBackpress() {
        Logger.d(TAG+ " Handle back press");
        StartAppAd.onBackPressed(context);
    }
}