package com.pokkt.thirdparty;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.os.Build;
import android.os.Bundle;
import android.support.annotation.Keep;
import android.support.annotation.RequiresApi;
import android.text.TextUtils;

import com.pokkt.PokktAds;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdFormat;
import com.pokkt.sdk.adnetworks.AdNetwork;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;
import com.tapjoy.TJActionRequest;
import com.tapjoy.TJConnectListener;
import com.tapjoy.TJEarnedCurrencyListener;
import com.tapjoy.TJError;
import com.tapjoy.TJGetCurrencyBalanceListener;
import com.tapjoy.TJPlacement;
import com.tapjoy.TJPlacementListener;
import com.tapjoy.TJVideoListener;
import com.tapjoy.Tapjoy;
import com.tapjoy.TapjoyConnectFlag;
import com.tapjoy.internal.TapjoyNative;

import org.json.JSONObject;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

@RequiresApi(api = Build.VERSION_CODES.ICE_CREAM_SANDWICH)
@Keep
public class TapJoyNetwork implements AdNetwork, Application.ActivityLifecycleCallbacks {

    private static final String TAG = TapJoyNetwork.class.getName();

    private String activityClassName = null;
    private final String SDK_KEY = "sdkKey";
    private boolean isInitialized = false;
    private Map<AdConfig, TJPlacement> adMap = new HashMap<>();
    private AdNetworkInfo adNetworkInfo;
    private double rewardAmount = 0;
    private AdConfig tapjoyAdConfig;
    private Map<AdConfig, TapJoyListener> listenerMap = new HashMap<>();

    @Override
    public void initNetwork(final Context context, AdNetworkInfo info) {
        this.adNetworkInfo = info;

        synchronized (TapJoyNetwork.class) {
            if (isInitialized) {
                Logger.i(TAG + " Already initialized !");
                return;
            }

            if (!(context instanceof Activity)) {
                Logger.e(TAG + " Init Configurations Error. Needs Activity context");
                return;
            }

            if (!sdkEnabled()) {
                Logger.e(TAG + " Init Configurations Error !");
                return;
            }
            Tapjoy.setDebugEnabled(PokktAds.Debugging.isEnabled());
            Tapjoy.setActivity((Activity)context);

            try {
                this.activityClassName = ((Activity) context).getClass().getName();
                ((Activity) context).getApplication().registerActivityLifecycleCallbacks(this);

                String sdkKey = "";
                if (extrasAreValid()) {
                    sdkKey = adNetworkInfo.getCustomData().get(SDK_KEY);
                    String REWARDED_AMOUNT_KEY = "rewardAmount";
                    if (adNetworkInfo.getCustomData().containsKey(REWARDED_AMOUNT_KEY)) {
                        rewardAmount = Double.parseDouble(adNetworkInfo.getCustomData().get(REWARDED_AMOUNT_KEY));
                    }
                } else {
                    Logger.i(TAG + " Init Configurations Error!");
                    return;
                }

                Hashtable<String, Object> connectFlags = new Hashtable<String, Object>();
                connectFlags.put(TapjoyConnectFlag.ENABLE_LOGGING, "true");

                connectFlags.put(TapjoyConnectFlag.USER_ID, AdManager.getInstance().getThirdPartyUserId());

                //Tapjoy.setDebugEnabled(true);
                //Tapjoy.setGcmSender("34027022155");

                Tapjoy.connect(context.getApplicationContext(), sdkKey, connectFlags, new TJConnectListener() {
                    @Override
                    public void onConnectSuccess() {
                        isInitialized = true;
                        Logger.i(TAG+" Connection Success");
                    }

                    @Override
                    public void onConnectFailure() {
                        Logger.i(TAG+" Connection failed");
                    }
                });
            } catch (Throwable e) {
                Logger.printStackTrace(TAG + " Init Configurations Error or SDK Error !", e);
            }
        }
    }

    @Override
    public AdNetworkInfo getAdNetworkInfo() {
        return adNetworkInfo;
    }

    private boolean extrasAreValid() {
        return adNetworkInfo != null
                && adNetworkInfo.getCustomData() != null
                && adNetworkInfo.getCustomData().containsKey(SDK_KEY);
    }

    @Override
    public void cacheAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
        Logger.i(TAG + " Cache Ad Called");
        try {
            if (tapjoyAdConfig != null && !tapjoyAdConfig.equals(config)) {
                callbacks.onFailure("Cache Ad Failed !");
                return;
            }
            if (tapjoyAdConfig == null) {
                tapjoyAdConfig = config.clone();
            }

            if (adMap.containsKey(config)) {
                callbacks.onSuccess(0.0);
                return;
            }

            if (Tapjoy.isConnected()) {
                TapJoyListener listener = new TapJoyListener(config, callbacks);
                String placementName = getPlacementName(config);
                TJPlacement placement = Tapjoy.getPlacement(placementName, listener);
                placement.requestContent();
                Tapjoy.setEarnedCurrencyListener(listener);
                Tapjoy.setVideoListener(listener);
                listenerMap.put(config, listener);
            } else {
                callbacks.onFailure("Not ready");
                tapjoyAdConfig = null;
            }
        } catch (Throwable e) {
            tapjoyAdConfig = null;
            listenerMap.remove(config);
            Logger.printStackTrace(TAG + " Cache Ad Failed", e);
            callbacks.onFailure("Caching Failed");
        }
    }

    @Override
    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {
        Logger.i(TAG + " Play Ad Called");
        try {
            if (tapjoyAdConfig == null || tapjoyAdConfig.equals(config)) {
                if (tapjoyAdConfig == null) {
                    tapjoyAdConfig = config.clone();
                }

                if (adMap.containsKey(config)) {
                    TJPlacement directPlayPlacement = adMap.get(config);
                    if (directPlayPlacement != null && directPlayPlacement.isContentAvailable() && directPlayPlacement.isContentReady()) {
                        directPlayPlacement.showContent();
                    } else {
                        Logger.i(TAG + " No rewarded ad is currently available!!");
                        adMap.remove(config);
                        requestFailed("Registration failed!", failureCallback);
                    }
                } else {
                    Logger.i(TAG + " No rewarded ad is currently available!!");
                    adMap.remove(config);
                    requestFailed("Registration failed!", failureCallback);
                }
            } else {
                requestFailed("Ad Not Available !", failureCallback);
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Play Ad Failed", e);
            requestFailed("Ad Not Available !", failureCallback);
        }
    }

    @Override
    public void adClosed(AdConfig adConfig) {

    }

    @Override
    public void checkAdAvailability(AdConfig config, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        try {
            if (tapjoyAdConfig == null || tapjoyAdConfig.equals(config)) {
                if (adMap.containsKey(config)) {
                    TJPlacement directPlayPlacement = adMap.get(config);
                    if (directPlayPlacement != null && directPlayPlacement.isContentAvailable() && directPlayPlacement.isContentReady()) {
                        callbacks.onSuccess(null);
                        return;
                    }
                }
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " No ad available", e);
        }
        callbacks.onFailure("Ad not available");
    }

    @Override
    public void loadBannerAd(AdConfig adConfig, BannerUnit bannerUnit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        callbacks.onFailure("Invalid Network !");
    }

    @Override
    public void destroyBannerAd(AdConfig adConfig) {

    }

    @Override
    public void notifyCachingTimeout(AdConfig adConfig) {
        if (listenerMap.get(adConfig) != null) {
            listenerMap.get(adConfig).notifyTimeout();
        }
    }

    @Override
    public double getAdVC(AdConfig adConfig) {
        if (adConfig.isRewarded) {
            return rewardAmount;
        } else {
            return 0;
        }
    }

    @Override
    public boolean supportsAdConfig(AdConfig config) {
        return AdFormat.VIDEO == config.adFormat;
    }

    @Override
    public boolean isInitialised() {
        return isInitialized;
    }

    @Override
    public boolean isPokktNetwork() {
        return false;
    }

    @Override
    public boolean verifyCampaignForAdConfig(AdConfig adConfig, boolean verifyCached) {
        return true;
    }

    private String getPlacementName(AdConfig config) {
        String placementName = "";
        if (adNetworkInfo.getCustomData().containsKey("screens_mapping_data")) {
            try {
                JSONObject mappingObject = new JSONObject(adNetworkInfo.getCustomData().get("screens_mapping_data"));
                String mapData = mappingObject.optString(config.screenName);
                if (!TextUtils.isEmpty(mapData)) {
                    JSONObject mapObj = new JSONObject(mapData);
                    placementName = mapObj.optString("network_screen", "").trim();
                }
            } catch (Exception e) {
                Logger.printStackTrace(TAG + " Invalid Mapping Json", e);
            }
        }
        if (placementName.length() == 0) {
            placementName = config.screenName;
        }
        return placementName;
    }

    @Override
    public void onActivityCreated(Activity activity, Bundle bundle) {

    }

    @Override
    public void onActivityStarted(Activity activity) {
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Logger.i(TAG + " Started");
                Tapjoy.onActivityStart(activity);
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG, e);
        }
    }

    @Override
    public void onActivityResumed(Activity activity) {

    }

    @Override
    public void onActivityPaused(Activity activity) {
    }

    @Override
    public void onActivityStopped(Activity activity) {
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Logger.i(TAG + " Stopped");
                Tapjoy.onActivityStop(activity);
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG, e);
        }
    }

    @Override
    public void onActivitySaveInstanceState(Activity activity, Bundle bundle) {

    }

    @Override
    public void onActivityDestroyed(Activity activity) {
    }

    private class TapJoyListener implements TJGetCurrencyBalanceListener, TJPlacementListener, TJEarnedCurrencyListener, TJVideoListener {

        private AdConfig adConfig;
        private Callbacks.WithSuccessAndFailure<Double, String> callback;
        private boolean isTimedOut = false;

        TapJoyListener(AdConfig adConfig, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
            this.adConfig = adConfig;
            this.callback = callbacks;
        }

        void notifyTimeout() {
            isTimedOut = true;
            String reason = " Time Out In Fetching Ad";
            Logger.e(TAG + reason);
            listenerMap.remove(adConfig);
            tapjoyAdConfig = null;
        }

        @Override
        public void onGetCurrencyBalanceResponse(String s, int rewardAmount) {
            //do nothing here
            Logger.i(TAG + " " + s + " " + rewardAmount);
        }

        @Override
        public void onGetCurrencyBalanceResponseFailure(String error) {
            //Nothing to do here
        }

        @Override
        public void onRequestSuccess(TJPlacement placement) {
            // If content is not available you can note it here and act accordingly as best suited for your app
            // This just means the SDK has made contact with Tapjoy's servers. It does not necessarily mean that any content is available.
            Logger.i(TAG + " on request success, contentAvailable: " + placement.isContentAvailable() + " ready " + placement.isContentReady());
            if (placement != null && !placement.isContentAvailable()) {
                if (!isTimedOut) {
                    isTimedOut = true;
                    listenerMap.remove(adConfig);
                    callback.onFailure("Content Not Available");
                    tapjoyAdConfig = null;
                }
            }
        }

        @Override
        public void onRequestFailure(TJPlacement tjPlacement, TJError error) {
            try {
                Logger.i(TAG + error.toString());
                if (!isTimedOut) {
                    isTimedOut = true;
                    listenerMap.remove(adConfig);
                    callback.onFailure("Content Not Available");
                    tapjoyAdConfig = null;
                }
                //AdManager.getInstance().adClosed(adConfig, adNetworkInfo);
            } catch (Exception e) {
                Logger.printStackTrace(TAG, e);
            }
        }

        @Override
        public void onContentReady(TJPlacement placement) {
            //This is called when the content is actually available to display.
            Logger.i(TAG + " onContentReady");
            if (!isTimedOut) {
                isTimedOut = true;
                listenerMap.remove(adConfig);
                if (placement != null && placement.isContentAvailable() && placement.isContentReady()) {
                    adMap.put(adConfig, placement);
                    callback.onSuccess(0.0);
                } else {
                    Logger.i(TAG + " Content Not Ready");
                    callback.onFailure("Download Failed");
                    tapjoyAdConfig = null;
                }
            }
        }

        @Override
        public void onContentShow(TJPlacement placement) {
            Logger.i(TAG + " onContentShow");
        }

        @Override
        public void onContentDismiss(TJPlacement placement) {
            Logger.i(TAG + " onContentDismiss");

            // this used to be in onViewDidClose of TJViewListener
            Tapjoy.getCurrencyBalance(this);
            AdManager.getInstance().adClosed(adConfig, adNetworkInfo);
            adMap.remove(adConfig);
            tapjoyAdConfig = null;
        }

        @Override
        public void onPurchaseRequest(TJPlacement placement, TJActionRequest tjActionRequest, String s) {
            Logger.i(TAG + " onPurchaseRequest");
        }

        @Override
        public void onRewardRequest(TJPlacement placement, TJActionRequest request, String itemId, int quantity) {
            Logger.i(TAG + " onRewardRequest");
        }

        @Override
        public void onEarnedCurrency(String currencyName, int amount) {
            if (adConfig.isRewarded) {
                Logger.i(TAG + " Earned " + amount + " " + currencyName);
                AdManager.getInstance().adGratified(adConfig, amount, adNetworkInfo);
            }
        }


        @Override
        public void onVideoStart() {
            Logger.i(TAG + " video has started");
            AdManager.getInstance().adDisplayed(adConfig, adNetworkInfo);
        }

        @Override
        public void onVideoError(int statusCode) {
            //If in any case video doesn't play then trigger the download failed event.
            Logger.i(TAG + " there was an error with the video: " + statusCode);
            AdManager.getInstance().adClosed(adConfig, adNetworkInfo);
        }

        @Override
        public void onVideoComplete() {
            Logger.i(TAG + " video has completed");

            // Best Practice: We recommend calling getCurrencyBalance as often as possible so the user�s balance is always up-to-date.
            Tapjoy.getCurrencyBalance(this);
            AdManager.getInstance().adCompleted(adConfig, adNetworkInfo);
        }
    }

    private void requestFailed(String msg, Callbacks.WithOnlyFailure<String> failureCallback) {
        tapjoyAdConfig = null;
        failureCallback.onFailure(msg);
    }

    private boolean sdkEnabled() {
        try {
            Class.forName("com.tapjoy.Tapjoy");
            Class.forName("com.tapjoy.TJVideoListener");
            Class.forName("com.tapjoy.TJPlacement");
            return true;
        } catch (Throwable ignore) {
            Logger.e(TAG + " SDK not found");
            return false;
        }
    }

}