package com.pokkt.thirdparty;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.os.Build;
import android.os.Bundle;
import android.support.annotation.Keep;
import android.support.annotation.RequiresApi;
import android.text.TextUtils;

import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdFormat;
import com.pokkt.sdk.adnetworks.AdNetwork;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;
import com.vungle.publisher.AdConfig;
import com.vungle.publisher.VungleAdEventListener;
import com.vungle.publisher.VungleInitListener;
import com.vungle.publisher.VunglePub;

import org.json.JSONObject;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

@RequiresApi(api = Build.VERSION_CODES.ICE_CREAM_SANDWICH)
@Keep
public class VungleNetwork implements AdNetwork, Application.ActivityLifecycleCallbacks {

    private static final String TAG = VungleNetwork.class.getName();
    private String targetClassname = null;
    private boolean isInitialized = false;
    private final VunglePub vunglePub = VunglePub.getInstance();
    private com.pokkt.sdk.AdConfig vungleAdConfig;
    private AdNetworkInfo adNetworkInfo;
    private VungleListener vungleListener;
    private boolean isTimedOut;
    private Callbacks.WithSuccessAndFailure<Double, String> cachingCallbacks;

    @Override
    public void initNetwork(final Context context, AdNetworkInfo info) {
        Logger.i(TAG + " Init network called !");

        synchronized (VungleNetwork.class) {
            if (isInitialized) {
                Logger.i(TAG + " Network already initialized!!");
                return;
            }

            if (!(context instanceof Activity)) {
                Logger.e(TAG + " Init Configurations Error. Needs Activity context");
                return;
            }

            if (!sdkEnabled()) {
                return;
            }

            this.adNetworkInfo = info;
            this.targetClassname = ((Activity) context).getClass().getName();
            ((Activity) context).getApplication().registerActivityLifecycleCallbacks(this);
            if (adNetworkInfo != null && adNetworkInfo.getCustomData() != null && adNetworkInfo.getCustomData().containsKey("appId")) {
                final String appId = adNetworkInfo.getCustomData().get("appId");

                // initialize the Publisher SDK
                ((Activity) context).runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            vunglePub.init(context, appId, getPlacements(), new VungleInitListener() {
                                @Override
                                public void onSuccess() {
                                    isInitialized = true;
                                    vungleListener = new VungleListener();
                                    vunglePub.addEventListeners(vungleListener);
                                }

                                @Override
                                public void onFailure(Throwable throwable) {
                                    isInitialized = false;
                                }
                            });
                            Logger.i(TAG + " Network initialized !");
                        } catch (Throwable e) {
                            Logger.printStackTrace(TAG + "  SDK Init error or SDK error", e);
                        }
                    }
                });
            } else {
                Logger.e(TAG + " Init Configurations Error !");
            }
        }
    }

    @Override
    public AdNetworkInfo getAdNetworkInfo() {
        return adNetworkInfo;
    }

    private boolean sdkEnabled() {
        try {
            Class.forName("com.vungle.publisher.AdConfig");
            Class.forName("com.vungle.publisher.VunglePub");
            return true;
        } catch (Throwable ignore) {
            Logger.e(TAG + " SDK not found");
            return false;
        }
    }

    private String[] getPlacements() {
        List<String> placements = new ArrayList<>();
        if (adNetworkInfo.getCustomData().containsKey("screens_mapping_data")) {
            try {
                JSONObject mappingObject = new JSONObject(adNetworkInfo.getCustomData().get("screens_mapping_data"));
                Iterator<String> iterator = mappingObject.keys();
                while (iterator.hasNext()) {
                    String key = iterator.next();
                    String mapData = mappingObject.optString(key);
                    if (!TextUtils.isEmpty(mapData)) {
                        JSONObject mapObj = new JSONObject(mapData);
                        String placement = mapObj.optString("network_screen", "").trim();
                        if (!TextUtils.isEmpty(placement)) {
                            placements.add(placement);
                        }
                    }
                }
            } catch (Exception e) {
                Logger.printStackTrace(TAG + " Invalid Mapping Json", e);
            }
        }
        return placements.toArray(new String[0]);
    }

    private String getPlacement(com.pokkt.sdk.AdConfig info) {
        String placement = "";
        if (adNetworkInfo.getCustomData().containsKey("screens_mapping_data")) {
            try {
                JSONObject mappingObject = new JSONObject(adNetworkInfo.getCustomData().get("screens_mapping_data"));
                String mapData = mappingObject.optString(info.screenName);
                if (!TextUtils.isEmpty(mapData)) {
                    JSONObject mapObj = new JSONObject(mapData);
                    int isVideo = mapObj.optInt("non_incent");
                    if (isVideo == 1) {
                        placement = mapObj.optString("network_screen", "").trim();
                    } else {
                        Logger.e(TAG + " Failed!.Requesting rewarded video from vungle.");
                        return placement;
                    }
                }
            } catch (Exception e) {
                Logger.printStackTrace(TAG + " Invalid Mapping Json", e);
            }
        }
        if (placement.length() == 0) {
            placement = info.screenName;
        }
        return placement;
    }

    @Override
    public void cacheAd(com.pokkt.sdk.AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {

        Logger.i(TAG + " Cache Ad called !");

        try {
            String placement = getPlacement(config);
            if (TextUtils.isEmpty(placement)) {
                callbacks.onFailure("Fetch Ad Failed. Invalid AdConfig");
                return;
            }

            if (vungleAdConfig != null && !vungleAdConfig.equals(config)) {
                callbacks.onFailure("Cache Ad Failed !");
                return;
            }

            if (vunglePub.isAdPlayable(placement)) {
                // maintain the config for which ad available is sent
                if (vungleAdConfig == null) {
                    vungleAdConfig = config.clone();
                }
                callbacks.onSuccess(0.0);
            } else {
                this.cachingCallbacks = callbacks;
                isTimedOut = false;
                vunglePub.loadAd(placement);
            }
        } catch (Throwable e) {
            callbacks.onFailure("Cache Video Failed");
        }
    }

    @Override
    public void showAd(com.pokkt.sdk.AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {
        Logger.i(TAG + " Play Ad called !");

        try {
            String placement = getPlacement(config);
            if (TextUtils.isEmpty(placement)) {
                Logger.e(TAG + " Play Ad Failed. Invalid placement.");
                requestFailed("Play Ad Failed. Invalid placement.", failureCallback);
                return;
            }
            if (this.vungleAdConfig != null && !(config.equals(this.vungleAdConfig))) {
                failureCallback.onFailure("Fetch Ad Failed. Wrong AdConfig.");
                return;
            }
            if (this.vungleAdConfig == null) {
                this.vungleAdConfig = config.clone();
            }

            if (vunglePub.isAdPlayable(placement)) {
                vungleListener.setConfig(config);
                // create a new AdConfig object
                final AdConfig overrideConfig = new AdConfig();
                // set incentivized option on
                overrideConfig.setIncentivizedUserId(AdManager.getInstance().getThirdPartyUserId());
                overrideConfig.setIncentivizedCancelDialogTitle("Careful!");
                overrideConfig.setIncentivizedCancelDialogBodyText("If the video isn't completed you won't get your reward! Are you sure you want to close early?");
                overrideConfig.setIncentivizedCancelDialogCloseButtonText("Close");
                overrideConfig.setIncentivizedCancelDialogKeepWatchingButtonText("Keep Watching");
                // the overrideConfig object will only affect this ad play.
                vunglePub.playAd(placement, overrideConfig);
            } else {
                requestFailed("Ad Not Available", failureCallback);
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Play Ad Failed", e);
            requestFailed("Play Ad Failed", failureCallback);
        }
    }

    @Override
    public void adClosed(com.pokkt.sdk.AdConfig adConfig) {

    }

    @Override
    public void checkAdAvailability(com.pokkt.sdk.AdConfig config, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        Logger.i(TAG + " Check Ad Available called !");
        String placement = getPlacement(config);
        if (!TextUtils.isEmpty(placement)) {
            if (this.vungleAdConfig != null && !(config.equals(this.vungleAdConfig))) {
                Logger.e(TAG + " Fetch Ad Failed. Wrong AdConfig.");
                callbacks.onFailure("Fetch Ad Failed. Wrong AdConfig.");
                return;
            }
        }
        if (vunglePub.isAdPlayable(placement)) {
            callbacks.onSuccess(null);
        } else {
            callbacks.onFailure("Ad not available");
        }
    }

    @Override
    public void loadBannerAd(com.pokkt.sdk.AdConfig adConfig, BannerUnit bannerUnit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        callbacks.onFailure("Invalid Network !");
    }

    @Override
    public void destroyBannerAd(com.pokkt.sdk.AdConfig adConfig) {

    }

    @Override
    public void notifyCachingTimeout(com.pokkt.sdk.AdConfig adConfig) {
        isTimedOut = true;
        String reason = " Time Out In Fetching Ad";
        vungleAdConfig = null;
        Logger.e(TAG + reason);
    }

    @Override
    public double getAdVC(com.pokkt.sdk.AdConfig adConfig) {
        return 0;
    }

    @Override
    public boolean supportsAdConfig(com.pokkt.sdk.AdConfig config) {
        return AdFormat.VIDEO == config.adFormat;//if adformat is video return true else return false;
    }

    @Override
    public boolean isInitialised() {
        return isInitialized;
    }

    @Override
    public boolean isPokktNetwork() {
        return false;
    }

    @Override
    public boolean verifyCampaignForAdConfig(com.pokkt.sdk.AdConfig adConfig, boolean verifyCached) {
        return true;
    }

    private void requestFailed(String message, Callbacks.WithOnlyFailure<String> failureCallback) {
        vungleAdConfig = null;
        failureCallback.onFailure(message);
    }

    @Override
    public void onActivityCreated(Activity activity, Bundle bundle) {
        try {
            if (activity.getClass().getName().equals(targetClassname)) {
                Logger.i(TAG + " Created Vungle");
                vunglePub.addEventListeners(vungleListener);
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Create failed", e);
        }
    }

    @Override
    public void onActivityStarted(Activity activity) {

    }

    @Override
    public void onActivityResumed(Activity activity) {
        try {
            if (activity.getClass().getName().equals(targetClassname)) {
                Logger.i(TAG + " Resumed Vungle");
                vunglePub.onResume();
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Resume failed", e);
        }

    }

    @Override
    public void onActivityPaused(Activity activity) {
        try {
            if (activity.getClass().getName().equals(targetClassname)) {
                Logger.i(TAG + " Paused Vungle");
                vunglePub.onPause();
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Pause failed", e);
        }
    }

    @Override
    public void onActivityStopped(Activity activity) {

    }

    @Override
    public void onActivitySaveInstanceState(Activity activity, Bundle bundle) {

    }

    @Override
    public void onActivityDestroyed(Activity activity) {
        try {
            if (activity.getClass().getName().equals(targetClassname)) {
                Logger.i(TAG + " Destroyed Vungle");
                vunglePub.removeEventListeners(vungleListener);
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Destroyed failed",e);
        }
    }

    private class VungleListener implements VungleAdEventListener {

        private com.pokkt.sdk.AdConfig config = null;

        public void setConfig(com.pokkt.sdk.AdConfig config) {
            this.config = config;
        }

        @Override
        public void onAdEnd(String placementReferenceId, boolean wasSuccessfulView, boolean wasCallToActionClicked) {
            Logger.i(TAG + " adPlaybackEnded ! " + "wasSuccessfulView :" + wasSuccessfulView + " wasCallToActionClicked: " + wasCallToActionClicked);
            if (wasSuccessfulView) {
                AdManager.getInstance().adCompleted(config, adNetworkInfo);
                if (config.isRewarded) {
                    AdManager.getInstance().adGratified(config, 0, adNetworkInfo);
                }
            } else {
                AdManager.getInstance().adSkipped(config, adNetworkInfo);
            }
            AdManager.getInstance().adClosed(config, adNetworkInfo);
            vungleAdConfig = null;
        }

        @Override
        public void onAdStart(String placementReferenceId) {
            Logger.i(TAG + " AdStart !");
            AdManager.getInstance().adDisplayed(config, adNetworkInfo);
        }

        @Override
        public void onUnableToPlayAd(String placementReferenceId, String reason) {
            Logger.i(TAG + " Failed to play ad for reason: " + reason);
            AdManager.getInstance().adClosed(config, adNetworkInfo);
            vungleAdConfig = null;
        }

        @Override
        public void onAdAvailabilityUpdate(String placementReferenceId, boolean isAdAvailable) {
            Logger.i(TAG + " Ad Available " + isAdAvailable);
            if (!isTimedOut) {
                Logger.i(TAG + " is not timedout" );
                isTimedOut = true;
                if (isAdAvailable) {
                    Logger.i(TAG + " avail true" );
                    if (cachingCallbacks != null) {
                        Logger.i(TAG + " send success callback" );
                        cachingCallbacks.onSuccess(0.0);
                    }
                } else {
                    Logger.i(TAG + " avail false" );
                    if (cachingCallbacks != null) {
                        Logger.i(TAG + " send fail callback" );
                        cachingCallbacks.onFailure("Ad Not Available");
                        vungleAdConfig = null;
                    }
                }
            }
        }
    }
}